#ifndef NCMAPI_H
#define NCMAPI_H

/***************************************************************

  %name:           NCMApi.H %
  %created_by:     perlovsg %
  %date_created:   Fri Oct 06 13:22:42 2000 %

  Description:	Contains API calls needed for NCM clients.

  %version:        9.1.1.1.1 %
  %derived_by:     perlovsg % 
  %date_modified:  Fri Oct 06 13:22:44 2000 %

  (c) 1996-1997, Dialogic Corporation.  All rights reserved.

***************************************************************/

//
#include "NCMTypes.h"
#include <winsvc.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef  _NCMAPI_
#define NCM_API __declspec(dllexport)
#else
#define NCM_API __declspec(dllimport)
#endif

// --------------------------------------------------------------------------------
//  Read
// --------------------------------------------------------------------------------

NCM_API NCMRetCode		NCM_GetAllFamilies( NCMFamily ** ppncmFamily );
NCM_API NCMRetCode		NCM_GetInstalledFamilies( NCMFamily ** ppncmFamily );

NCM_API NCMRetCode		NCM_GetAllDevices( NCMFamily *, NCMDevice ** );
NCM_API NCMRetCode		NCM_GetInstalledDevices( NCMFamily *,
												 NCMDevice ** ppncmDevice );

NCM_API NCMRetCode		NCM_GetProperties( NCMFamily *, NCMDevice *,
										   NCMProperty ** ppncmProp );
NCM_API NCMRetCode		NCM_GetVariables( NCMFamily *, NCMDevice *,
										  NCMProperty *, NCMVariable ** );
NCM_API NCMRetCode		NCM_GetValue( NCMFamily *, NCMDevice *, 
									  NCMProperty *, NCMVariable *,
									  NCMValue ** );
NCM_API NCMRetCode		NCM_GetValueRange( NCMFamily *, NCMDevice *, 
										   NCMProperty *, NCMVariable *,
										   NCMValue ** );

NCM_API NCMRetCode  	NCM_IsEditable(	NCMFamily *, NCMDevice *, 
									    NCMProperty *, NCMVariable *,
										BOOL * );

NCM_API NCMRetCode      NCM_GetErrorMsg( NCMRetCode ncmRcIn, NCMErrorMsg** );

NCM_API NCMRetCode		NCM_IsBoardEnabled( NCMFamily *,
											   NCMDevice *,
											   BOOL * );


// --------------------------------------------------------------------------------
//  Create, Update, Delete
// --------------------------------------------------------------------------------

NCM_API NCMRetCode		NCM_SetValue( NCMFamily *, NCMDevice *, NCMProperty *, NCMVariable *, NCMValue * );

NCM_API NCMRetCode		NCM_AddDevice( NCMFamily *family, NCMDevice *modelName, NCMDevice *uniqueName );
NCM_API NCMRetCode		NCM_DeleteEntry( NCMFamily *, NCMDevice * );

NCM_API NCMRetCode		NCM_EnableBoard( NCMFamily *,
										    NCMDevice *,
											BOOL );

NCM_API NCMRetCode		NCM_DetectBoards( GL_PROG_FUNC*, int * );
////////////////////////// Third Party Configuration, RChen  11/17/99////////////////////////
NCM_API NCMRetCode		NCM_GetCtrlServiceInfo( char* ctrlService, 
																NCM_CTRLSERVICEINFO* ctrlServiceInfo);  

NCM_API NCMRetCode      NCM_GetDcmInfo(char* ncm, char* wndtitle );	 

//Added the function to synchronize software versions. Greg Perlovsky 08-08-00 
NCM_API NCMRetCode		NCM_GetVersionInfo(NCMSysVersion *psysver);
////////////////////////// Third Party Configuration, RChen  11/17/99////////////////////////
// --------------------------------------------------------------------------------
//  Dialogic service
// --------------------------------------------------------------------------------

NCM_API NCMRetCode		NCM_GetDlgSrvStartupMode( NCMDlgSrvStartupMode *);
NCM_API NCMRetCode  	NCM_SetDlgSrvStartupMode( NCMDlgSrvStartupMode );

NCM_API NCMRetCode		NCM_GetDlgSrvState( NCMDlgSrvState * );

//M.Chia 2-3-99 added the function below from J.Xu
NCM_API NCMRetCode		NCM_GetDlgSrvStateEx( SERVICE_STATUS * );

NCM_API NCMRetCode  	NCM_StartDlgSrv();
NCM_API NCMRetCode  	NCM_StopDlgSrv();

// --------------------------------------------------------------------------------
//  Country Specific Parameters (CSP)
// --------------------------------------------------------------------------------

NCM_API NCMRetCode	NCM_GetCspCountries( NCMValue ** ncmCountries );
NCM_API NCMRetCode	NCM_GetCspCountryCode( char *countryName,
											   NCMValue ** ppncmCountryCode 
										 );
NCM_API NCMRetCode	NCM_GetCspCountryName( char *countryCode,
											   NCMValue ** ppncmCountryName 
									     );
NCM_API NCMRetCode	NCM_GetCspFeaturesVariables( char *country,
												 NCMVariable ** ppncmCspFeatures 
											   );
NCM_API NCMRetCode	NCM_GetCspFeaturesValue( char *country, char *features, 
											 NCMVariable *,
											 NCMValue ** ppncmFeatValues
										   );
NCM_API NCMRetCode	NCM_GetCspFeaturesValueRange( char *country, NCMVariable *,
												  NCMValue ** ppncmCspValRange 
												);

// --------------------------------------------------------------------------------
//  Administration
// --------------------------------------------------------------------------------
NCM_API NCMRetCode	NCM_Dealloc( NCMString * pncmStr );

NCM_API NCMRetCode  NCM_GetValueEx(      IN     NCMFamily   *,
                                         IN     NCMDevice   *, 
                                         IN     NCMVariable *, 
                                         OUT    NCMValueEx  ** );

NCM_API NCMRetCode  UNCM_GetValueEx(     IN     NCMFamily   *,
                                         IN     NCMDevice   *, 
                                         IN     NCMVariable *, 
                                         OUT    NCMValueEx  ** );

NCM_API NCMRetCode  NCM_GetValueRangeEx( IN     NCMFamily   *, 
                                         IN     NCMDevice   *, 
                                         IN     NCMVariable *, 
                                         OUT    NCMValueEx  ** );
NCM_API NCMRetCode  NCM_SetValueEx(      IN     NCMFamily   *, 
                                         IN     NCMDevice   *, 
                                         IN     NCMVariable *, 
                                         IN     NCMValueEx  * );

NCM_API NCMRetCode  UNCM_SetValueEx(     IN     NCMFamily   *, 
                                         IN     NCMDevice   *, 
                                         IN     NCMVariable *, 
                                         IN     NCMValueEx  * );

NCM_API NCMRetCode  NCM_DeallocValue(    IN     NCMValueEx  *  );
NCM_API NCMRetCode  NCM_DetectBoardsEx(  IN     NCM_DETECTION_INFO   *,  
                                         INOUT  NCM_DETECTION_RESULT * );

// --------------------------------------------------------------------------------
//  Attributes
// --------------------------------------------------------------------------------
NCM_API NCMRetCode	NCM_GetPropertyAttributes( IN     NCMFamily   *,
											   IN     NCMDevice   *, 
											   IN     NCMProperty *, 
											   OUT    NCMPropertyAttributes * );

NCM_API NCMRetCode	NCM_GetVariableAttributes( IN     NCMFamily   *,
											   IN     NCMDevice   *, 
											   IN     NCMVariable *, 
											   OUT    NCMVariableAttributes * );

//M.Chia 2-3-99 ~* added code for calling getGenericDeviceName() in NCMMgr (J.Xu).
NCM_API NCMRetCode NCM_GetModel(			NCMFamily*,
											NCMDevice*,
											char**);
//end of definition

//Jack Xu 7/23/98 add 2 new API functions calling start and stop boards.

NCM_API NCMRetCode NCM_StartBoard(IN NCMFamily*, IN NCMDevice*);

NCM_API NCMRetCode NCM_StopBoard(IN NCMFamily*, IN NCMDevice*); 

// ----------------------------------------------------------------------------
//  Clock Master Fallback List - Pulsepoint     4/23/99 M.Chia
// ----------------------------------------------------------------------------

NCM_API NCMRetCode	NCM_GetClockMasterFallbackList(	IN		NCMDevice	*, 
													OUT		int			* , 
													OUT	    NCMDevice	** );

NCM_API NCMRetCode	NCM_SetClockMasterFallbackList(	IN		NCMDevice	*,  
													IN	    NCMDevice	* );


NCM_API NCMRetCode	NCM_SetTDMBusValue(				IN		NCMDevice	*, 
													IN		NCMVariable	*, 
													IN		NCMValue	*);

NCM_API NCMRetCode	NCM_SetTDMBusValueEx(				IN		NCMDevice	*, 
													IN		NCMVariable	*, 
													IN		NCMValue	*,
													IN		DWORD  *, 
													OUT     char * );

NCM_API NCMRetCode	NCM_GetTDMBusValue(				IN		NCMDevice	*, 
													IN		NCMVariable	*, 
													IN		NCMValue	** );

////// OpenSwitch ////////////////////// 5/24/99 ///// Lisa Yang ///////
////////////////////////////////////////////////////////////////////////
NCM_API NCMRetCode  NCM_GetDefault( IN NCMVariable*, 
									IN DWORD, 
									INOUT NCMValue** );


NCM_API NCMRetCode  NCM_GetDialogicDir( IN     char *,
									    INOUT  int  *, 
									    OUT    char * );

#ifdef __cplusplus 
}
#endif

#endif