/***********************************************************************
 *        FILE: dcblib.c
 * DESCRIPTION: Cross compatibility Library file for DIALOGIC Windows NT
 *
 *   Copyright (c) 1991-1996 Dialogic Corp. All Rights Reserved
 *
 *   THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *   The copyright notice above does not evidence any actual or
 *   intended publication of such source code.
 **********************************************************************/

#ifndef __CROSS_COMPAT_LIB__
#define __CROSS_COMPAT_LIB__
#endif

#include <windows.h>
#include "srllib.h"
#include "dtilib.h"
#include "msilib.h"
#include "dcblib.h"

//
// function prototypes
//
static int  dcb_int_notimp(int ddd);
static char *dcb_null_notimp(int ddd);

//
// typedefs and defines
//
typedef void (*func_ptr)(void);
struct func_table {
	char *fname;
	func_ptr fptr;
};

#define dcb_setbrdparm_ID           0
#define dcb_getbrdparm_ID           1
#define dcb_estconf_ID              2
#define dcb_delconf_ID              3
#define dcb_addtoconf_ID            4
#define dcb_remfromconf_ID          5
#define dcb_getcde_ID               6
#define dcb_setcde_ID               7
#define dcb_monconf_ID              8
#define dcb_unmonconf_ID            9
#define dcb_getatibits_ID          10 
#define dcb_getdigitmsk_ID         11
#define dcb_open_ID                12
#define dcb_tstcom_ID              13
#define dcb_tstdat_ID              14
#define dcb_close_ID               15     
#define dcb_setdigitmsk_ID         16
#define dcb_dsprescount_ID         17
#define dcb_getcnflist_ID          18
#define dcb_gettalkers_ID          19
#define dcb_getevt_ID              20
#define dcb_evtstatus_ID           21
#define dcb_getatibitsEx_ID		   25    

typedef   int (*ptr_dcb_setbrdparm)(int, unsigned char, void *);        /* Set dcb Board parameter */
typedef   int (*ptr_dcb_getbrdparm)(int, unsigned char, void *);        /* Get dcb Board parameter */
typedef   int (*ptr_dcb_estconf)(int, MS_CDT *,int, int, int *);        /* Estab a conference      */
typedef   int (*ptr_dcb_delconf)(int, int);                             /* Delete conference       */
typedef   int (*ptr_dcb_addtoconf)(int,int, MS_CDT *);                  /* Add a party to conf     */
typedef   int (*ptr_dcb_remfromconf)(int,int, MS_CDT *);                /* Remove a party from conf*/
typedef   int (*ptr_dcb_getcde)(int, int, MS_CDT *);                    /* Get conf. descr. elem   */
typedef   int (*ptr_dcb_setcde)(int,int,MS_CDT *);                      /* Set conf. descr. elem   */
typedef   int (*ptr_dcb_monconf)(int,int, long *);                      /* Monitor conference      */
typedef   int (*ptr_dcb_unmonconf)(int, int);                           /* Unmonitor conference    */
typedef   int (*ptr_dcb_getatibits)(int, unsigned long *);              /* Get ATI bits            */
typedef   int (*ptr_dcb_getdigitmsk)(int, int ,unsigned short *);       /* Get DTMF mask           */
typedef   int (*ptr_dcb_open)(char *, int);                             /* Open DCB/SC device      */
typedef   int (*ptr_dcb_tstcom)(int, unsigned int);                     /* Test communication      */
typedef   int (*ptr_dcb_tstdat)(int, unsigned int);                     /* Test data communication */
typedef   int (*ptr_dcb_close)(int);                                    /* Close DCB/SC device     */   
typedef   int (*ptr_dcb_setdigitmsk)(int, int, unsigned short, int);    /* Set DTMF mask           */
typedef   int (*ptr_dcb_dsprescount)(int, int *);                       /* Get config list         */
typedef   int (*ptr_dcb_getcnflist)(int, int, int *, MS_CDT *);         /* Get DSP resource count  */
typedef   int (*ptr_dcb_gettalkers)(int, int, int *, int *);            /* Get Active talker       */
typedef   int (*ptr_dcb_getevt)(int, EV_EBLK *, int);                   /* Get DCB event           */
typedef   int (*ptr_dcb_evtstatus)(int, int, int *);                    /* Get/Set DCB/SC event status */                 
typedef   int (*ptr_dcb_getatibitsEx)(int, int*, DCB_CT*, void*);       /* Get ATI bits  Extended          */


static struct func_table DcbFunc[] = {
{ "dcb_setbrdparm",              (func_ptr)dcb_int_notimp },
{ "dcb_getbrdparm",              (func_ptr)dcb_int_notimp }, 
{ "dcb_estconf",                 (func_ptr)dcb_int_notimp },    
{ "dcb_delconf",                 (func_ptr)dcb_int_notimp },    
{ "dcb_addtoconf",               (func_ptr)dcb_int_notimp },  
{ "dcb_remfromconf",             (func_ptr)dcb_int_notimp }, 
{ "dcb_getcde",                  (func_ptr)dcb_int_notimp },     
{ "dcb_setcde",                  (func_ptr)dcb_int_notimp },     
{ "dcb_monconf",                 (func_ptr)dcb_int_notimp },    
{ "dcb_unmonconf",               (func_ptr)dcb_int_notimp }, 
{ "dcb_getatibits",              (func_ptr)dcb_int_notimp }, 
{ "dcb_getdigitmsk",             (func_ptr)dcb_int_notimp }, 
{ "dcb_open",                    (func_ptr)dcb_int_notimp },      
{ "dcb_tstcom",                  (func_ptr)dcb_int_notimp }, 
{ "dcb_tstdat",                  (func_ptr)dcb_int_notimp }, 
{ "dcb_close",                   (func_ptr)dcb_int_notimp },          
{ "dcb_setdigitmsk",             (func_ptr)dcb_int_notimp }, 
{ "dcb_dsprescount",             (func_ptr)dcb_int_notimp }, 
{ "dcb_getcnflist",              (func_ptr)dcb_int_notimp },  
{ "dcb_gettalkers",               (func_ptr)dcb_int_notimp }, 
{ "dcb_getevt",                  (func_ptr)dcb_int_notimp },
{ "dcb_evtstatus",               (func_ptr)dcb_int_notimp },
{ "dcb_getatibitsEx",            (func_ptr)dcb_int_notimp } 
};

int dcb_libinit(HINSTANCE LibHandle)
{
	struct func_table *dcbp = DcbFunc;
	func_ptr ProcAddress;
	DWORD NumberofFunctions;

	NumberofFunctions = sizeof(DcbFunc)/sizeof(DcbFunc[0]);

	//
	// The library has been loaded previously. Set up each entry to a default
	// not implemented function and then overwrite the true address if
	// the function is present in the DLL.
	//
	while (NumberofFunctions --) {
		ProcAddress = (func_ptr)GetProcAddress(LibHandle, dcbp->fname);
		if (ProcAddress) {
			dcbp->fptr = ProcAddress;
		}
		dcbp++;
	}

	return 0;
}

int dcb_setbrdparm(int devh, unsigned char param, void *valuep)
{
	ptr_dcb_setbrdparm p;

	p = (ptr_dcb_setbrdparm)*DcbFunc[dcb_setbrdparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

int dcb_getbrdparm(int devh, unsigned char param, void *valuep)
{
	ptr_dcb_getbrdparm p;

	p = (ptr_dcb_getbrdparm)*DcbFunc[dcb_getbrdparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

int dcb_estconf(int devh, MS_CDT *cdtp,int numpty, int confattr, int *confID)
{
	ptr_dcb_estconf p;

	p = (ptr_dcb_estconf)*DcbFunc[dcb_estconf_ID].fptr;
	return (*p)(devh,cdtp, numpty, confattr, confID);
}

int dcb_delconf(int devh, int confID)
{
	ptr_dcb_delconf p;

	p = (ptr_dcb_delconf)*DcbFunc[dcb_delconf_ID].fptr;
	return (*p)(devh,confID);
}

int dcb_addtoconf(int devh,int confID, MS_CDT *cdtp)
{
	ptr_dcb_addtoconf p;

	p = (ptr_dcb_addtoconf)*DcbFunc[dcb_addtoconf_ID].fptr;
	return (*p)(devh, confID, cdtp);
}

int dcb_remfromconf(int devh,int confID, MS_CDT *cdtp)
{
	ptr_dcb_remfromconf p;

	p = (ptr_dcb_remfromconf)*DcbFunc[dcb_remfromconf_ID].fptr;
	return (*p)(devh, confID, cdtp);
}

int dcb_getcde(int devh, int confID, MS_CDT *cdtp)
{
	ptr_dcb_getcde p;

	p = (ptr_dcb_getcde)*DcbFunc[dcb_getcde_ID].fptr;
	return (*p)(devh,confID, cdtp);
}

int dcb_setcde(int devh,int confID,MS_CDT *cdtp )
{
	ptr_dcb_setcde p;

	p = (ptr_dcb_setcde)*DcbFunc[dcb_setcde_ID].fptr;
	return (*p)(devh, confID, cdtp);
}

int dcb_monconf(int devh,int confID, long *lts)
{
	ptr_dcb_monconf p;

	p = (ptr_dcb_monconf)*DcbFunc[dcb_monconf_ID].fptr;
	return (*p)(devh, confID, lts);
}

int dcb_unmonconf(int devh, int confID)
{
	ptr_dcb_unmonconf p;

	p = (ptr_dcb_unmonconf)*DcbFunc[dcb_unmonconf_ID].fptr;
	return (*p)(devh, confID);
}

int dcb_getatibits(int devh, unsigned long *atibits)
{
	ptr_dcb_getatibits p;

	p = (ptr_dcb_getatibits)*DcbFunc[dcb_getatibits_ID].fptr;
	return (*p)(devh, atibits);
}

// @@@ Changed to match header definition - TF Feb.4/02
// int dcb_getatibitsEx(int devh, unsigned int *pCount, DCB_CT *pActiveTalkerIndicators, void* RFU)
int dcb_getatibitsEx(int devh, int *pCount, DCB_CT *pActiveTalkerIndicators, void* RFU)
{
	ptr_dcb_getatibitsEx p;

	p = (ptr_dcb_getatibitsEx)*DcbFunc[dcb_getatibitsEx_ID].fptr;
	return (*p)(devh, pCount, pActiveTalkerIndicators, RFU);
}

int dcb_getdigitmsk(int devh, int confid, unsigned short *bitmaskp)
{
	ptr_dcb_getdigitmsk p;

	p = (ptr_dcb_getdigitmsk)*DcbFunc[dcb_getdigitmsk_ID].fptr;
	return (*p)(devh, confid, bitmaskp);
}

int dcb_open(char *name, int flag)
{
	ptr_dcb_open p;

	p = (ptr_dcb_open)*DcbFunc[dcb_open_ID].fptr;
	return (*p)(name, flag);
}

int dcb_tstcom(int devh, unsigned int tmo)
{
	ptr_dcb_tstcom p;

	p = (ptr_dcb_tstcom)*DcbFunc[dcb_tstcom_ID].fptr;
	return (*p)(devh, tmo);
}

int dcb_tstdat(int devh, unsigned int tmo)
{
	ptr_dcb_tstcom p;

	p = (ptr_dcb_tstdat)*DcbFunc[dcb_tstdat_ID].fptr;
	return (*p)(devh, tmo);
}

int dcb_close(int devh)      
{
	ptr_dcb_close p;

	p = (ptr_dcb_close)*DcbFunc[dcb_close_ID].fptr;
	return (*p)(devh);
}

int dcb_setdigitmsk(int devh, int confid, unsigned short bitmask, int action)
{
	ptr_dcb_setdigitmsk p;

	p = (ptr_dcb_setdigitmsk)*DcbFunc[dcb_setdigitmsk_ID].fptr;
	return (*p)(devh, confid, bitmask, action);
}

int dcb_dsprescount(int devh, int *cntp)
{
	ptr_dcb_dsprescount p;

	p = (ptr_dcb_dsprescount)*DcbFunc[dcb_dsprescount_ID].fptr;
	return (*p)(devh, cntp);
}

int dcb_getcnflist(int devh, int confID, int * numpty, MS_CDT * cdtp)
{
	ptr_dcb_getcnflist p;

	p = (ptr_dcb_getcnflist)*DcbFunc[dcb_getcnflist_ID].fptr;
	return (*p)(devh, confID, numpty, cdtp);
}

int dcb_gettalkers(int devh, int confID, int * numids, int * talkers)
{
	ptr_dcb_gettalkers p;

	p = (ptr_dcb_gettalkers)*DcbFunc[dcb_gettalkers_ID].fptr;
	return (*p)(devh, confID, numids, talkers);
}

int dcb_getevt(int devh, EV_EBLK *eblkp, int tmo)
{
	ptr_dcb_getevt p;

	p = (ptr_dcb_getevt)*DcbFunc[dcb_getevt_ID].fptr;
	return (*p)(devh, eblkp, tmo);
}

int dcb_evtstatus(int event, int action, int * status)
{
	ptr_dcb_evtstatus p;

	p = (ptr_dcb_evtstatus)*DcbFunc[dcb_evtstatus_ID].fptr;
	return (*p)(event, action, status);
}

int dcb_int_notimp(int devh)
{
	dt_set_lasterr(devh, EDT_INVMSG);
	return -1;
}

char *dcb_null_notimp(int devh)
{
	dt_set_lasterr(devh,EDT_INVMSG);
	return (char *)NULL;
}

