/***************************************************************************
 *		Copyright (c) 1996 Dialogic Corp.
 *		All Rights Reserved.
 *
 *		THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *		The copyright notice above does not evidence any actual or
 *		intended publication of such source code.
 ***************************************************************************/

/****************************************************************************
 * FILE:         dllmnti.h
 * ORG:          Dialogic Corporation
 * ORIG-DATE:    28-Aug-96
 * Last Mod.:	 20-Feb-98
 * PURPOSE:
 *    This file contains the defines, data structures, error messages,
 *    and macros used in the MNTI.DLL.
 *
 ***************************************************************************/


#ifndef __DLLMNTI_H_
#define __DLLMNTI_H_

#ifdef MNT_DEBUG
#define MNT_TRACE    mntTrace
#else 
#define MNT_TRACE 
#endif

// Define constants
#define MNT_DEFAULT_TIMEOUT         30
#define MNT_NO_TIMEOUT              -1
#define MNT_MAX_CMD_PAYLOAD_SIZE	   2040
#define MNT_MAX_COMMAND_SIZE        sizeof(QMsg) + MNT_MAX_CMD_PAYLOAD_SIZE
#define MNT_MAX_REPLY_COUNT         10
#define MNT_MAX_REPLY_PAYLOAD_SIZE	   2040
#define MNT_MAX_REPLY_SIZE        sizeof(QMsg) + MNT_MAX_REPLY_PAYLOAD_SIZE
#define MNT_MAX_ASYNC_MSGS          64
#define MNT_MAX_ASYNC_STRMS         64

// Define constant FIRST_AVAILABLE for mntAttachMercStream
#define FIRST_AVAILABLE		0
#define MERC_DEVICE_STATUS_FREE                 0x01
#define MERC_DEVICE_STATUS_INUSE_EXCLUSIVE      0x02
#define MERC_DEVICE_STATUS_INUSE_SHARED         0x06

// Define request mode flags for mntEnumMpath/StreamDevices
#define MNT_GET_FIRST         0x01
#define MNT_GET_NEXT          0x02
#define MNT_FIRST_AVAILABLE   0x04

#define CREATE_DEVICE_STREAM     FILE_DEVICE_MPATH
#define CREATE_DEVICE_MPATH      FILE_DEVICE_STRM

#define MNTI_MAX_TRACE_NAME_SIZE	   256
#define MNTI_MAX_DEVICE_NAME_SIZE   128

// Define MCD device name 
//	   mcd0 - non-shareable, admin version 
//    mcd1 - shareable, DLL version
#define MCD_ADMIN_DEVICE_NAME	   "\\\\.\\dlgcmcd0"
#define MCD_DLL_DEVICE_NAME      "\\\\.\\dlgcmcd1"

// MNTI Global Control Block
typedef struct _MNTI_CONTROL_BLK{
	ULONG	MntiState;
	ULONG	MntiFlags;
} MNTI_CONTROL_BLK, *PMNTI_CONTROL_BLK;

// MNTI States
#define MNTI_STATE_PRE_INIT		0x00000000
#define MNTI_STATE_INITIALIZED	0x00000001

// MNTI Trace Control Block
typedef struct _MNTI_TRACE_BLK{
	ULONG	TraceLevel;
   ULONG TraceDeviceType;
	HANDLE	TraceDeviceHandle;
	CHAR	TraceDeviceName[MNTI_MAX_TRACE_NAME_SIZE];
} MNTI_TRACE_BLK, *PMNTI_TRACE_BLK;

// Define trace levels
#define MNTI_TRACE_LEVEL0	0
#define MNTI_TRACE_LEVEL1	1

// Define I/O completion flags
#define MATCH_ON_SRC_ADDR			  0x01
#define MATCH_ON_DEST_ADDR			  0x02
#define MATCH_ON_TRANSACTION_ID	  0x04
#define MATCH_ON_MSGTYPE			  0x08
#define EMPTY_MSG                  0x40


// Define macros to access MMB 
#define MNT_GET_MMB_MATCH_ON_SRC_ADDR(lpMMB, x)		(*(x) = (((lpMMB)->mmbCtrl).flags.matchSrcAddr))
#define MNT_GET_MMB_MATCH_ON_DEST_ADDR(lpMMB, x)	(*(x) = (((lpMMB)->mmbCtrl).flags.matchDstAddr))
#define MNT_GET_MMB_MATCH_ON_TRANS_ID(lpMMB, x)		(*(x) = (((lpMMB)->mmbCtrl).flags.matchXID))
#define MNT_GET_MMB_MATCH_ON_MSGTYPE(lpMMB, x)		(*(x) = (((lpMMB)->mmbCtrl).flags.matchMsgType))
#define MNT_GET_MMB_EMPTY_MSG(lpMMB, x)		      (*(x) = (((lpMMB)->mmbCtrl).flags.emptyMsg))
#define MNT_GET_MMB_CMD_SIZE(lpMMB, x)				   (*(x) = (((lpMMB)->mmbCtrl).cmdSize))
#define MNT_GET_MMB_ACTUAL_REPLY_COUNT(lpMMB, x)	(*(x) = (((lpMMB)->mmbCtrl).actualReplyCount))
#define MNT_GET_MMB_EXPECTED_REPLY_COUNT(lpMMB, x)	(*(x) = (((lpMMB)->mmbCtrl).expectedReplyCount))
#define MNT_GET_MMB_REPLY_MAX_SIZE(lpMMB, x)		   (*(x) = (((lpMMB)->mmbCtrl).replySize))
#define MNT_GET_MMB_CURRENT_REPLY_OFFSET(lpMMB, x)	(*(x) = (((lpMMB)->mmbCtrl).replyOffset))
#define MNT_GET_MMB_CMD_TIMEOUT(lpMMB, x)			   (*(x) = (((lpMMB)->mmbCtrl).timeout))

#define MNT_SET_MMB_MATCH_ON_SRC_ADDR(lpMMB)	   (((lpMMB)->mmbCtrl).flags.matchSrcAddr = 1)
#define MNT_SET_MMB_MATCH_ON_DEST_ADDR(lpMMB)	(((lpMMB)->mmbCtrl).flags.matchDstAddr = 1)
#define MNT_SET_MMB_MATCH_ON_TRANS_ID(lpMMB)	   (((lpMMB)->mmbCtrl).flags.matchXID     = 1)
#define MNT_SET_MMB_MATCH_ON_MSGTYPE(lpMMB)		(((lpMMB)->mmbCtrl).flags.matchMsgType = 1)
#define MNT_SET_MMB_EMPTY_MSG(lpMMB)		      (((lpMMB)->mmbCtrl).flags.emptyMsg     = 1)
#define MNT_SET_MMB_CMD_SIZE(lpMMB, x)			   (((lpMMB)->mmbCtrl).cmdSize = (USHORT)(x))
#define MNT_SET_MMB_EXPECTED_REPLY_COUNT(lpMMB, x) (((lpMMB)->mmbCtrl).expectedReplyCount = (UCHAR)(x))
#define MNT_SET_MMB_REPLY_MAX_SIZE(lpMMB, x)	   (((lpMMB)->mmbCtrl).replySize = (USHORT)(x))
#define MNT_SET_MMB_CMD_TIMEOUT(lpMMB, x)		   (((lpMMB)->mmbCtrl).timeout = (x))

#define MNT_GET_CMD_QMSG(lpMMB, pMsg)		   (*(pMsg) = (QMsgRef)((ULONG)(lpMMB) + sizeof(MMB_CONTROL)))

#define MNT_GET_REPLY_QMSG(lpMMB, ReplyCount, pMsg) \
{	int _index, _replyCount;	  \
	*(pMsg) = (QMsgRef)((ULONG)(lpMMB) + sizeof(MMB_CONTROL)); \
   _replyCount = lpMMB->mmbCtrl.actualReplyCount; \
   if (ReplyCount > _replyCount) \
     *(pMsg) = NULL; \
   else { \
	  for(_index=0;_index<ReplyCount;_index++) \
      (QMsgRef)(*(ULONG *)pMsg += sizeof(QMsg) + (*(pMsg))->msgsize); \
   } \
}

// Error messages
#define ERROR_MNT_BASE                 0xE0000000
#define ERROR_MNT_MMB_ALLOC_FAILED     ERROR_MNT_BASE+0
#define ERROR_MNT_INVALID_VALUE_TYPE   ERROR_MNT_BASE+1
#define ERROR_MNT_NO_MCD_VERSION_ID    ERROR_MNT_BASE+2
#define ERROR_MNT_NO_TRACE_HANDLE      ERROR_MNT_BASE+3
#define ERROR_MNT_CANTCLOSE  	         ERROR_MNT_BASE+4
#define ERROR_MNT_INVALID_ATTR_KEY     ERROR_MNT_BASE+5
#define ERROR_MNT_NO_BOARDS_BY_ATTR    ERROR_MNT_BASE+6
#define ERROR_MNT_NO_MEM               ERROR_MNT_BASE+7		
#define ERROR_MNT_SYSTEM_ERR           ERROR_MNT_BASE+8		
#define ERROR_MNT_MERCURY_STD_MSG      ERROR_MNT_BASE+9		
#define ERROR_MNT_MERCURY_KRNL         ERROR_MNT_BASE+10		
#define ERROR_MNT_HEAP_FREE_FAILED     ERROR_MNT_BASE+11
#define ERROR_MNT_HEAP_ALLOC_FAILED    ERROR_MNT_BASE+12
#define ERROR_MNT_MMB_INVALID_CMDSIZE  ERROR_MNT_BASE+13

#define ERROR_MNT_BAD_STREAM_ID        ERROR_MNT_BASE+0x40003
#define ERROR_MNT_ALREADY_CLOSED       ERROR_MNT_BASE+0x40004
#define ERROR_MNT_ALREADY_OPEN         ERROR_MNT_BASE+0x40005
#define ERROR_MNT_STRM_NOT_OPEN        ERROR_MNT_BASE+0x40007
#define ERROR_MNT_BAD_CT_CAP           ERROR_MNT_BASE+0x4000B

#ifndef	NO_HOTPLUG_HOTSWAP
#define	ERROR_MNT_HP_INVALID_CALLBACK_ADDR		ERROR_MNT_BASE+0x4000c
#define	ERROR_MNT_HP_INVALID_DRIVER_ID			ERROR_MNT_BASE+0x4000d
#define	ERROR_MNT_HPHS_DIAGS_NOT_SUPPORTED		ERROR_MNT_BASE+0x4000e
#define	ERROR_MNT_HPHS_INVALID_SLOT_EVENT		ERROR_MNT_BASE+0x4000f
#endif // NO_HOTPLUG_HOTSWAP

#endif


