#include <winsock2.h>
#include <math.h>                      // pow

#include <fstream>
using std::ofstream;

#include "..\inet\inet_msg.h"
#include "..\escutil\iv_str.h"         // FindCaseInsensitiveString
#include "brd_generic.h"
#include "..\TFEngine\TFEngine_communication.h"
#include "..\TFServer\TFTCPClient_communication.h"
#include "..\Voice\Voice.h"

BoardEvent::BoardEvent(void)
{
	// Do nothing
}

BoardEvent::~BoardEvent(void)
{
	// Do nothing
}

AudioState::AudioState(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub) : commHub(commHub)
{
   Reset();
}

AudioState::~AudioState(void)
{
   if(IsPlaying())
   {
      StopPlaying();
   }

   if(IsDictating())
   {
      StopDictating();
   }

   if(IsRecording())
   {
      StopRecording();
   }

   if(Streamers() > 0)
   {
      RestartStreaming(false);
      RemoveStreamers();
   }

   if(IsStreaming())
   {
      StopStreaming();
   }
}

bool AudioState::ActivateBargeIn(void)
{
   isBargeInActive = true;
   reactivateBargeIn = true;
   return(IsBargeInActive());
}

bool AudioState::AddStreamer(ACE_INET_Addr destination, TCPSocket* client)
{
   streamClientsMutex.Get();

   if(streamClients.count(destination) > 0)
   {
      RemoveStreamer(destination);
   }
   streamClients[destination].client = client;
   streamClients[destination].socket = new UDPSocket(ACE_INVALID_HANDLE);

   streamClientsMutex.Release();
   return(true);
}

bool AudioState::DeactivateBargeIn(void)
{
   isBargeInActive = false;
   reactivateBargeIn = false;
   return(!IsBargeInActive());
}

const char* AudioState::GetRecordFileName(void)
{
   return(recordFileName.c_str());
}

bool AudioState::IsBargeInPermitted(void)
{
   return(isBargeInPermitted);
}

bool AudioState::IsBargeInActive(void)
{
   return(isBargeInActive);
}

bool AudioState::IsDictating(void)
{
   return(isDictating);
}

bool AudioState::IsInterruptable(void)
{
   return(isInterruptable);
}

bool AudioState::IsPaused(void)
{
   return(isPaused);
}

bool AudioState::IsPlaying(void)
{
   return(isPlaying);
}

bool AudioState::IsRecording(void)
{
   return(isRecording);
}

bool AudioState::IsStreaming(void)
{
   return(isStreaming);
}

bool AudioState::IsStreamingPermitted(void)
{
   return(isStreamingPermitted);
}

void AudioState::Pause(void)
{
   isPaused = true;
}

bool AudioState::PauseStreamer(ACE_INET_Addr destination)
{
   streamClientsMutex.Get();

   // Client not in list
   if(streamClients.count(destination) == 0)
   {
      streamClientsMutex.Release();
      return(false);
   }

   // Close socket
   streamClients[destination].socket->Close();

   streamClientsMutex.Release();

   return(true);
}

void AudioState::PauseStreamers(void)
{
   TFTCPClientCommunicationPacket* message;

   streamClientsMutex.Get();

   // Iterate through list of clients
   for(map<ACE_INET_Addr, StreamerClientInfo>::iterator pos = streamClients.begin();
      pos != streamClients.end(); pos++)
   {
      // Close socket
      PauseStreamer(pos->first);

      // Notify client
      message = new TFTCPClientCommunicationPacket(pos->second.client, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_PAUSE_STREAM);
      commHub->SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, message);
   }

   streamClientsMutex.Release();
}

void AudioState::PermitBargeIn(bool permit)
{
   isBargeInPermitted = permit;
}

SOUND_FORMAT AudioState::PlayFormat(void)
{
   return(playFormat);
}

void AudioState::PermitStreaming(bool permission)
{
   isStreamingPermitted = permission;
}

SOUND_FORMAT AudioState::RecordFormat(void)
{
   return(recordFormat);
}

bool AudioState::RemoveStreamer(ACE_INET_Addr destination)
{
   TFTCPClientCommunicationPacket* message;

   streamClientsMutex.Get();

   // Client not in list
   if(streamClients.count(destination) == 0)
   {
      streamClientsMutex.Release();
      return(false);
   }

   delete streamClients[destination].socket;

   // Notify client
   message = new TFTCPClientCommunicationPacket(streamClients[destination].client, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_STOP_STREAM);

   // Remove client from list
   streamClients.erase(destination);

   streamClientsMutex.Release();

   commHub->SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, message);

   return(true);
}

void AudioState::RemoveStreamers(void)
{
   streamClientsMutex.Get();

   // Iterate through list of clients
   // RemoveStreamer() removes clients from the list so the iterator must be reset
   for(map<ACE_INET_Addr, StreamerClientInfo>::iterator pos = streamClients.begin();
      pos != streamClients.end(); streamClients.begin())
   {
      RemoveStreamer(pos->first);
   }

   streamClientsMutex.Release();
}

bool AudioState::ReactivateBargeIn(void)
{
   return(reactivateBargeIn);
}

void AudioState::ReactivateBargeIn(bool state)
{
   reactivateBargeIn = state;
}

void AudioState::Reset(void)
{
   isBargeInActive = false;
   isBargeInPermitted = false;
   isDictating = false;
   isInterruptable = false;
   isPaused = false;
   isPlaying = false;
   isRecording = false;
   isStreaming = false;
   isStreamingPermitted = false;
   reactivateBargeIn = false;
   restartStreaming = false;
   streamFormat = TF_WAV_8KHZ_16BIT;
}

bool AudioState::RestartStreaming(void)
{
   return(restartStreaming);
}

void AudioState::RestartStreaming(bool state)
{
   restartStreaming = state;
}

void AudioState::SetInterruptable(bool interruptable)
{
   isInterruptable = interruptable;
}

void AudioState::SetUninterruptable(void)
{
   isInterruptable = false;
}

bool AudioState::StartDictating(void)
{
   isDictating = true;
   return(IsDictating());
}

bool AudioState::StartPlaying(SOUND_FORMAT format)
{
   isPlaying = true;
   isPaused = false;
   playFormat = format;

   // To support SpeechTrainer functionality - TF 2005/03/28
   playStartTime.Update();
   playEndTime.Invalidate();
   playedEntireAudio = false;

   return(IsPlaying());
}

bool AudioState::StartRecording(const char* fileName, SOUND_FORMAT format)
{
   if(fileName != NULL)
   {
		recordFileName = fileName;
   }
   else
   {
		recordFileName.clear();
   }
   recordFormat = format;
   isRecording = true;
   isPaused = false;
   return(IsRecording());
}

bool AudioState::StartStreamer(ACE_INET_Addr destination, SOUND_FORMAT format)
{
   TFTCPClientCommunicationPacket* message;

   streamClientsMutex.Get();

   // Client not in list
   if(streamClients.count(destination) == 0)
   {
      streamClientsMutex.Release();
      return(false);
   }

   // Create new socket if needed
   if(streamClients[destination].socket->get_handle() == ACE_INVALID_HANDLE)
   {
      ACE_INET_Addr hostAddress;
      hostAddress.set((unsigned short) Socket::ANY_PORT);
      if(streamClients[destination].socket->open(hostAddress) != Socket::SUCCESS)
      {
         streamClientsMutex.Release();
         return(false);
      }
   }

   // Notify Client
   message = new TFTCPClientCommunicationPacket(streamClients[destination].client, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_START_STREAM);

   streamClientsMutex.Release();

   message->format = format;
   commHub->SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, message);

   return(true);
}

void AudioState::StartStreamers(SOUND_FORMAT format)
{
   streamClientsMutex.Get();

   // Iterate through list of clients
   for(map<ACE_INET_Addr, StreamerClientInfo>::iterator pos = streamClients.begin();
      pos != streamClients.end(); pos++)
   {
      StartStreamer(pos->first, format);
   }

   streamClientsMutex.Release();
}

bool AudioState::StartStreaming(void)
{
   isStreaming = true;
   restartStreaming = false;
   return(IsStreaming());
}

void AudioState::StopDictating(void)
{
   isDictating = false;
}

void AudioState::SetPlayedEntireAudio(void)
{
   playedEntireAudio = true;
}

bool AudioState::GetPlayedEntireAudio(void)
{
   return(playedEntireAudio);
}

ESCTime AudioState::GetPlayStartTime(void)
{
   return(playStartTime);
}

ESCTime AudioState::GetPlayEndTime(void)
{
   return(playEndTime);
}

void AudioState::StopPlaying(void)
{
   isPlaying = false;
   isPaused = false;

   // To support SpeechTrainer functionality - TF 2005/03/28
   playEndTime.Update();
}

void AudioState::StopRecording(void)
{
   isRecording = false;
   isPaused = false;
}

void AudioState::StopStreaming(void)
{
   isStreaming = false;
}

void AudioState::StreamAudio(const char* buffer, int bufferSize)
{
   streamClientsMutex.Get();

   // Iterate through list of clients
   for(map<ACE_INET_Addr, StreamerClientInfo>::iterator pos = streamClients.begin();
      pos != streamClients.end(); pos++)
   {
      if(pos->second.socket->get_handle() != ACE_INVALID_HANDLE)
      {
         pos->second.socket->SendBytes(buffer, bufferSize, pos->first);
      }
   }

   streamClientsMutex.Release();
}

int AudioState::Streamers(void)
{
   int size;

   streamClientsMutex.Get();

   size = streamClients.size();

   streamClientsMutex.Release();

   return(size);
}

SOUND_FORMAT AudioState::StreamFormat(void)
{
   return(streamFormat);
}

void AudioState::UnPause(void)
{
   isPaused = false;
}

BoardGeneric::BoardGeneric(PIVRINFO pIVRInfo)
{
	lineOffHook = false;
   curLine = -1;
   pInfo = pIVRInfo;
   speakAudioExtension = ".wav";
	bBrdInit = false;
   trunkType = (pIVRInfo->iPort > 0) ? TF_TRUNK_TYPE_INVALID : TF_TRUNK_TYPE_NONE;
   ivLog = pIVRInfo->ivLog;
   TTSDirectory = NULL;
   lastLine = -1;		// Once initialized, lastLine will equal 'n'.
   // hWaitTimer is used as the primary timer for WaitForEvent()
   hWaitTimer = CreateWaitableTimer(0, false, 0);
   // hGenericTimer is used by any procedure that wants an additional timer event
   hGenericTimer = CreateWaitableTimer(0, false, 0);
   // hFakeGenericEvent is used to invalidate a wait object so it does not get signaled
   hFakeGenericEvent = CreateWaitableTimer(0, false, 0);
	// numWaitObjects can be changed if a board, such as NMS,  needs more than one
   // handle for board events
   numWaitObjects = 5;
   waitObjects = new HANDLE[numWaitObjects];
   waitObjects[0] = pInfo->messages->GetMessageSignal();
   waitObjects[1] = hWaitTimer;
   waitObjects[2] = hGenericTimer;
	// This handle is for generic events
   waitObjects[3] = hFakeGenericEvent;
	// This handle is for board events
   waitObjects[4] = NULL;
   memset(lines, 0, sizeof(BRD_LINE) * MAX_LINES);
   // Warning: Be careful to delete base class' audioState before overwriting
   // it with the derived class' audioState
   audioState = new AudioState(pIVRInfo->commHub);

   // Set reorder tone to North American standard by default
   reorderTone.frequency1 = 480;
   reorderTone.frequency2 = 620;
   reorderTone.minimumOnTime = 200;
   reorderTone.maximumOnTime = 300;
   reorderTone.minimumOffTime = 200;
   reorderTone.maximumOffTime = 300;
   disableReorderToneDetection = false;   // reorder detection on by default

   // Default to 750 ms
   SREndOfSpeechTimeout = 750;
}

BoardGeneric::~BoardGeneric(void)
{
   if(TTSDirectory)
   {
	   delete[] TTSDirectory;
   }

   CloseHandle(hWaitTimer);
   CloseHandle(hGenericTimer);
   CloseHandle(hFakeGenericEvent);

   delete[] waitObjects;
   numWaitObjects = 0;
   waitObjects = NULL;
   delete audioState;
}

IV_STATUS BoardGeneric::SetParameter(const char*, DWORD)
{
   return(IVR_NOTSUPPORTED);
}

void BoardGeneric::Reset(void)
{
   audioState->Reset();
}

bool BoardGeneric::GetLineOnHook(void) const
{
   return(!lineOffHook);
}

SOUND_FORMAT BoardGeneric::GetPreferredFormat(void) const
{
   return(preferredFormat);
}

const char* BoardGeneric::GetTrunkTypeString(void) const
{
   // Return the current port's trunkType setting
   // (protected property of the BoardGeneric class)
   return(GetTrunkTypeString(trunkType));
}

const char* BoardGeneric::GetTrunkTypeString(TFTrunkType inputType) const
{
   // Get TeleFlow trunk type definition from board
   // Return appropriate string to application
   switch(inputType)
   {
      case TF_TRUNK_TYPE_NONE:
         return("NONE");

      case TF_TRUNK_TYPE_SIMULATOR:
         return("SIMULATED");

      case TF_TRUNK_TYPE_T1:
         return("T1");

      case TF_TRUNK_TYPE_E1:
         return("E1");

      case TF_TRUNK_TYPE_ANALOG:
         return("ANALOG");

      case TF_TRUNK_TYPE_VOIP:
         return("VOIP");

      case TF_TRUNK_TYPE_INVALID:
      default:
         return("");
   }
}
//BoardGeneric::GetTrunkTypeString

IV_STATUS BoardGeneric::WaitWhilePaused(void)
{
   if(pInfo->hResume != NULL)
   {
      return(WaitForSingleObject(pInfo->hResume, INFINITE) == WAIT_OBJECT_0 ? IVR_SUCCESS : IVR_FAIL);
   }
   else
   {
      return(IVR_SUCCESS);
   }
}

IV_STATUS BoardGeneric::ProcessBoardMessage(BoardCommunicationPacket* boardMessage)
{
   ivLog->Print("      ERROR: Board specific messages not processed on this hardware\n");
   delete boardMessage;
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::ProcessSystemEvent(void)
{
   IV_STATUS status = IVR_SUCCESS;

   if(pInfo == NULL)
   {
      return(IVR_FATAL);
   }

   TFEngineCommunicationPacket* message = (TFEngineCommunicationPacket*) pInfo->messages->ReceiveMessage();

   if(message == NULL)
   {
      ivLog->Print("      An event didn't really arrive on mySignal...\n");
      return(IVR_FATAL);
   }

   switch(message->event)
   {
      case TFEngineCommunicationPacket::SRV_EVT_TERMINATE:
      {
         ivLog->Print("Termination request\n");
         pInfo->bTerminate = true;
         status = IVR_SHUTDOWN;
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_SD_NORMAL:
      {
         ivLog->Print("Shutdown normal request\n");
         pInfo->shutdownRequested = true;
         if(pInfo->safeToShutdown)
         {
            ivLog->Print("Shutdown permitted\n");
            status = IVR_SHUTDOWN;
         }
         else
         {
            ivLog->Print("Shutdown not permitted\n");
         }
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_SD_ABORT:
      {
         ivLog->Print("Shutdown abort request\n");
         pInfo->shutdownRequested = true;
         // Application must terminate
         pInfo->safeToShutdown = true;
         status = IVR_SHUTDOWN;
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_RELOAD_NORMAL:
      {
         ivLog->Print("Reload normal request\n");
         pInfo->reloadRequested = true;
         pInfo->shutdownRequested = true;
         if(pInfo->safeToShutdown)
         {
            ivLog->Print("Shutdown permitted\n");
            status = IVR_SHUTDOWN;
         }
         else
         {
            ivLog->Print("Shutdown not permitted\n");
         }
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_RELOAD_ABORT:
      {
         ivLog->Print("Reload abort request\n");
         pInfo->reloadRequested = true;
         pInfo->shutdownRequested = true;
         // Application must terminate
         pInfo->safeToShutdown = true;
      	status = IVR_SHUTDOWN;
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_START_STREAM:
      {
         ivLog->Print("Stream start request received from %s:%d\n",
            message->destination.get_host_addr(), message->destination.get_port_number());
         status = AddStreamer(message->destination, message->client);
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_STOP_STREAM:
      {
         ivLog->Print("Stream stop request received from %s:%d\n",
            message->destination.get_host_addr(), message->destination.get_port_number());
         status = RemoveStreamer(message->destination);
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_PAUSE_STREAM:
      {
         ivLog->Print("Stream pause request received from %s:%d\n",
            message->destination.get_host_addr(), message->destination.get_port_number());
         status = PauseStreamer(message->destination);
         break;
      }

      case TFEngineCommunicationPacket::SRV_EVT_BOARD_EVENT:
      {
         status = ProcessBoardMessage(message->boardMessage);
         break;
      }
   }

   delete message;

   return(status);
}

ESCTime BoardGeneric::GetCallStartTime(void) const
{
   return(callStart);
}

ESCTime BoardGeneric::GetCallEndTime(void) const
{
   return(callEnd);
}

float BoardGeneric::GetCallElapsedTime(void) const
{
   return(callEnd.GetSecondsSince(callStart));
}

float BoardGeneric::GetPlayElapsedTime(void) const
{
   return(audioState->GetPlayEndTime().GetSecondsSince(audioState->GetPlayStartTime()));
}

bool BoardGeneric::GetPlayCompleted(void) const
{
   return(audioState->GetPlayedEntireAudio());
}

WORD BoardGeneric::GetCallReference(void)
{
   return(0);
}

IV_STATUS BoardGeneric::DisableReorderToneDetection(void)
{
   disableReorderToneDetection = true;
   return(StopReorderToneDetection());
}

IV_STATUS BoardGeneric::EnableReorderToneDetection(void)
{
   disableReorderToneDetection = false;
   return(StartReorderToneDetection());
}

IV_STATUS BoardGeneric::SetReorderTone(int frequency1, int frequency2, int minimumOnTime,
   int maximumOnTime, int minimumOffTime, int maximumOffTime)
{
   reorderTone.frequency1 = frequency1;
   reorderTone.frequency2 = frequency2;
   reorderTone.minimumOnTime = minimumOnTime;
   reorderTone.maximumOnTime = maximumOnTime;
   reorderTone.minimumOffTime = minimumOffTime;
   reorderTone.maximumOffTime = maximumOffTime;

   return(EnableReorderToneDetection());
}

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::WaitForEvent                             (private)

   Purpose:     This function blocks, waiting for an event to arrive.
                When an event arrives, it is returned in the provided
                struct.

   Notes:       - Operating system dependant code has been moved into a NMS
                  function.
                - May want to change printing methods to previous methods.

   Modified:    19-Nov-96 (TF) - Support a timeout parameter
                20-Nov-96 (TF) - Changed return values
                17-Aug-97 (Jamie Royer) - Changed to use CTA functions.
                01-Jan-99 (Jamie Royer) - Changed to monitor NMS, TeleFlow,
                   and other events (e.g. MSC events).
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::WaitForEvent(
	BoardEvent* boardEvent,   // Event structure
   long timeout)  // Timeout in ms or FOREVER to wait forever
{
   DWORD ret;
   bool bFound;
   LARGE_INTEGER waitTimeout;

   // NMS may signal a wait object when internal processing is required.
   // After processing the internal event, this function should continue
   // waiting but should not exceed the original wait time.  If the wait
   // function is called again after processing an internal event, the
   // timeout would be reset without reducing the timer by the amount
   // of time elapsed since the first wait.  To solve this problem,
   // I'm using a waitable timer.

   if(timeout != FOREVER)
   {
      waitTimeout.QuadPart = -(LONGLONG)(timeout * 10000);
      SetWaitableTimer(hWaitTimer, &waitTimeout, 0, 0, 0, false);
   }

   bFound = false;
   while(!bFound)
   {
      ret = WaitForMultipleObjects(numWaitObjects, waitObjects, false, INFINITE);

      switch(ret)
      {
         // WaitForMultipleObjects has failed.  This will terminate the Engine.
         // **** May want to add code to terminate the thread completely ****.
         case WAIT_FAILED:
         {
            DWORD errorID = GetLastError();
            char MsgBuf[1024];

            CancelWaitableTimer(hWaitTimer);

            FormatMessage(
               FORMAT_MESSAGE_FROM_SYSTEM,
               0,
               errorID,
               MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
               MsgBuf,
               1024,
               0
            );

            ivLog->Print("      ERROR: WaitForEvent returned: %x: %s", errorID, MsgBuf);
            return(IVR_FATAL);
         }

         // An event has arrived from the Service or Monitor.
         case WAIT_OBJECT_0:
         {
            IV_STATUS status = ProcessSystemEvent();

            if(status != IVR_SUCCESS)
            {
               CancelWaitableTimer(hWaitTimer);
               return(status);
            }
            break;
         }

         // hWaitTimer has been signalled.  Timeout has occurred.
         case WAIT_OBJECT_0 + 1:
         {
            // Used by WaitForEvent for its own internal timeout: how long to wait for an event
            CancelWaitableTimer(hWaitTimer);
            return(IVR_TIMEOUT);
         }

         // hGenericTimer has been signalled.
         case WAIT_OBJECT_0 + 2:
         {
            // Used external to WaitForEvent to set a separate timeout
            CancelWaitableTimer(hWaitTimer);
            return(IVR_GENERIC_TIMEOUT);
         }

         // hGenericEvent has been signalled.
         case WAIT_OBJECT_0 + 3:
         {
            // Used by passive functionality to signal when events in CallTriggerQueue 
            CancelWaitableTimer(hWaitTimer);
            return(IVR_GENERIC_EVENT);
         }

         // A board event has been signalled.
         default:
         {
            // Used by active functionality to signal when events in main (board) event queue
            IV_STATUS status = ProcessBoardEvent(boardEvent, bFound);
            if(status != IVR_SUCCESS)
            {
               RecreateTimer(hWaitTimer);

            	if(status != IVR_HANGUP)
               {
		   			ivLog->Print("      ERROR: Problem with board level event queue\n");
               }

         	   return(status);
            }
         }
      }
   }

   RecreateTimer(hWaitTimer);

   return(IVR_SUCCESS);
}

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::WaitForSpecificEvent                     (private)

   Purpose:     This function loops, calling WaitForEvent and discarding
                the result until the desired event arrives.  That event is
                then returned in the provided structure.  Displaying of
                events is controlled by IvSetReportLevel.

   Returns:     IVR_SUCCESS   Desired event received
                IVR_FATAL     Error occurred

   Questions:   - Should the application wait for the event to terminate
                  before responsing to "SHUTDOWN" events?

   Modified:    20-Nov-96 (Tim Forner) - Changed return values
                17-Aug-97 (Jamie Royer) - Changed to use CTA functions.
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::WaitForSpecificEvent(BoardEvent* desiredEvent, BoardEvent* boardEvent, long timeout)
{
   IV_STATUS  status;

   while((status = WaitForEvent(boardEvent, timeout)) == IVR_SUCCESS)
   {
      if(boardEvent->SameEvent(desiredEvent))
      {
         return(IVR_SUCCESS);
      }
   }

   return(status);
}

void BoardGeneric::SetGenericEvent(HANDLE event)
{
   waitObjects[3] = event;
}

void BoardGeneric::ClearGenericEvent(void)
{
   // All handles must be valid (and not NULL) in an array passed to WaitForMultipleObjects
   // This HANDLE is created to provide a valid handle but is never signalled
   waitObjects[3] = hFakeGenericEvent;
}

void BoardGeneric::SetBoardEvent(HANDLE event)
{
   // Set the wait object handle for the board-level event queue
   waitObjects[4] = event;
}

void BoardGeneric::ClearBoardEvent(void)
{
   // All handles must be valid (and not NULL) in an array passed to WaitForMultipleObjects
   // This HANDLE is created to provide a valid handle but is never signalled
   waitObjects[4] = hFakeGenericEvent;
}

void BoardGeneric::RecreateTimer(HANDLE requestedTimer)
{
   // Reverts one of the Waitable Timer objects to un-signaled state; deletes and recreates
   // the timer so that it will be cancelled even if it is already in the signalled state
   //
   if(requestedTimer == hWaitTimer)
   {
      CancelWaitableTimer(hWaitTimer);
      CloseHandle(hWaitTimer);
      hWaitTimer = CreateWaitableTimer(0, false, 0);
      waitObjects[1] = hWaitTimer;
   }
   else if(requestedTimer == hGenericTimer)
   {
      CancelWaitableTimer(hGenericTimer);
      CloseHandle(hGenericTimer);
      hGenericTimer = CreateWaitableTimer(0, false, 0);
      waitObjects[2] = hGenericTimer;
   }
   else
   {
      ivLog->Print("      ERROR: Object handle 0x%x is not a valid timer\n", requestedTimer);
   }
}

// This needs to be fixed to determine proper .wav format
SOUND_FORMAT BoardGeneric::GetExtensionFormat(const char* fileName)
{
   const char* dot;

   if(fileName != NULL)
   {
      dot = strrchr(fileName, '.');

      if(FindCaseInsensitiveString(dot, ".wav") != std::string::npos)
      {
         return(TF_WAV_8KHZ_16BIT);
      }
      else if(FindCaseInsensitiveString(dot, ".vce") != std::string::npos)
      {
         return(TF_VCE_24);
      }
      else if(FindCaseInsensitiveString(dot, ".vox") != std::string::npos)
      {
         return(TF_VOX);
      }
      else if(FindCaseInsensitiveString(dot, ".gsm") != std::string::npos)
      {
         return(TF_WAV_8KHZ_16BIT_G610);
      }
   }

   return(TF_WAV_8KHZ_16BIT);
}

IV_STATUS BoardGeneric::CallTrigger(const char*, const char*&, char*&, long)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::Switch(int, bool)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::UnSwitch(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::BlockCalls(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::UnblockCalls(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::TransferCall(WORD)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::GetDigit(char& ivNextTT, int newTime)
{
   char* digit;
   IV_STATUS status = GetDigits(1, digit, newTime, "X", "");

   ivNextTT = digit[0];
   delete[] digit;

   return(status);
}

IV_STATUS BoardGeneric::HTTPRequest(HTTPTransaction&, int, int, int, const char*, SOUND_FORMAT, int)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::StartRecordFile(const char*, bool, SOUND_FORMAT, int, int, int, bool, bool)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::StartRecordBuffer(char*, unsigned int, bool, SOUND_FORMAT, int, int, bool, bool)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::Dictate(StepDictate::TTAction*, int, std::streamoff&, int,
   std::streamoff, const char*, StepDictate::START_BEHAVIOUR, bool, SOUND_FORMAT, bool,
   StepDictate::END_BEHAVIOUR, int, const char*, SOUND_FORMAT, int, int)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioStopRecord(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioWaitForRecordStop(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioRecord(const char* filename, bool interruptable,
   SOUND_FORMAT format, int volLevel, int maxRecTime, int maxSilence,
   bool disableBeep, bool automaticGainControl)
{
   IV_STATUS status;

   status = StartRecordFile(filename, interruptable, format, volLevel, maxRecTime, maxSilence, disableBeep, automaticGainControl);

   if(status != IVR_SUCCESS)
   {
      return(status);
   }

   status = AudioWaitForRecordStop();

   return(status);
}

IV_STATUS BoardGeneric::AudioRecordBuffer(char* buffer, unsigned int bufsize,
         bool interruptable, SOUND_FORMAT format, int volLevel,
         int maxSilence, bool disableBeep, bool automaticGainControl)
{
   IV_STATUS status;

   status = StartRecordBuffer(buffer, bufsize, interruptable, format, volLevel, maxSilence, disableBeep, automaticGainControl);

   if(status != IVR_SUCCESS)
   {
      return(status);
   }

   return(AudioWaitForRecordStop());
}

IV_STATUS BoardGeneric::StartPlayFile(const char*, bool, SOUND_FORMAT, int, unsigned int, unsigned int)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::StartPlayBuffer(char*, unsigned, bool, SOUND_FORMAT, int)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioStopPlay(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioWaitForPlayStop(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::FancyPlayFileControl(void)
{
   ivLog->Print("      ERROR: Message Review Mode not supported on this hardware\n");
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioPlay(const char* filename, bool interruptable,
   SOUND_FORMAT format, int volLevel, bool fancy, unsigned int playFromTime,
   unsigned int playForTime)
{
   IV_STATUS status;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   if(interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         // Reset play information, essentially record a zero-length play (for Speech Trainer to use)
         audioState->StartPlaying(audioState->PlayFormat());
         audioState->StopPlaying();
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

   if((status = StartPlayFile(filename, fancy ? false : interruptable, format,
      volLevel, playFromTime, playForTime)) != IVR_SUCCESS)
   {
      return(status);
   }

   if(fancy)
   {
      return(FancyPlayFileControl());
   }
   else
   {
      return(AudioWaitForPlayStop());
   }
}
/*BoardGeneric::AudioPlay*/

IV_STATUS BoardGeneric::AudioPlayBuffer(char* audioBuffer, unsigned bufferSize,
         bool interruptable, SOUND_FORMAT format, int volLevel)
{
   IV_STATUS status;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   if(interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

   if((status = StartPlayBuffer(audioBuffer, bufferSize, interruptable, format, volLevel)) != IVR_SUCCESS)
   {
      return(status);
   }

	return(AudioWaitForPlayStop());
}
/*BoardGeneric::AudioPlayBuffer*/

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::AudioSpeak                                (public)

   Purpose:     Speak all types of data (dates, amounts, digits, etc.)

   Notes:       - ** Code to support language should be changed.  Language
                  should be selected via a parameter, not relying on a
                  directory name.

   Modified:    31-Mar-1997 Tim Forner: Support the 'lang' parameter for
                   specifying directory of prompt files.
                21-Nov-1997 Jamie Royer: Rudimentary support added for
                   English or French support.
                03-Oct-2002 TF: Modified log output to move "speaking %s"
                   to the step level.
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::Say(const char* text, bool interruptable, const char* langDir,
	int volLevel, LanguageModule::SAY_FORMAT format)
{
   string filePath;
   IV_STATUS status = IVR_SUCCESS;
   char* wordList = pInfo->language->BuildWordList(text, format);
   char* wordStart = wordList;
   char* wordEnd;

   if(lineOffHook && interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

   while((wordStart != NULL) && (wordStart[0] != '\0') && (status == IVR_SUCCESS))
   {
      wordEnd = strstr(wordStart, ",");
      if(wordEnd != NULL)
      {
         *wordEnd = '\0';
      }

      if(strlen(wordStart) > 0)
      {
         filePath = langDir;
         if(*(filePath.end() - 1) != '\\')
         {
            filePath += '\\';
         }
         filePath += wordStart;
         filePath += speakAudioExtension;

         ivLog->Print("      Playing: '%s'\n", filePath.c_str());

         status = AudioPlay(filePath.c_str(), interruptable, TF_BY_EXTENSION, volLevel, false);
      }

      wordStart = wordEnd != NULL ? wordEnd + 1 : NULL;
   }

   delete[] wordList;

   return(status);
}

SOUND_FORMAT BoardGeneric::GetSRAudioFormat(void)
{
   return(TF_WAV_8KHZ_16BIT);
}

IV_STATUS BoardGeneric::PermitBargeIn(bool permit)
{
   audioState->PermitBargeIn(permit);
   audioState->ReactivateBargeIn(permit);
   return(permit ? ActivateBargeIn() : DeactivateBargeIn());
}

IV_STATUS BoardGeneric::ActivateBargeIn(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::DeactivateBargeIn(void)
{
   return(IVR_SUCCESS);
}

void BoardGeneric::SetSREndOfSpeechTimeout(unsigned int timeout)
{
   SREndOfSpeechTimeout = timeout;
}

unsigned int BoardGeneric::GetSREndOfSpeechTimeout(void)
{
   return(SREndOfSpeechTimeout);
}

IV_STATUS BoardGeneric::FaxQueueAdd(char*)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::FaxQueueClr(void)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::FaxReceive(char*, char*)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::FaxSend(const char*, int, const char*)
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::AudioCopy(
	char *srcFile,				// Name of source file.
   char *dstFile)          // Name of destination file.
{
   FILE   *fin;
   FILE   *fout;
   char    buf[1024];
   size_t  amtRead;

   fin = fopen (srcFile, "rb");
   if (fin == NULL)
      return IVR_FAIL;

   fout = fopen (dstFile, "wb");
   if (fout == NULL) {
      fclose (fin);
      return IVR_FAIL;
   }

   amtRead = fread (buf, 1, 1024, fin);
   while (!feof (fin)) {
      fwrite (buf, amtRead, 1, fout);
      amtRead = fread (buf, 1, 1024, fin);
   }

   if (amtRead > 0)
      fwrite (buf, amtRead, 1, fout);

   fclose (fout);
   fclose (fin);

   return IVR_SUCCESS;
}

IV_STATUS BoardGeneric::AudioDelete(char* filename)
{
   if(remove(filename))
   {
   	return(IVR_FAIL);
   }
   else
   {
   	return(IVR_SUCCESS);
   }
}

// ****************************************************************************
// **** CONFERENCING REQUIRED FUNCTIONS ***************************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::ConfCreate
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::ConfCreate(DWORD, bool, bool, bool, bool, int*)
{
   return(IVR_NOTSUPPORTED);
}

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::ConfDestroy
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::ConfDestroy(unsigned)
{
   return(IVR_NOTSUPPORTED);
}

// -------------------------------------------------------------------------
// Name:        BoardGeneric::ConfDSP
// -------------------------------------------------------------------------
IV_STATUS BoardGeneric::ConfDSP(unsigned, bool, int)
{
   return(IVR_NOTSUPPORTED);
}

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::ConfJoin
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::ConfJoin(unsigned, bool, int)
{
   return(IVR_NOTSUPPORTED);
}

/* -------------------------------------------------------------------------
   Name:        BoardGeneric::ConfLeave
   ------------------------------------------------------------------------- */
IV_STATUS BoardGeneric::ConfLeave()
{
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::StartStreaming(SOUND_FORMAT)
{
   audioState->RestartStreaming(false);
   RemoveStreamers();
   return(IVR_NOTSUPPORTED);
}

IV_STATUS BoardGeneric::StopStreaming(void)
{
   return(IVR_SUCCESS);
}

IV_STATUS BoardGeneric::AddStreamer(ACE_INET_Addr destination, TCPSocket* client)
{
   TFTCPClientCommunicationPacket* message;

   // Add streamer
   if(!audioState->AddStreamer(destination, client))
   {
      // Notify Client
      message = new TFTCPClientCommunicationPacket(client, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_STOP_STREAM);
      pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, message);
      return(IVR_FAIL);
   }

   // Should be streaming audio
   if(audioState->IsStreamingPermitted())
   {
      // Audio already streaming
      if(audioState->IsStreaming())
      {
         // Start new streamer
         audioState->StartStreamer(destination, audioState->IsRecording() ? audioState->RecordFormat() : audioState->StreamFormat());
      }
      // Start streaming audio
      else
      {
         IV_STATUS status = StartStreaming(audioState->StreamFormat());

         if(status != IVR_SUCCESS)
         {
            // Notify Client
            // May be an non-error status such as IVR_HANGUP so send pause and let
            // other error handling code send a stop if needed
            message = new TFTCPClientCommunicationPacket(client, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_PAUSE_STREAM);
            pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, message);
         }

         // Don't report failure to start streaming
         return(status == IVR_FAIL ? IVR_SUCCESS : status);
      }
   }
   // Should not stream audio
   else
   {
      // Notify Client
      message = new TFTCPClientCommunicationPacket(client, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_PAUSE_STREAM);
      pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, message);
   }

   return(IVR_SUCCESS);
}

IV_STATUS BoardGeneric::RemoveStreamers(void)
{
   audioState->RestartStreaming(false);
   audioState->RemoveStreamers();

   if(audioState->IsStreaming() && !audioState->IsRecording())
   {
      return(StopStreaming());
   }

   return(IVR_SUCCESS);
}

IV_STATUS BoardGeneric::RemoveStreamer(ACE_INET_Addr destination)
{
   IV_STATUS status = IVR_SUCCESS;

   audioState->RemoveStreamer(destination);

   if((audioState->Streamers() == 0) && audioState->IsStreaming() && !audioState->IsRecording())
   {
      status = StopStreaming();
   }

   return(status);
}

IV_STATUS BoardGeneric::PauseStreamers(void)
{
   audioState->RestartStreaming(false);
   audioState->PauseStreamers();

   if(audioState->IsStreaming() && !audioState->IsRecording())
   {
      return(StopStreaming());
   }

   return(IVR_SUCCESS);
}

IV_STATUS BoardGeneric::PauseStreamer(ACE_INET_Addr destination)
{
   audioState->PauseStreamer(destination);

   if((audioState->Streamers() == 0) && audioState->IsStreaming() && !audioState->IsRecording())
   {
      return(StopStreaming());
   }

   return(IVR_SUCCESS);
}

IV_STATUS BoardGeneric::PermitStreaming(bool permission)
{
   IV_STATUS status = IVR_SUCCESS;

   audioState->PermitStreaming(permission);

   if(audioState->IsStreamingPermitted() &&  !audioState->IsStreaming())
   {
      status = StartStreaming(audioState->StreamFormat());
   }
   else if(!audioState->IsStreamingPermitted() && audioState->IsStreaming())
   {
      status = StartStreaming(audioState->StreamFormat());
   }

   return(status);
}

