// Copyright
//
// This software is Copyright (c) 1994 - 2008, Engenic Communications Solutions Corporation
// http://www.enGenic.com
//
//
// License
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
// Executable code that uses provided source code, or derivative works thereof, may
// only be used in conjunction with TeleFlow.
//
// Redistributions of source code must retain the above copyright notice, this list
// of conditions and the following disclaimer.
//
// Redistributions in binary form must reproduce the above copyright notice, this
// list of conditions and the following disclaimer in the documentation, about box
// and/or other materials provided with the distribution.
//
// No personal names or organizations names associated with enGenic may be used to
// endorse or promote products derived from this software without specific prior
// written permission of the specific individual or organization.
//
//
// THIS SOFTWARE IS PROVIDED BY ENGENIC "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
// OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <algorithm>
using std::transform;
#include <ctype.h>   // isalpha, isdigit
#include <stdio.h>   // sscanf
#include <stdlib.h>  // atoi

#include "LanguageModule.h"

WordList::WordList(void)
{
}

WordList& WordList::operator+=(char character)
{
   if(wordList.empty())
   {
      wordList = character;
   }
   else
   {
      wordList += ",";
      wordList += character;
   }
   return(*this);
}

WordList& WordList::operator+=(string word)
{
   if(wordList.empty())
   {
      wordList = word;
   }
   else
   {
      wordList += "," + word;
   }
   return(*this);
}

WordList& WordList::operator+=(WordList list)
{
   if(!wordList.empty() && !list.wordList.empty())
   {
      wordList += ",";
   }

   if(!list.wordList.empty())
   {
      wordList += list.wordList;
   }
   return(*this);
}

WordList& WordList::operator=(char character)
{
   wordList = character;
   return(*this);
}

WordList& WordList::operator=(string word)
{
   wordList = word;
   return(*this);
}

WordList& WordList::operator=(WordList list)
{
   wordList = list.wordList;
   return(*this);
}

const char* WordList::c_str(void)
{
   return(wordList.c_str());
}

string::size_type WordList::length(void)
{
   return(wordList.length());
}

LanguageModule::LanguageModule(void)
{
}

/* -------------------------------------------------------------------------
   Function:    BuildWordList

   Purpose:     Generate a list of words (file names without extensions) to play
                in order to speak a given value.

   ------------------------------------------------------------------------- */
char* LanguageModule::BuildWordList(const char* text, LanguageModule::SAY_FORMAT format)
{
   WordList wordList;
   char* returnString;

   // If the string is empty, we're done.
   if((text == NULL) || (text[0] == '\0'))
   {
      returnString = new char[1];
      returnString[0] = '\0';
      return(returnString);
   }

   switch(format)
   {
      case SAY_ALPHA:
      {
         wordList = AlphaNumericWordList(text);
         break;
      }

      case SAY_DIGITS:
      {
         wordList = DigitsWordList(text);
         break;
      }

      case SAY_DATE:
      case SAY_YEARMONTH:
      case SAY_MONTHDAY:
      {
         wordList = DateWordList(text, format);
         break;
      }

      case SAY_12HOUR:
      case SAY_24HOUR:
      {
         wordList = TimeWordList(text, format);
         break;
      }

      case SAY_NUMBER:
      {
         wordList = RealNumberWordList(text);
         break;
      }

      case SAY_MONEY:
      {
         wordList = MoneyWordList(text);
         break;
      }
   }

   returnString = new char[wordList.length() + 1];
   strcpy(returnString, wordList.c_str());

   return(returnString);
}

WordList LanguageModule::AlphaNumericWordList(string text)
{
   WordList wordList;

   #ifdef _MSC_VER
      transform(text.begin(), text.end(), text.begin(), toupper);
   #else
      transform(text.begin(), text.end(), text.begin(), std::toupper);
   #endif

   for(string::iterator character = text.begin(); character != text.end(); character++)
   {
      if(isdigit(*character))
      {
         wordList += wordTable[WB_CARDINAL + *character - '0'];
      }
      else if(((*character >= ' ') && (*character <= '@'))
         || ((*character >= '[') && (*character <= '\''))
         || ((*character >= '{') && (*character <= ''))
         || (*character == '') || (*character == ''))
      {
         if(wordTable.count(WB_ASCII + *character) != 0)
         {
            wordList += wordTable[WB_ASCII + *character];
         }
         else
         {
            wordList += "pause";
         }
      }
      // Make upper case
      else if((((*character >= '') && (*character <= '')) || ((*character >= '') && (*character <= '')))
            && (wordTable.count(WB_ASCII + *character - 32) != 0))
      {
         wordList += char (*character - '' + '');
      }
      else
      {
         wordList += *character;
      }
   }

   return(wordList);
}

WordList LanguageModule::DigitsWordList(string digits)
{
   WordList wordList;

   for(string::iterator digit = digits.begin(); digit != digits.end(); digit++)
   {
      if(isdigit(*digit))
      {
         wordList += wordTable[WB_CARDINAL + *digit - '0'];
      }
      else
      {
         wordList += "pause";
      }
   }

   return(wordList);
}

string LanguageModule::Integer(string realNumber)
{
   string::size_type decimalPosition = realNumber.rfind('.');

   if(decimalPosition == string::npos)
   {
      return(realNumber);
   }

   return(realNumber.substr(0, decimalPosition));
}

string LanguageModule::Fraction(string realNumber)
{
   string::size_type decimalPosition = realNumber.rfind('.');

   if(decimalPosition == string::npos)
   {
      return("");
   }

   return(realNumber.substr(decimalPosition + 1));
}

WordList LanguageModule::IntegerWordList(string integer, bool ordinal)
{
   int numberOfCommas = 0;
   // Group (Millions = 2, Thousands = 1, Ones = 0)
   int currentGroupNumber;
   // Position within group (hundreds = 2, tens = 1, ones = 0)
   int initialGroupIndex;
   string currentGroup;
   int currentGroupValue;
   string::iterator digit = integer.begin();
   WordList wordList;

   for(string::size_type index = integer.find(','); index != string::npos; index = integer.find(',', index + 1))
   {
      numberOfCommas++;
   }

   currentGroupNumber = (integer.length() - numberOfCommas - 1)/3;
   initialGroupIndex = (integer.length() - numberOfCommas - 1)%3;

   // Not capable of speaking numbers larger than 999 trillion.
   if(currentGroupNumber > 4)
   {
      return(wordList);
   }

   if(atoi(integer.c_str()) == 0)
   {
      return(AlphaNumericWordList("0"));
   }

   for(;currentGroupNumber >= 0; currentGroupNumber--)
   {
      currentGroup = "";

      for(int counter = 0; (digit != integer.end()) && (counter <= initialGroupIndex); counter++)
      {
         currentGroup += *digit;

         if((*digit == '+') || (*digit == '-'))
         {
            counter--;
         }

         digit++;
      }

      currentGroupValue = atoi(currentGroup.c_str());
      if((currentGroupValue != 0) || (currentGroupNumber == 0))
      {
         wordList += NumberGroupWordList(currentGroup, currentGroupNumber == 0 ? ordinal : false);
         wordList += NumberGroupWordList(currentGroupNumber, currentGroupValue != 1);
      }
      initialGroupIndex = 2;
   }

   return(wordList);
}
