/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>      // BCB6 upgrade - TF Mar.28/02
#include "iv_math2.h"
#include "..\escutil\esctime.h"
#include "..\escutil\VarController.h"

char *weekDayName[] = { "Sunday", "Monday", "Tuesday", "Wednesday",
                        "Thursday", "Friday", "Saturday" };
char *monthName[]   = { "January", "February", "March", "April", "May",
                        "June", "July", "August", "September", "October",
                        "November", "December" };


/* JSH
static void PrintMathFn(Log* ivLog, const char* varA, const char* varB,
   const char* varC, const char* varD, MATH_FN fn);
*/

// static void PrintCmpFn(Log* ivLog, const char* varA, const char* varB, CMP_FN fn);

static IV_STATUS VarMathWeekday (
   char  *varB,
   char *&value);

//static const char * GetMathFnText (MATH_FN fn, bool verbose = false);
//static const char * GetCmpFnText (CMP_FN fn, bool verbose = false);

// **** MATH FUNCTION PROTOTYPES **********************************************
static bool VarValidDate (const char *dt);
static bool VarValidDateTime (const char *dt);

static IV_STATUS VarMathDate(PIVRINFO pInfo, const char* varB, const char* varC, MATH_FN fn, char*& value);

static IV_STATUS VarMathTime(const char* varB, const char* varC, MATH_FN fn, char*& value);

static IV_STATUS VarMathDateTime(PIVRINFO pInfo, const char* varB, const char* varC, MATH_FN fn, char*& value);

static IV_STATUS VarMathTimeDiff(const char* varB, const char* varC, char*& value);

static IV_STATUS VarMathDateDiff(const char* varB, const char* varC, char*& value);

static IV_STATUS VarMathDateTimeDiff(const char* varB, const char* varC, char*& value);

static IV_STATUS VarMathInt(const char* varB, const char* varC, MATH_FN fn, char*& value);

static int GetPrecision (const char *value);

static IV_STATUS VarMathFloat(const char* varB, const char* varC, MATH_FN fn, char*& value);

// **** COMPARE FUNCTION PROTOTYPES *******************************************
static IV_STATUS VarCmpInt(const char* varA, const char* varB, CMP_FN fn, bool& result);

static IV_STATUS VarCmpFloat(const char* varA, const char* varB, CMP_FN fn, bool& result);
static IV_STATUS VarCmpDate(const char* varA, const char* varB, CMP_FN fn, bool& result);

// **** VALIDATION FUNCTION PROTOTYPES ****************************************

static double CalcMonthlyPayment (double annualIntRate, double mortgageAmt);
static double CalcMaxMortgageAmount (double annualIntRate, double grossIncome);

/* -------------------------------------------------------------------------
   Name:        VarTypeString
   Note:        - JSH 2003-5-14 changed these to all lower case
   ------------------------------------------------------------------------- */
const char* VarTypeString(VAR_TYPE varType)
{
   switch(varType)
   {
      case VAR_DATE:    return("date");
      case VAR_TIME:    return("time");
      case VAR_DATETIME: return("datetime");
      case VAR_INTEGER: return("integer");
      case VAR_FLOAT:   return("float");
      case VAR_STRING:
      default:          return("string");
   }
}
/*VarTypeString*/

/* -------------------------------------------------------------------------
   Name:        DetermineType

   Purpose:     Determine the type of a user or system variable.  If a
                string is not a date, time, integer or float then it is
                labeled a string.

                Date:    9999/99/99      (10 char)
                Time:    99:99           (5 char)
                Time:    99:99:99        (8 char)
                Integer: 99...           (1+ char - no decimal)
                Float:   99.... . 99...  (1+.1+)

   Notes:       - This is not in the class so it can be used for strings
                  that are not variable.

   Modified:    ??-???-???? Jamie Royer: Created.
   ------------------------------------------------------------------------- */
VAR_TYPE DetermineType (const char *s)
{
   int      offset;
   int      lSide = 0;     // Count of digits on the left  side of the decimal.
   int      rSide = 0;     // Count of digits on the right side of the decimal.
   bool     point = false; // Indicates if decimal point found.
   VAR_TYPE type = VAR_STRING;

   if (!s || !*s)
      return type;

   // Check to see if this is a date variable (YYYY/MM/DD) or (YYYY-MM-DD)
   if (isdigit (s[0]) && isdigit (s[1]) && isdigit (s[2]) &&
       isdigit (s[3]) && (s[4] == '/' || s[4] == '-') && isdigit (s[5]) &&
       isdigit (s[6]) && (s[7] == '/' || s[7] == '-') && isdigit (s[8]) &&
       isdigit (s[9]) )
   {
      if(s[10] == '\0')
      {
         type = VAR_DATE;
      }
      // Check to see if this is a "datetime" variable (YYYY/MM/DD HH:MM:SS)
      else if (isdigit (s[11]) && isdigit (s[12]) && s[13] == ':' &&
              isdigit (s[14]) && isdigit (s[15]) && s[16] == ':' &&
              isdigit (s[17]) && isdigit (s[18]) )
      {
         if(s[19] == '\0')
         {
            type = VAR_DATETIME;
         }
         else if(s[19] == '.' && isdigit(s[20]))
         {
            int x = 21;
            while (isdigit(s[x]) /*|| s[x] != '\0'*/)
            {
               x++;
            }
            if (s[x] == '\0')
            {
               type = VAR_DATETIME;
            }
         }
      }
   // Check to see if this is a time variable (HH:MM).
   } else if (isdigit (s[0]) && isdigit (s[1]) && s[2] == ':' &&
              isdigit (s[3]) && isdigit (s[4]) && s[5] == '\0')
   {
      type = VAR_TIME;

   // Check to see if this is a time variable (HH:MM:SS).
   } else if (isdigit (s[0]) && isdigit (s[1]) && s[2] == ':' &&
              isdigit (s[3]) && isdigit (s[4]) && s[5] == ':' &&
              isdigit (s[6]) && isdigit (s[7]) && s[8] == '\0')
   {
      type = VAR_TIME;

   // Check to see if this is an integer (999) or a float (99.99) variable.
   } else if (s[0] == '-' || s[0] == '+' || s[0] == '.' || isdigit (s[0])) {
      offset = 0;
      if (s[0] == '+' || s[0] == '-' )
         ++offset;

      while (s[offset]) {
         if (s[offset] == '.') {
            if (point)
               return VAR_STRING;         // Numbers can only have one decimal.
            point = true;

         } else if (isdigit (s[offset]))
            point ? ++rSide : ++lSide;

         else
            return VAR_STRING;  // Found something other than digit or decimal.
         ++offset;
      }

      if (/*lSide &&*/ point && rSide) // [+|-][9[..]].9[9[..]] = VAR_FLOAT
         type = VAR_FLOAT;
      else if (lSide && !point)        // [+|-][9[..]] = VAR_INTEGER
         type = VAR_INTEGER;
      else                             // [+|-][9[..]]. = VAR_STRING
         type = VAR_STRING;
   } else
      type = VAR_STRING;

   return type;
}
/*DetermineType*/


/*****************************************************************************
 **** Variable Math Functions ************************************************
 *****************************************************************************/

/* -------------------------------------------------------------------------
   Name:        VarMath
   ------------------------------------------------------------------------- */
IV_STATUS VarMath(PIVRINFO pInfo, char*& value, const char* varB, const char* varC, const char* varD, MATH_FN fn)
{
   IV_STATUS status = IVR_SUCCESS;
   value = NULL;
   
   // ------------------------------------------------- POW
   // 2003-05-22 JSH: send all power operations off to VarMathFloat
   if (fn == FN_POW)
   {
      status = VarMathFloat (varB, varC, fn, value);
   }
   // ------------------------------------------------- CAT
   else if(fn == FN_CAT)
   {
      value = new char [strlen (varB) + strlen (varC) + 1];
      strcpy (value, varB);
      strcat (value, varC);
   }
   // ------------------------------------------ LEN
   else if(fn == FN_LEN)
   {
      value = new char [10];
      sprintf (value, "%d", strlen (varB));
   }
   // ------------------------------------ SUBSTR
   else if (fn == FN_SUBSTR)
   {
      // A = substr (B, C, D)
      // Display contents of variable D.
      if (pInfo->ivLog->GetDebug ('v'))
      {
         pInfo->ivLog->Print("         (%s): %s\n",
                	VarTypeString(DetermineType(varD)), varD);
      }

		// If pszVarB is empty set it's length to 0
      int intB = varB ? strlen(varB) : 0;
		// If pszVarC is empty set it to the first character (1)
      int intC = varC ? atoi(varC) : 1;
		// If pszVarD is empty set it to the whole string
      int intD = varD ? atoi(varD) : intB;

      // Starting position (varC) must at least 1.
      if(intC < 1)
      {
         pInfo->ivLog->Print ("      ERROR: Invalid start position in string copy\n"
                       "         Start position value '%d' in string copy must be at least 1\n", intC);
      	return(IVR_FATAL);
      }

      // Number of characters to copy (varD) must at least 1.
      if(intD < 0)
      {
         pInfo->ivLog->Print ("      ERROR: Invalid number of characters to copy\n"
                       "         Number of characters value '%d' must be at least 0\n", intD);
      	return(IVR_FATAL);
      }

      // Starting position (varC) must be less then the length of varB.
      if (intC <= intB)
      {
         value = new char[intD + 1];
         strncpy(value, varB + intC - 1, intD);
         value[intD] = 0;
      }
      else
      {
         value = new char[1];
         value[0] = '\0';
      }
   // ------------------------------------------------------------ INTEGER MATH
   } else if (DetermineType (varB) == VAR_INTEGER &&
              DetermineType (varC) == VAR_INTEGER)
   {
      status = VarMathInt (varB, varC, fn, value);
   // -------------------------------------------------------------- FLOAT MATH
   } else if (
      (DetermineType (varB) == VAR_FLOAT ||
       DetermineType (varB) == VAR_INTEGER) &&
      (DetermineType (varC) == VAR_FLOAT ||
       DetermineType (varC) == VAR_INTEGER))
   {
      status = VarMathFloat (varB, varC, fn, value);
   // --------------------------------------------------------------- DATE MATH
   } else if (
      (DetermineType (varB) == VAR_INTEGER &&
       DetermineType (varC) == VAR_DATE) ||
      (DetermineType (varC) == VAR_INTEGER &&
       DetermineType (varB) == VAR_DATE))
   {
      status = VarMathDate (pInfo, varB, varC, fn, value);
   // --------------------------------------------------------------- TIME MATH
   } else if (
      (DetermineType (varB) == VAR_INTEGER &&
       DetermineType (varC) == VAR_TIME) ||
      (DetermineType (varC) == VAR_INTEGER &&
       DetermineType (varB) == VAR_TIME))
   {
      status = VarMathTime (varB, varC, fn, value);
   // ----------------------------------------------------------- DATETIME MATH
   } else if (
      (DetermineType (varB) == VAR_INTEGER &&
       DetermineType (varC) == VAR_DATETIME) ||
      (DetermineType (varC) == VAR_INTEGER &&
       DetermineType (varB) == VAR_DATETIME))
   {
      if (fn == FN_ADD || fn == FN_SUB)
      {
         status = VarMathDateTime (pInfo, varB, varC, fn, value);
      }
      else
      {
         pInfo->ivLog->Print ("      ERROR: Invalid function for operaton on 'datetime' type\n");
         return IVR_FATAL;
      }
   // --------------------------------------------------------- TIME DIFFERENCE
   } else if (fn == FN_SUB && DetermineType (varB) == VAR_TIME &&
              DetermineType (varC) == VAR_TIME)
   {
      status = VarMathTimeDiff (varB, varC, value);
   // --------------------------------------------------------- DATE DIFFERENCE
   } else if (fn == FN_SUB && DetermineType (varB) == VAR_DATE &&
              DetermineType (varC) == VAR_DATE)
   {
      status = VarMathDateDiff (varB, varC, value);
   // ----------------------------------------------------- DATETIME DIFFERENCE
   } else if (fn == FN_SUB && DetermineType (varB) == VAR_DATETIME &&
              DetermineType (varC) == VAR_DATETIME)
   {
      status = VarMathDateTimeDiff (varB, varC, value);
   // ------------------------------------------------------------------- ERROR
   } else {
      pInfo->ivLog->Print ("      ERROR: Unhandled operation in VarMath\n");
      pInfo->ivLog->Print ("      HINT: Ensure variables in operation are of the correct types\n");
      return IVR_FATAL;
   }

   return status;
}
/*VarMath*/

/* -------------------------------------------------------------------------
   Name:        VarDateInfo

   Notes:       - varA = to_date(varB, fn)
                  if bUpperCase then to_upper(varA)
   ------------------------------------------------------------------------- */
IV_STATUS VarDateInfo(PIVRINFO pInfo, const char* varA, const char* varB, DATEINFO_FN fn, bool bUpperCase)
{
   int        year;
   int        month;
   int        day;
   struct tm  t;
   char       temp[20];

   if(!VarValidDate(varB))
   {
      pInfo->ivLog->Print("      ERROR: '%s' not a valid date\n", varB);
      return(IVR_FATAL);
   }

   // Date = YYYY/MM/DD
   sscanf(varB    , "%d", &year);
   sscanf(varB + 5, "%d", &month);
   sscanf(varB + 8, "%d", &day);

   if (year < 1970 || (year > 2038 && month > 1 && day > 19))
   {
      pInfo->ivLog->Print("      ERROR: Date '%s' out of range (1970 - 2038)\n", varB);
      return(IVR_FATAL);
   }

   memset (&t, 0, sizeof(struct tm));
   t.tm_year  = year - 1900;
   t.tm_mon   = month - 1;
   t.tm_mday  = day;
   t.tm_isdst = -1;

   if (mktime (&t) == -1)
   {
      pInfo->ivLog->Print("      ERROR: Failed to convert date '%s'\n", varB);
      return(IVR_FATAL);
   }

   switch (fn)
   {
   case FN_WEEKDAY_SHORT:
      sprintf (temp, "%.3s", weekDayName[t.tm_wday]);
      break;
   case FN_WEEKDAY_LONG:
      sprintf (temp, "%s", weekDayName[t.tm_wday]);
      break;
   case FN_MONTH_SHORT:
      sprintf (temp, "%.3s", monthName[t.tm_mon]);
      break;
   case FN_MONTH_LONG:
      sprintf (temp, "%s", monthName[t.tm_mon]);
      break;
   default:
      *temp = 0;
   }
   if (bUpperCase) {
      for (int offset = 0; temp[offset]; offset++)
         temp[offset] = (char)toupper(temp[offset]);
   }
   return(pInfo->variables->Set(varA, temp) ? IVR_SUCCESS : IVR_FATAL);
}
/*VarDateInfo*/

/* -------------------------------------------------------------------------
   Name:        VarMathDate

   Purpose:     Adds or subtracts days to/from a TeleFlow date variable or
                value.  One of the variables must be a date and the other
                must be an integer.
   ------------------------------------------------------------------------- */
static IV_STATUS VarMathDate(PIVRINFO pInfo, const char* varB, const char* varC, MATH_FN fn, char*& value)
{
   int        year;
   int        month;
   int        day;
   int        days;
                        //  J   F   M   A   M   J   J   A   S   O   N   D
   const int  mthDays[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
   char      *myDate;
   char       sepChar;

   if (DetermineType (varB) != VAR_DATE && DetermineType (varC) != VAR_DATE)
	{
	   return(IVR_FATAL);
   }

   // Get date separator character from Variable Controller
   sepChar = pInfo->variables->Get("@DATE_SEPARATOR")[0]; // just the first character
   if(sepChar != '-')      // only '-' and '/' are valid
      sepChar = '/';       // default is '/'

   myDate = new char [11];
   if (DetermineType (varB) == VAR_DATE) {
      strcpy (myDate, varB);
      sscanf (varC, "%d", &days);
   } else {
      strcpy (myDate, varC);
      sscanf (varB, "%d", &days);
   }
   myDate[4] = '\0';
   myDate[7] = '\0';
   sscanf (myDate    , "%d", &year);
   sscanf (myDate + 5, "%d", &month);
   sscanf (myDate + 8, "%d", &day);
   delete [] myDate;
// myDate = 0;

   switch (fn) {
      case FN_ADD:
         while (days > 0) {
            ++day;
            if (day > mthDays[month - 1]) {
               //
               // - Years ending in 00 as in 1900, 2000 are not leap years
               //   if they cannot be divided by 400.
               // - Years not ending in 00 are leap years if they can be
               //   divided by 4.
               //
               if (
                  month == 2 && (                                   // February
                     (!(year % 100) && !(year % 400)) ||             // Century
                     ((year % 100) && !(year % 4)))              // Non-century
               ) {
                  if (day == 30)                                   // leap year
                     day = 1;
               } else                         // not Feb and/or not a leap year
                  day = 1;

               if (day == 1) {
                  ++month;
                  if (month > 12) {
                     month = 1;
                     ++year;
                  }
               }
            }
            --days;
         }
         break;

      case FN_SUB:
         while (days > 0) {
            --day;
            if (day < 1) {
               --month;
               if (month == 0) {
                  month = 12;
                  --year;
               }

               if (
                  month == 2 && (                                   // February
                     (!(year % 100) && !(year % 400)) ||             // Century
                     ((year % 100) && !(year % 4)))              // Non-century
               )
                  day = 29;
               else
                  day = mthDays[month - 1];
            }
            --days;
         }
         break;

      default:
		{
	   	return(IVR_FATAL);
	   }
   } // endswitch

   delete [] value;
   value = new char [11];
   sprintf (value, "%.4d%c%.2d%c%.2d", year, sepChar, month, sepChar, day);

   return IVR_SUCCESS;
}
/*VarMathDate*/

/* -------------------------------------------------------------------------
   Name:        VarMathTime

   Purpose:     Adds or subtracts seconds to/from a TeleFlow time variable
                or value.  One of the variables must be a time and the other
                must be an integer.
   ------------------------------------------------------------------------- */
static IV_STATUS VarMathTime(const char* varB, const char* varC, MATH_FN fn, char*& value)
{
   int    hour;
   int    minute;
   int    second;
   int    chg_seconds;
   char  *myTime;
   bool   haveSeconds = false;

   if (DetermineType (varB) != VAR_TIME && DetermineType (varC) != VAR_TIME)
   {
      return(IVR_FATAL);
   }

   myTime = new char [9];
   if (DetermineType (varB) == VAR_TIME) {
      strcpy (myTime, varB);
      sscanf (varC, "%d", &chg_seconds);
   } else {
      strcpy (myTime, varC);
      sscanf (varB, "%d", &chg_seconds);
   }
   if (strlen (myTime) == 8)
      haveSeconds = true;
   myTime[2] = 0;
   myTime[5] = 0;
   sscanf (myTime    , "%d", &hour);
   sscanf (myTime + 3, "%d", &minute);
   if (haveSeconds)
      sscanf (myTime + 6, "%d", &second);
   else
      second = 0;
   delete [] myTime;

   switch (fn)
   {
      case FN_ADD:
         hour = hour + (chg_seconds / 3600);
         chg_seconds = chg_seconds % 3600;

         minute = minute + (chg_seconds / 60);
         chg_seconds = chg_seconds % 60;

         second += chg_seconds;
         break;

      case FN_SUB:
         hour = hour - (chg_seconds / 3600);
         chg_seconds = chg_seconds % 3600;

         minute = minute - (chg_seconds / 60);
         chg_seconds = chg_seconds % 60;

         second -= chg_seconds;
         break;

      default:
         return(IVR_FATAL);
   }

   while(second > 59)
   {
      second -= 60;
      ++minute;
   }
   while(second < 0)
   {
      second += 60;
      --minute;
   }

   while(minute > 59)
   {
      minute -= 60;
      ++hour;
   }
   while(minute < 0)
   {
      minute += 60;
      --hour;
   }

   while(hour > 23)
   {
      hour -= 24;
   }
   while(hour < 0)
   {
      hour += 24;
   }

   delete [] value;
   value = new char [9];
   sprintf (value, "%.2d:%.2d:%.2d", hour, minute, second);
   return IVR_SUCCESS;
}
/*VarMathTime*/

// -------------------------------------------------------------------------
// Name:        VarMathDateTime
//
// Purpose:     Adds or subtracts seconds to/from a TeleFlow 'datetime'
//              variable or value.  One of the parameters must be a datetime
//              and the other must be an integer.
// -------------------------------------------------------------------------
static IV_STATUS VarMathDateTime(PIVRINFO pInfo, const char* varB, const char* varC, MATH_FN fn, char*& value)
{
   ESCTime tempTime;
   int     chg_seconds;
   bool    bSuccess;
   string  strTemp;
   char    sepChar[2];  // ESCTime object expects a NULL-terminated string

   // Only add and subtract are valid functions
   if(fn != FN_ADD && fn != FN_SUB)
   {
      return(IVR_FATAL);
   }

   // Get date separator character from Variable Controller
   sepChar[0] = pInfo->variables->Get("@DATE_SEPARATOR")[0]; // just the first character
   if(sepChar[0] != '-')      // only '-' and '/' are valid
      sepChar[0] = '/';       // default is '/'
   sepChar[1] = '\0';         // ESCTime object expects a NULL-terminated string

   // Determine which is the date and copy to 'ESCTime' object
   if(DetermineType(varB) == VAR_DATETIME && DetermineType(varC) == VAR_INTEGER)
   {
      tempTime.Assign(varB);
      sscanf(varC, "%d", &chg_seconds);
   }
   else if(DetermineType(varC) == VAR_DATETIME && DetermineType(varB) == VAR_INTEGER)
   {
      tempTime.Assign(varC);
      sscanf(varB, "%d", &chg_seconds);
   }
   else
   {
      return(IVR_FATAL);
   }

   if(chg_seconds < 0)
   {
      // Account for negative seconds
      fn = (fn == FN_ADD) ? FN_SUB : FN_ADD;
      chg_seconds = abs(chg_seconds);
   }

   // Add or Subtract the seconds
   if(fn == FN_ADD)
   {
      bSuccess = tempTime.AddSeconds(chg_seconds);
   }
   else
   {
      bSuccess = tempTime.SubtractSeconds(chg_seconds);
   }

   if(bSuccess != true)
   {
      return(IVR_FAIL);
   }

   // convert back to datetime
   strTemp = (string) tempTime.GetDateTime(sepChar);

   // update the 'value' parameter
   value = new char [strTemp.length()+1];
   strcpy (value, strTemp.c_str());

   return(IVR_SUCCESS);
}
//VarMathDateTime

/* -------------------------------------------------------------------------
   Name:        VarMathTimeDiff

   Purpose:     Subtract two TeleFlow time variables or values to get the
                difference between them in seconds.

   Modified:    27-Jun-1996 Tim Forner: Created.
   ------------------------------------------------------------------------- */
static IV_STATUS VarMathTimeDiff(const char* varB, const char* varC, char*& value)
{
   int    st_hour;
   int    en_hour;
   int    st_min;
   int    en_min;
   int    st_sec;
   int    en_sec;
   int    st_seconds;
   int    en_seconds;
   int    seconds;
   char   startTime[9];
   char   endTime[9];

   strcpy (startTime, varB);
   startTime[2] = '\0';
   startTime[5] = '\0';
   sscanf (startTime    , "%d", &st_hour);
   sscanf (startTime + 3, "%d", &st_min);
   sscanf (startTime + 6, "%d", &st_sec);

   strcpy (endTime, varC);
   endTime[2] = '\0';
   endTime[5] = '\0';
   sscanf (endTime    , "%d", &en_hour);
   sscanf (endTime + 3, "%d", &en_min);
   sscanf (endTime + 6, "%d", &en_sec);

   st_seconds = (st_hour * 3600) + (st_min * 60) + st_sec;
   en_seconds = (en_hour * 3600) + (en_min * 60) + en_sec;

   if (en_seconds > st_seconds)
      seconds = en_seconds - st_seconds;
   else
      seconds = st_seconds - en_seconds;

   delete [] value;
   value = new char [20];
   sprintf (value, "%i", seconds);
   return IVR_SUCCESS;
}
/*VarMathTimeDiff*/

static IV_STATUS VarMathDateDiff(const char* varB, const char* varC, char*& value)
{
   ESCTime firstDate;
   ESCTime secondDate;
   string  strTemp;
   float   diffSeconds;
   float   diffDays;
   unsigned int uDays;

   // Convert 1st date to a ESC time
   strTemp = varB;
   strTemp += " 00:00:00";
   firstDate.Assign(strTemp.c_str());

   // Convert 2nd date to a ESC time
   strTemp = varC;
   strTemp += " 00:00:00";
   secondDate.Assign(strTemp.c_str());

   diffSeconds = firstDate.GetSecondsSince(secondDate);

   // Convert to days
   diffDays = diffSeconds / 86400;

   // For legacy reasons, the difference between two times is always a positive value
   uDays = abs(diffDays);

   value = new char [12];     // enough for 32-bit int (10 digits), NULL and margin of error
   sprintf (value, "%i", uDays);
   return IVR_SUCCESS;
}
//VarMathDateDiff

static IV_STATUS VarMathDateTimeDiff(const char* varB, const char* varC, char*& value)
{
   ESCTime firstDate;
   ESCTime secondDate;
   float   diffSeconds;
   unsigned int uSeconds;

   // Convert 1st date to a ESC time
   firstDate.Assign(varB);

   // Convert 2nd date to a ESC time
   secondDate.Assign(varC);

   diffSeconds = firstDate.GetSecondsSince(secondDate);

   // For legacy reasons, the difference between two times is always a positive value
   uSeconds = abs(diffSeconds);

   value = new char [22];     // big enough for 64-bit int (20 digits), NULL and margin of error
                              // even though, only a 32-bit value will be stored in here
   sprintf (value, "%i", uSeconds);
   return IVR_SUCCESS;
}
//VarMathDateTimeDiff

/* -------------------------------------------------------------------------
   Name:        VarMathInt

   Purpose:     Performs mathematical operations on two TeleFlow integer
                variables or values.
   ------------------------------------------------------------------------- */
static IV_STATUS VarMathInt(const char* varB, const char* varC, MATH_FN fn, char*& value)
{
   long  intA;
// Sean change
// Added:
   long  intB = atol(varB);
   long  intC = atol(varC);

// Sean change
/* Removed:
   sscanf (varB, "%ld", &intB);
   sscanf (varC, "%ld", &intC);*/

   switch (fn) {
      case FN_ADD:  intA = intB + intC; break;
      case FN_SUB:  intA = intB - intC; break;
      case FN_MULT: intA = intB * intC; break;
      case FN_DIV:
      {
      	if(intC != 0)
         {
         	intA = intB / intC;
            break;
         }
         else
         {
		   	return(IVR_FATAL);
			}
      }
      case FN_MOD:
      {
      	if(intC != 0)
         {
				intA = intB % intC;
            break;
         }
         else
         {
		   	return(IVR_FATAL);
			}
      }
      default:
      {
	   	return(IVR_FATAL);
		}
   }

   delete [] value;
   value = new char [20];
   sprintf (value, "%ld", intA);

   return IVR_SUCCESS;
}
/*VarMathInt*/

/* -------------------------------------------------------------------------
   Name:        GetPrecision

   Purpose:     Count the number of digits after the decimal point.
   ------------------------------------------------------------------------- */
static int GetPrecision (const char *value)
{
   bool foundDecimal = false;
   int  cnt          = 0;
   int  pos          = 0;

   while (value[pos]) {
      if (foundDecimal)
         ++cnt;
      else if (value[pos] == '.')
         foundDecimal = true;
      ++pos;
   }
   return cnt;
}
/*GetPrecision*/

/* -------------------------------------------------------------------------
   Name:        VarMathFloat

   Purpose:     Performs mathematical operations on two TeleFlow float
                variables or values.
   ------------------------------------------------------------------------- */
static IV_STATUS VarMathFloat(const char* varB, const char* varC, MATH_FN fn, char*& value)
{
   double floatA;
   double floatB = atof(varB);
   double floatC = atof(varC);

   int    precisionA;
   int    precisionB;
   int    precisionC;

   switch (fn) {
      case FN_ADD:  floatA = floatB + floatC; break;
      case FN_SUB:  floatA = floatB - floatC; break;
      case FN_MULT: floatA = floatB * floatC; break;
      case FN_DIV:
      {
      	if(floatC != 0)
         {
				floatA = floatB / floatC;
         }
         else
         {
		   	return(IVR_FATAL);
			}
         break;
      }
      // 2003-05-22 JSH 2003-05-22
      case FN_POW:
      {
         // all sorts of integrity checks: no pow(0,0), no pow(intB, -1.23)
         if( ( (floatB == 0) && (floatC <= 0) ) && ( (floatC < 0) && (DetermineType(varC) != VAR_INTEGER) ) )
         {
           return(IVR_FATAL);
         }
         else
         {

           if (((floatC < 0) && (DetermineType(varC) == VAR_INTEGER)) || ((floatC > 0) && (DetermineType(varC) == VAR_FLOAT)))
           {
            // precision is somewhat meaningless with negative and fractional exponents, so set it arbitrarily
            precisionA = 5;
           }
           else
           {
            precisionB = GetPrecision (varB);
            precisionC = GetPrecision (varC);
            precisionA = (precisionB < precisionC ? precisionC : precisionB);
           }


           floatA = powl(floatB, floatC);

         }
         break;
      }
      default:
      {
	   	return(IVR_FATAL);
		}
   }
   if (fn != FN_POW) // precision already set for FN_POW
   {
    precisionB = GetPrecision (varB);
    precisionC = GetPrecision (varC);
    precisionA = (precisionB < precisionC ? precisionC : precisionB);
   }
   
   delete [] value;
   value = new char [20];
   sprintf (value, "%.*f", precisionA, floatA);
   return IVR_SUCCESS;
}
/*VarMathFloat*/

/*****************************************************************************
 **** Variable Comparison Functions ******************************************
 *****************************************************************************/

/* -------------------------------------------------------------------------
   Name:        VarCmp

   Purpose:     Performs relational comparisons on two TeleFlow variables or
                values.
   ------------------------------------------------------------------------- */
IV_STATUS VarCmp(PIVRINFO pInfo, const char* varA, const char* varB, CMP_FN fn, bool &result, bool bSupressLog)
{
   IV_STATUS  status;
   VAR_TYPE varTypeA;
   VAR_TYPE varTypeB;

   varTypeA = DetermineType(varA);
   varTypeB = DetermineType(varB);

   if((varTypeA == VAR_INTEGER) && (varTypeB == VAR_INTEGER))
   {
      status = VarCmpInt(varA, varB, fn, result);
   }
   else if(((varTypeA == VAR_FLOAT) || (varTypeA == VAR_INTEGER)) &&
      ((varTypeB == VAR_FLOAT) || (varTypeB == VAR_INTEGER)))
   {
      status = VarCmpFloat(varA, varB, fn, result);
   }
   else if((varTypeA == VAR_DATE && varTypeB == VAR_DATE) ||
      (varTypeA == VAR_DATETIME && varTypeB == VAR_DATETIME))
   {
      status = VarCmpDate(varA, varB, fn, result);
   }
   else
   {
      if(!(varTypeA == VAR_STRING && varTypeB == VAR_STRING) && fn != FN_NULL && !bSupressLog)
      {
         pInfo->ivLog->Print("         Unmatched data types, comparing as strings\n");
      }
      int cmpRslt = strcmp(varA, varB);

      result = ((cmpRslt == 0) && ((fn == FN_EQ) || (fn == FN_LE) || (fn == FN_GE) || (fn == FN_NULL)))
         || ((cmpRslt < 0) && ((fn == FN_LT) || (fn == FN_LE) || (fn == FN_NE)))
         || ((cmpRslt > 0) && ((fn == FN_GT) || (fn == FN_GE) || (fn == FN_NE)));

      status = IVR_SUCCESS;
   }

   return(status);
}
/*VarCmp*/

/* -------------------------------------------------------------------------
   Name:        VarCmpInt
   ------------------------------------------------------------------------- */
static IV_STATUS VarCmpInt(const char* varA, const char* varB, CMP_FN fn, bool& result)
{
   long     intA;
   long     intB;

   sscanf(varA, "%ld", &intA);
   sscanf(varB, "%ld", &intB);

   switch (fn) {
      case FN_EQ: result = (intA == intB); break;
      case FN_NE: result = (intA != intB); break;
      case FN_LT: result = (intA <  intB); break;
      case FN_LE: result = (intA <= intB); break;
      case FN_GT: result = (intA >  intB); break;
      case FN_GE: result = (intA >= intB); break;
      default:
		// Sean change
		// was: return IVR_FAIL;
	   {
	   	return(IVR_FATAL);
		}
   }
   return IVR_SUCCESS;
}
/*VarCmpInt*/

/* -------------------------------------------------------------------------
   Name:        VarCmpFloat
   ------------------------------------------------------------------------- */
static IV_STATUS VarCmpFloat(const char* varA, const char* varB, CMP_FN fn, bool& result)
{
   double    floatA = atof(varA);
   double    floatB = atof(varB);

   switch (fn) {
      case FN_EQ: result = (floatA == floatB); break;
      case FN_NE: result = (floatA != floatB); break;
      case FN_LT: result = (floatA <  floatB); break;
      case FN_LE: result = (floatA <= floatB); break;
      case FN_GT: result = (floatA >  floatB); break;
      case FN_GE: result = (floatA >= floatB); break;
      default:
		// Sean change
		// was: return IVR_FAIL;
	   {
	   	return(IVR_FATAL);
		}
   }
   return IVR_SUCCESS;
}
/*VarCmpFloat*/

/* -------------------------------------------------------------------------
   Name:        VarCmpDate
   ------------------------------------------------------------------------- */
static IV_STATUS VarCmpDate(const char* varA, const char* varB, CMP_FN fn, bool& result)
{
   char* temp1;
   char* temp2;
   int cmpRslt;

   temp1 = new char[strlen(varA)+1];
   temp2 = new char[strlen(varB)+1];
   
   // Ensure both dates have the same separator...
   strcpy(temp1, varA);
   temp1[4] = '-';
   temp1[7] = '-';

   strcpy(temp2, varB);
   temp2[4] = '-';
   temp2[7] = '-';

   // ...then they can be compared as strings
   cmpRslt = strcmp(temp1, temp2);

   result = ((cmpRslt == 0) && ((fn == FN_EQ) || (fn == FN_LE) || (fn == FN_GE) || (fn == FN_NULL)))
      || ((cmpRslt < 0) && ((fn == FN_LT) || (fn == FN_LE) || (fn == FN_NE)))
      || ((cmpRslt > 0) && ((fn == FN_GT) || (fn == FN_GE) || (fn == FN_NE)));

   delete[] temp1;
   delete[] temp2;

   return(IVR_SUCCESS);
}
//VarCmpDate

/*****************************************************************************
 **** Variable Validation Functions ******************************************
 *****************************************************************************/

/* -------------------------------------------------------------------------
   Name:        VarValid

   Purpose:     Determines if the value in a variable is a valid value of
                the expected type.
   ------------------------------------------------------------------------- */
IV_STATUS VarValid(PIVRINFO pInfo, const char* testVar, VAR_TYPE dataType, bool& result)
{
   VAR_TYPE varType;

   if((dataType != VAR_INTEGER) && (dataType != VAR_FLOAT) && (dataType != VAR_DATE)
      && (dataType != VAR_TIME) && (dataType != VAR_DATETIME))
   {
      pInfo->ivLog->Print("      ERROR: Invalid data type for validity test: %d\n", dataType);
      return(IVR_FATAL);
   }

   if(testVar == NULL)
   {
      result = false;
      return(IVR_SUCCESS);
   }

   varType = DetermineType(testVar);

   switch(dataType)
   {
      case VAR_INTEGER:
      {
         result = varType == VAR_INTEGER;
         break;
      }

      case VAR_FLOAT:
      {
         result = (varType == VAR_FLOAT) || (varType == VAR_INTEGER);
         break;
      }

      case VAR_DATE:
      {
         result = VarValidDate(testVar);
         break;
      }

      case VAR_TIME:
      {
         result = VarValidTime (testVar);
         break;
      }

      case VAR_DATETIME:
      {
         result = VarValidDateTime (testVar);
         break;
      }

      default:
	   {
  		   pInfo->ivLog->Print("      ERROR: Unknown variable type\n"
      	  					     "         Failed trying to validate '%s'\n", testVar);
	   	return(IVR_FATAL);
		}
   }

   return(IVR_SUCCESS);
}
/*VarValid*/

/* -------------------------------------------------------------------------
   Name:        VarValidDate

   Purpose:     Check that the year, month, and day values in the string
                constitute a valid date.

   Questions:   - Is the leap year code correct?  It doesn't look right to
                  me.

   Notes:       - Could use unsigned integers here.  If 'DetermineType'
                  thinks this is a valid date, than it can't be a negative
                  number.
   ------------------------------------------------------------------------- */
static bool VarValidDate (const char *dt)
{
                  //  J   F   M   A   M   J   J   A   S   O   N   D
   int  mthDays[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
   int  year;
   int  month;
   int  day;
   bool leapYear;

   if(DetermineType(dt) != VAR_DATE)
      return false;

   sscanf (dt    , "%d", &year);
   sscanf (dt + 5, "%d", &month);
   sscanf (dt + 8, "%d", &day);

   // Check the year...
   if (year < 0 || year > 9999)
      return false;

   // Check the month...
   if (month < 1 || month > 12)
      return false;

   // Check the day...
   //
   // - Years ending in 00 as in 1900, 2000 are not leap years
   //   if they cannot be divided by 400.
   // - Years not ending in 00 are leap years if they can be
   //   divided by 4.
   //

   leapYear = false;
   // Year is divisible by 4, but not divisible by 100
   if ((year % 4 == 0) && (year % 100 != 0))
      leapYear = true;

   // If Feb, and divisible by 4, and divisible by 100, and divisible by 400
   if (year % 400 == 0)
      leapYear = true;

   if (month == 2 && leapYear) {
      if (day < 1 || day > 29)
         return false;

   } else {                                     // not Feb and/or not leap year
      if (day < 1 || day > mthDays[month-1])
         return false;
   }

   // If we make it this far, date is valid
   return true;
}
/*VarValidDate*/

/* -------------------------------------------------------------------------
   Name:        VarValidTime

   Purpose:     Check that the hour, minute, and second values in the string
                constitute a valid time.

   Notes:       - Could use unsigned integers here.  If 'DetermineType'
                  thinks this is a valid time, than it can't be a negative
                  number.
   ------------------------------------------------------------------------- */
/*static*/ bool VarValidTime(const char* timeStr)
{
   int  hour;
   int  minute;
   int  second;

   sscanf (timeStr    , "%d", &hour);
   sscanf (timeStr + 3, "%d", &minute);
   if (timeStr[5])
      sscanf (timeStr + 6, "%d", &second);
   else
      second = 0;

   if (hour < 0 || hour > 23)                           // Check the hour...
      return false;

   if (minute < 0 || minute > 59)                       // Check the minutes...
      return false;

   if (second < 0 || second > 59)                       // Check the seconds...
      return false;

   // If we make it this far, time is valid.
   return true;
}
/*VarValidTime*/

static bool VarValidDateTime (const char *dt)
{
   char datepart[11];
   char timepart[9];

   if(DetermineType(dt) != VAR_DATETIME)
   {
      return(false);
   }

   strncpy(datepart, dt, 10);
   datepart[10] = '\0';
   if(!VarValidDate(datepart))
   {
      return(false);
   }

   strncpy(timepart, dt+11, 8);
   timepart[8] = '\0';
   if(!VarValidTime(timepart))
   {
      return(false);
   }

   // If we make it this far, date/time is valid
   return(true);
}
//VarValidDateTime

/* -------------------------------------------------------------------------
   Name:        VarLUHN

   Purpose:     Determines if the value in a variable is a valid LUHN (mod-10 number)

   Notes: 		 Fails if the value is not a VAR_INTEGER

   Added:       2003/04/18 - Jason Holmes
   ------------------------------------------------------------------------- */
IV_STATUS VarLUHN(PIVRINFO pInfo, char* testVar, bool& result)
{

  VAR_TYPE varType;
  char* currentchar;
  int   check_value;
  int   tmp_item;
  int   sum = 0;

  varType = DetermineType(testVar);
  
  if((testVar == NULL) || (strlen(testVar) == 0))
  {
     result = false;
     return(IVR_SUCCESS);
  }


  if (varType != VAR_INTEGER)
  {
     pInfo->ivLog->Print("      ERROR: Invalid data type for LUHN (mod 10) test: %d\n", varType);
     return(IVR_FATAL);
  }

  currentchar = testVar + strlen(testVar) - 1;
  // get the value that is the checksum (rightmost digit)
  check_value = *currentchar - '0';
  currentchar--;

  // calculate the running sum using the 2121212.... running multiplication mask
  for (int multiplier=2; currentchar >= testVar; currentchar--)
  {
    tmp_item = (*currentchar - '0') * multiplier; // multiply by 1 or 2

    // this works even if result <= 9 as result % 10 is result
    sum += ((tmp_item / 10) + (tmp_item % 10)); // sum the resulting digits

    if (multiplier == 2)
    {
      multiplier = 1; // set counter to not 2 <grin>
    }
    else
    {
      multiplier = 2;
    }
  } // end for

  result = ((sum % 10 > 0) && (check_value == 10 - sum % 10)) || (check_value == 0);

  return(IVR_SUCCESS);

}
/*VarLUHN*/

/* -------------------------------------------------------------------------
   Name:        VarFinCalc
   ------------------------------------------------------------------------- */
IV_STATUS VarFinCalc(PIVRINFO pInfo, const char* pszResult, const char* pszAnnualIntRate,
   const char* pszMortgageAmt, const char* pszGrossIncome, FINCALC_FN fn)
{
   IV_STATUS  status;
   double     annualIntRate;
   double     mortgageAmt;
   double     grossIncome;
   double     result;
   char       pszValue[80];

   if (!pszResult || *pszResult != '@') {
      pInfo->ivLog->Print("      ERROR: VarFinCalc cannot return value; no variable specified\n");
      return IVR_FATAL;
   }

   // Cannot store values into system variables->
   if (pInfo->variables->IsSystem(pszResult))
   {
      pInfo->ivLog->Print("      ERROR: Attempt to assign a value to system variable '%s'\n", pszResult);
      return IVR_FATAL;
   }

   // Display contents of pszAIR variable.
   if (pInfo->ivLog->GetDebug ('v'))
   {
      pInfo->ivLog->Print("         (%s): %s\n", VarTypeString(DetermineType(pszAnnualIntRate)), pszAnnualIntRate);
   }

   if (fn == FIN_MTHLY_PYMT) { // -------------------- MONTHLY MORTGAGE PAYMENT
      // Display contents of pszMA variable.
      if (pInfo->ivLog->GetDebug ('v'))
      {
         pInfo->ivLog->Print("         (%s): %s\n",
      				VarTypeString(DetermineType(pszMortgageAmt)), pszMortgageAmt);
      }

      sscanf(pszAnnualIntRate, "%lf", &annualIntRate);
      sscanf(pszMortgageAmt, "%lf", &mortgageAmt);
      result = CalcMonthlyPayment (annualIntRate, mortgageAmt);
      sprintf (pszValue, "%.2lf", result);
      status = pInfo->variables->Set(pszResult, pszValue) ? IVR_SUCCESS : IVR_FATAL;
  }
   else if (fn == FIN_MORT_AMT) { // -------------------------- MORTGAGE AMOUNT
      // Display contents of pszGI variable.
      if (pInfo->ivLog->GetDebug ('v'))
      {
         pInfo->ivLog->Print("         (%s): %s\n",
                	VarTypeString(DetermineType(pszGrossIncome)), pszGrossIncome);
      }

      sscanf(pszAnnualIntRate, "%lf", &annualIntRate);
      sscanf(pszGrossIncome, "%lf", &grossIncome);
      result = CalcMaxMortgageAmount (annualIntRate, grossIncome);
      sprintf (pszValue, "%.2lf", result);
      status = pInfo->variables->Set(pszResult, pszValue) ? IVR_SUCCESS : IVR_FATAL;
   }
   else { // ------------------------------------------------------------------
      pInfo->ivLog->Print("      ERROR: Unhandled operation in VarFinCalc\n");
      pInfo->ivLog->Print("      HINT: Ensure variables in operation are of the correct types\n");
      status = IVR_FATAL;
   }

   return(status);
}
/*VarFinCalc*/

/* -------------------------------------------------------------------------
   Name:        CalcMonthlyPayment
   ------------------------------------------------------------------------- */
double CalcMonthlyPayment (double annualIntRate, double mortgageAmt)
{
   double  monthlyPmt;
   double  monthlyIntRate;

   monthlyIntRate = pow (1.0 + annualIntRate / 100.0 / 2.0, 1.0 / 6.0) - 1.0;
   monthlyPmt = mortgageAmt
              / ((1.0 - pow (1.0 + monthlyIntRate, -300.0)) / monthlyIntRate);

   return monthlyPmt;
}
/*CalcMonthlyPayment*/

/* -------------------------------------------------------------------------
   Name:        CalcMaxMortgageAmount
   ------------------------------------------------------------------------- */
double CalcMaxMortgageAmount (double annualIntRate, double grossIncome)
{
   double  maxMortgageAmt;
   double  monthlyIntRate;
   double  monthlyPmt;

   monthlyPmt = grossIncome * 0.3 / 12.0;
   monthlyIntRate = pow (1.0 + annualIntRate / 100.0 / 2.0, 1.0 / 6.0) - 1.0;
   maxMortgageAmt = monthlyPmt * ((1.0 - pow (1.0 + monthlyIntRate, -300.0))
                                  / monthlyIntRate);

   return maxMortgageAmt;
}
/*CalcMaxMortgageAmount*/
