#ifndef PARSETAM_H
#define PARSETAM_H

#include <iostream>
using std::cout;
using std::cin;
using std::endl;

#include <string>
using std::string;

#include <vector>
using std::vector;

#define TIXML_USE_STL

// Warnings disabled
#pragma warn -8027
#include "..\3rdParty\tinyxml\tinyxml.h"
// Warnings enabled
#pragma warn +8027

#include "XMLTFAppParser.h"
#include "txtfile.h"

class ParseTam : public XMLTFAppParser
{
   public:

      //Attempts to open a XML Tam file.
      //The XML tam file steps get loaded into a vector. If it fails to load XML
      //tam file the vector stays empty. 
      ParseTam(string fileName,Log* outLog);
      ~ParseTam(void);

      //Fetches a single teleflow step and loads it into the step structure.
      //IVR_FAILURE or IVR_SUCCESS is returned on failure or success. 
      IV_STATUS FetchXmlStep(int step, PSTEPREC stepRec);

      //Deallocates the memory allocated by FetchXMLStep.
      void FreeStepRecMemory(PSTEPREC stepRec);

      //Checks if a parameter field exists of the value 'parameterValue' if non is
      //found zero is returned otherwise one is returned.
      bool TamParameterValue(int parameterValue,int step);

      //Retrieves the  Description from the Parameter field specified by parameterValue.
      // 1 or 0 is returned on success or failure.
      int TamParameterDescription(int parameterValue, int step, string& value);

      //Retrieves the Parameter Name from the parameter field. Is either a variable
      //or a constant. 1 or 0 is returned on success or failure.
      int TamParameterName(int parameterValue,int step, string& value);

      //Retrieves the Parameter Type from the Parameter field in a tam xml file.
      //If passed by reference 1 is returned, if by value 0 is returned.
      int TamParameterType(int parameterValue, int step, int* num);
      
   private:

      //Loads all the Teleflow steps into the VectorStepNodes vector.
      //-> 'TiXmlElement *pElem' points to the root of the Teleflow XML file.
      // 1 or 0 is returned on success or failure.
      int LoadVectorSteps(TiXmlElement* pElem);
   
      //Loads the child steps of a Teleflow step. The child steps also get pushed
      //onto the VectorStepNodes right after the Parent step.
      //-> 'TiXmlNode *pParent' points to the TeleFlowStep node.
      // 1 or 0 is returned on success or failure.
      int LoadChildStep(TiXmlNode* pParent);

      Log *ivLog;
};
#endif
