/* c headers */
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* enGenic headers */
#include "speak.h"
#include "iv_math2.h"

typedef struct {
   char  *element;
   char  *file;
} SPEAKDATA;

typedef struct {
   int         maxElements;
   bool        onesBeforeGroups;
   bool        teenFor70;
   bool        teenFor90;
   bool        andFor1;
   bool        speakFullYear;
   SPEAKDATA * data;
} SPEAKLANG;

static SPEAKDATA SpeakFrench[] = {
   {"ZERO",             "00"},           //   0
   {"UN",               "01"},           //   1
   {"DEUX",             "02"},           //   2
   {"TROIS",            "03"},           //   3
   {"QUATRE",           "04"},           //   4
   {"CINQ",             "05"},           //   5
   {"SIX",              "06"},           //   6
   {"SEPT",             "07"},           //   7
   {"HUIT",             "08"},           //   8
   {"NEUF",             "09"},           //   9
   {"DIX",              "10"},           //  10
   {"ONZE",             "11"},           //  11
   {"DOUZE",            "12"},           //  12
   {"TRIEZE",           "13"},           //  13
   {"QUATORZE",         "14"},           //  14
   {"QUINZE",           "15"},           //  15
   {"SEIZE",            "16"},           //  16
   {"DIX-SEPT",         "17"},           //  17
   {"DIX-HUIT",         "18"},           //  18
   {"DIX-NEUF",         "19"},           //  19
   {"VINGT",            "20"},           //  20
   {"TRENTE",           "30"},           //  21
   {"QUARANTE",         "40"},           //  22
   {"CINQUANTE",        "50"},           //  23
   {"SOIXANTE",         "60"},           //  24
   {"SOIXANTE-DIX",     "70"},           //  25
   {"QUATRE-VINGT",     "80"},           //  26
   {"QUATRE-VINGT-DIX", "90"},           //  27
   {"CENT",             "hundred"},      //  28
   {"MILLE",            "thousand"},     //  29
   {"million (sp?)",    "million"},      //  30 - How to say 'Million'?
   {"billion (sp?)",    "billion"},      //  31 - How to say 'Billion'?
   {"trillion (sp?)",   "trillion"},     //  32 - How to say 'Trillion'?
   {"JANVIER",          "jan"},          //  33
   {"FEVRIER",          "feb"},          //  34
   {"MARS",             "mar"},          //  35
   {"AVRIL",            "apr"},          //  36
   {"MAI",              "may"},          //  37
   {"JUIN",             "jun"},          //  38
   {"JUILLET",          "jul"},          //  39
   {"AOUT",             "aug"},          //  40
   {"SEPTEMBRE",        "sep"},          //  41
   {"OCTOBRE",          "oct"},          //  42
   {"NOVEMBRE",         "nov"},          //  43
   {"DECEMBRE",         "dec"},          //  44
   {"SECONDES",         "seconds"},      //  45
   {"MOINS",            "minus"},        //  46
   {"DOLLAR",           "dollar"},       //  47
   {"DOLLAR",           "dollar"},       //  48 - 'dollar' plural?
   {"SOUS",             "cent"},         //  49
   {"SOUS",             "cent"},         //  50 - 'sous' (cent) plural?
   {"ET",               "and"},          //  51
   {"PREMIER",          "1st"},          //  52
   {"DEUXIEME",         "2nd"},          //  53
   {"TROISIEME",        "3rd"},          //  54
   {"QUATRIEME",        "4th"},          //  55
   {"CINQUIEME",        "5th"},          //  56
   {"SIXIEME",          "6th"},          //  57
   {"SEPTIEME",         "7th"},          //  58
   {"HUITIEME",         "8th"},          //  59
   {"NEUVIEME",         "9th"},          //  60
   {"DIXIEME",          "10th"},         //  61
   {"ONZIEME",          "11th"},         //  62
   {"DOUXIEME",         "12th"},         //  63
   {"TREIZIEME",        "13th"},         //  64
   {"QUATORZIEME",      "14th"},         //  65
   {"QUINZIEME",        "15th"},         //  66
   {"SEIZIEME",         "16th"},         //  67
   {"DIX-SEPTIEME",     "17th"},         //  68
   {"DIX-HUITIEME",     "18th"},         //  69
   {"DIX-NEUVIEME",     "19th"},         //  70
   {"VINGTIEME",        "20th"},         //  71
   {"TRENTIEME",        "30th"},         //  72
   {"(pause)",          "pause"},        //  73
   {"HEURE",            "oclock"},       //  74
   {"AVANT-MIDI",       "am"},           //  75
   {"APRES-MIDI",       "pm"},           //  76
   {NULL,               NULL},               //  77 - Don't speak 'Oh'.
   {"POINT",            "point"},        //  78
   {"LUNDI",            "mon"},          //  79
   {"MARDI",            "tue"},          //  80
   {"MERCREDI",         "wed"},          //  81
   {"JEUDI",            "thu"},          //  82
   {"VENDREDI",         "fri"},          //  83
   {"SAMEDI",           "sat"},          //  84
   {"DIMANCHE",         "sun"},          //  85
   {"A",                "at"},           //  86
   {"HEURE",            "hour"},         //  87 - 'Heure' plural?
   {"A",                "a"},            //  88
   {"B",                "b"},            //  89
   {"C",                "c"},            //  90
   {"D",                "d"},            //  91
   {"E",                "e"},            //  92
   {"F",                "f"},            //  93
   {"G",                "g"},            //  94
   {"H",                "h"},            //  95
   {"I",                "i"},            //  96
   {"J",                "j"},            //  97
   {"K",                "k"},            //  98
   {"L",                "l"},            //  99
   {"M",                "m"},            // 100
   {"N",                "n"},            // 101
   {"O",                "o"},            // 102
   {"P",                "p"},            // 103
   {"Q",                "q"},            // 104
   {"R",                "r"},            // 105
   {"S",                "s"},            // 106
   {"T",                "t"},            // 107
   {"U",                "u"},            // 108
   {"V",                "v"},            // 109
   {"W",                "w"},            // 110
   {"X",                "x"},            // 111
   {"Y",                "y"},            // 112
   {"Z",                "z"},            // 113
   {"ZEE",              "zee"}           // 114
}; // SpeakFrench

SPEAKDATA SpeakEnglish[] = {
   {"ZERO",             "00"},           //   0
   {"ONE",              "01"},           //   1
   {"TWO",              "02"},           //   2
   {"THREE",            "03"},           //   3
   {"FOUR",             "04"},           //   4
   {"FIVE",             "05"},           //   5
   {"SIX",              "06"},           //   6
   {"SEVEN",            "07"},           //   7
   {"EIGHT",            "08"},           //   8
   {"NINE",             "09"},           //   9
   {"TEN",              "10"},           //  10
   {"ELEVEN",           "11"},           //  11
   {"TWELVE",           "12"},           //  12
   {"THIRTEEN",         "13"},           //  13
   {"FOURTEEN",         "14"},           //  14
   {"FIFTEEN",          "15"},           //  15
   {"SIXTEEN",          "16"},           //  16
   {"SEVENTEEN",        "17"},           //  17
   {"EIGHTEEN",         "18"},           //  18
   {"NINETEEN",         "19"},           //  19
   {"TWENTY",           "20"},           //  20
   {"THIRTY",           "30"},           //  21
   {"FORTY",            "40"},           //  22
   {"FIFTY",            "50"},           //  23
   {"SIXTY",            "60"},           //  24
   {"SEVENTY",          "70"},           //  25
   {"EIGHTY",           "80"},           //  26
   {"NINETY",           "90"},           //  27
   {"HUNDRED",          "hundred"},      //  28
   {"THOUSAND",         "thousand"},     //  29
   {"MILLION",          "million"},      //  30
   {"BILLION",          "billion"},      //  31
   {"TRILLION",         "trillion"},     //  32
   {"JANUARY",          "jan"},          //  33
   {"FEBRUARY",         "feb"},          //  34
   {"MARCH",            "mar"},          //  35
   {"APRIL",            "apr"},          //  36
   {"MAY",              "may"},          //  37
   {"JUNE",             "jun"},          //  38
   {"JULY",             "jul"},          //  39
   {"AUGUST",           "aug"},          //  40
   {"SEPTEMBER",        "sep"},          //  41
   {"OCTOBER",          "oct"},          //  42
   {"NOVEMBER",         "nov"},          //  43
   {"DECEMBER",         "dec"},          //  44
   {"SECONDS",          "seconds"},      //  45
   {"MINUS",            "minus"},        //  46
   {"DOLLAR",           "dollar"},       //  47
   {"DOLLARS",          "dollars"},      //  48
   {"CENT",             "cent"},         //  49
   {"CENTS",            "cents"},        //  50
   {"AND",              "and"},          //  51
   {"FIRST",            "1st"},          //  52
   {"SECOND",           "2nd"},          //  53
   {"THIRD",            "3rd"},          //  54
   {"FOURTH",           "4th"},          //  55
   {"FIFTH",            "5th"},          //  56
   {"SIXTH",            "6th"},          //  57
   {"SEVENTH",          "7th"},          //  58
   {"EIGHTH",           "8th"},          //  59
   {"NINTH",            "9th"},          //  60
   {"TENTH",            "10th"},         //  61
   {"ELEVENTH",         "11th"},         //  62
   {"TWELFTH",          "12th"},         //  63
   {"THIRTEENTH",       "13th"},         //  64
   {"FOURTEENTH",       "14th"},         //  65
   {"FIFTEENTH",        "15th"},         //  66
   {"SIXTEENTH",        "16th"},         //  67
   {"SEVENTEENTH",      "17th"},         //  68
   {"EIGHTEENTH",       "18th"},         //  69
   {"NINETEENTH",       "19th"},         //  70
   {"TWENTIETH",        "20th"},         //  71
   {"THIRTIETH",        "30th"},         //  72
   {"(pause)",          "pause"},        //  73
   {"O'CLOCK",          "oclock"},       //  74
   {"A.M.",             "am"},           //  75
   {"P.M.",             "pm"},           //  76
   {"OH",               "oh"},           //  77
   {"POINT",            "point"},        //  78
   {"MONDAY",           "mon"},          //  79
   {"TUESDAY",          "tue"},          //  80
   {"WEDNESDAY",        "wed"},          //  81
   {"THURSDAY",         "thu"},          //  82
   {"FRIDAY",           "fri"},          //  83
   {"SATURDAY",         "sat"},          //  84
   {"SUNDAY",           "sun"},          //  85
   {"AT",               "at"},           //  86
   {"Hours",            "hours"},        //  87
   {"A",                "a"},            //  88
   {"B",                "b"},            //  89
   {"C",                "c"},            //  90
   {"D",                "d"},            //  91
   {"E",                "e"},            //  92
   {"F",                "f"},            //  93
   {"G",                "g"},            //  94
   {"H",                "h"},            //  95
   {"I",                "i"},            //  96
   {"J",                "j"},            //  97
   {"K",                "k"},            //  98
   {"L",                "l"},            //  99
   {"M",                "m"},            // 100
   {"N",                "n"},            // 101
   {"O",                "o"},            // 102
   {"P",                "p"},            // 103
   {"Q",                "q"},            // 104
   {"R",                "r"},            // 105
   {"S",                "s"},            // 106
   {"T",                "t"},            // 107
   {"U",                "u"},            // 108
   {"V",                "v"},            // 109
   {"W",                "w"},            // 110
   {"X",                "x"},            // 111
   {"Y",                "y"},            // 112
   {"Z",                "z"},            // 113
   {"ZEE",              "zee"},          // 114
   {"Exclamation",      "exclamation"},  // 115  (Exclamation point)
   {"Double quote",     "dquote"},       // 116  (Double quote)
   {"Pound sign",       "pound"},        // 117  (Pound sign)
   {"Dollar sign",      "dsign"},        // 118  (Dollar sign)
   {"Percent sign",     "percent"},      // 119  (Percent sign)
   {"Ampersand",        "ampersand"},    // 120  (Ampersand)
   {"Single quote",     "squote"},       // 121  (Single quote)
   {"Open parentheses", "openparen"},    // 122  (Open parentheses)
   {"Close parentheses","closeparen"},   // 123  (Close parentheses)
   {"Asterisk",         "asterisk"},     // 124  (Asterisk)
   {"Plus sign",        "plus"},         // 125  (Plus sign)
   {"Comma",            "comma"},        // 126  (Comma)
   {"Hyphen",           "hyphen"},       // 127  (Hyphen)
   {"Period",           "period"},       // 128  (Period)
   {"Forward slash",    "fslash"},       // 129  (Forward slash)
   {"Colon",            "colon"},        // 130  (Colon)
   {"Semicolon",        "semicolon"},    // 131  (Semicolon)
   {"Less than",        "lessthan"},     // 132  (Less than symbol)
   {"Equal sign",       "equal"},        // 133  (Equal sign)
   {"Greater than",     "morethan"},     // 134  (Greater than symbol)
   {"Question mark",    "question"},     // 135  (Question mark)
   {"At sign",          "atsign"},       // 136  (At sign)
   {"Open square",      "lsqbracket"},   // 137  (Open square bracket)
   {"Back slash",       "bslash"},       // 138  (Back slash)
   {"Close square",     "rsqbracket"},   // 139  (Close square bracket)
   {"Caret",            "caret"},        // 140  (Caret)
   {"Underscore",       "underscore"},   // 141  (Underscore)
   {"Grave accent",     "grave"},        // 142  (Grave accent)
   {"Open brace",       "lbrace"},       // 143  (Open brace)
   {"Vertical bar",     "vertbar"},      // 144  (Vertical bar (line))
   {"Close brace",      "rbrace"},       // 145  (Close brace)
   {"Tilde",            "tilde"},        // 146  (Tilde)
   {"Minute",           "minute"},       // 147  (Minute)
   {"Minutes",          "minutes"}       // 148  (Minutes)
}; // SpeakEnglish

/* -------------------------------------------------------------------------
   Function:    SetLang

   Notes:       - May want to default to English.  I think it is better to
                  fail.
   ------------------------------------------------------------------------- */
static IV_STATUS SetLang (SPEAKLANG *sl, int lang)
{
   switch (lang) {
      case SPEAK_FRENCH:
         sl->data = SpeakFrench;
         sl->maxElements = sizeof(SpeakFrench) / sizeof(SPEAKDATA);
         sl->teenFor70 = true;
         sl->teenFor90 = true;
         sl->andFor1 = true;
         sl->onesBeforeGroups = false;
         sl->speakFullYear = true;
         break;

      case SPEAK_ENGLISH:
         sl->data = SpeakEnglish;
         sl->maxElements = sizeof(SpeakEnglish) / sizeof(SPEAKDATA);
         sl->teenFor70 = false;
         sl->teenFor90 = false;
         sl->andFor1 = false;
         sl->onesBeforeGroups = true;
         sl->speakFullYear = false;
         break;

      default:
         sl->data = NULL;
         sl->maxElements = 0;
         return IVR_FAIL;
   }

   return IVR_SUCCESS;
}
/*SetLang*/

/* -------------------------------------------------------------------------
   Function:    AddMsg

   Purpose:     Only add a message to the message list, if it is defined.
   ------------------------------------------------------------------------- */
static void AddMsg (SPEAKLANG *sl, int *msgList, int *msgPos, int msgNo)
{
   if (sl && sl->data && msgNo < sl->maxElements && sl->data[msgNo].element) {
      msgList[*msgPos] = msgNo;
      ++(*msgPos);
      msgList[*msgPos] = -1;    // Terminate the list.
   }
}
/*AddMsg*/


/* -------------------------------------------------------------------------
   Function:    bslAlphaNum

   Purpose:     Populate a list of files to play in order to "spell" a given
                alpha/numeric value.

   Notes:       - All strings must be null terminated.
                - This function adds to the msgList array, pushing
                  the -1 marker along with it.
                - Also updates msgPos.

   Modified:    09-Aug-1996: Tim Forner: Created.
                19-Nov-1997: Jamie Royer: Added SPEAKLANG structure. Removed
                   hard-coded references to SPEAKDATA elements.
   ------------------------------------------------------------------------- */
static bool bslAlphaNum (
   SPEAKLANG *sl,            // IN:     Language information structure.
   char      *str,           // IN:     String to speak.
   int       *msgList,       // OUT:    Array of messages to be played.
   int       *msgPos)        // IN/OUT: Offset into msgList.
{
   int   i;                  // Offset into 'string'.
   bool  PrevPause;          // Flag to denote previous character was a pause.

   // Build a list of messages to play.
   PrevPause = true;
   i = 0;

   while (str[i] != '\0') {
      if (isalpha((int)str[i])) {
         AddMsg (sl, msgList, msgPos, toupper((int)str[i]) - 'A' + SPEAK_A);
         PrevPause = false;

      } else if (isdigit((int)str[i])) {
         AddMsg (sl, msgList, msgPos, (int)str[i] - '0' + SPEAK_ZERO);
         PrevPause = false;

      } else if (str[i] == '.') {
         AddMsg (sl, msgList, msgPos, SPEAK_POINT);
         PrevPause = false;

      } else if (!PrevPause) {
         AddMsg (sl, msgList, msgPos, SPEAK_PAUSE);
         PrevPause = true;
      }
      ++i;
   }

   return true;
}
/*bslAlphaNum*/


/* -------------------------------------------------------------------------
   Function:    bslThreeNumbers

   Purpose:     Populate a list of messages to play in order to speak a
                given numeric value, up to three digits.

   Notes:       - All strings must be null terminated.
                - This function adds to the msgList array, pushing
                  the -1 marker along with it.
                - Also updates msgPos.

   Modified:    09-Aug-1996 Tim Forner: Created.
                19-Nov-1997 Jamie Royer: Added SPEAKLANG structure. Removed
                   hard-coded references to SPEAKDATA elements.
                21-Nov-1997 Jamie Royer: Modified code to support French.
                   It *should* be possible to cleanup this code.
   ------------------------------------------------------------------------- */
static bool bslThreeNumbers (
   SPEAKLANG *sl,            // IN:     Language information structure.
   char      *str,           // IN:     String to speak.
   int       *msgList,       // OUT:    Array of messages to be played.
   int       *msgPos,        // IN/OUT: Offset into msgList.
   int        groupCnt = 0,  // IN:
   int        groupCurr = 0) // IN:
{
   int        len;
   int        i;
   bool       useTeen;

   useTeen = false;
   i = len = strlen (str);

   while (i > 0) {
      switch (i) {
         case  3:                                           // Hundreds column.
            if (str[0] != '0') {
               if (sl->onesBeforeGroups && str[0] == '1')
                  AddMsg (sl, msgList, msgPos, SPEAK_ONE);
               else if (str[0] > '1')
                  AddMsg (sl, msgList, msgPos, str[0] - '0' + SPEAK_ZERO);
               AddMsg (sl, msgList, msgPos, SPEAK_HUNDRED);
            }
            break;

         case  2:                                           // Tens column.
            if (sl->teenFor70 && str[len-2] == '7') {
               AddMsg (sl, msgList, msgPos, SPEAK_SIXTY);
               useTeen = true;

            } else if (sl->teenFor90 && str[len-2] == '9') {
               AddMsg (sl, msgList, msgPos, SPEAK_EIGHTY);
               useTeen = true;

            } else if (str[len-2] >= '2')
               AddMsg (sl, msgList, msgPos, str[len-2] - '2' + SPEAK_TWENTY);
            else if (str[len-2] == '1')
               useTeen = true;
            break;

         case  1:                                           // Ones column.
            if (useTeen)
               AddMsg (sl, msgList, msgPos, str[len-1] - '0' + SPEAK_TEN);

            else if (len == 2 && str[0] == '0' && str[1] == '0')
               AddMsg (sl, msgList, msgPos, SPEAK_ZERO);

            else if (len == 3 && groupCnt > 0 &&
                     str[0] == '0' && str[1] == '0' &&
                     sl->andFor1 && str[2] == '1')
            {
               AddMsg (sl, msgList, msgPos, SPEAK_AND);
               AddMsg (sl, msgList, msgPos, SPEAK_ONE);
            }

            else if (len == 3 && str[0] != '0' && str[1] == '0' &&
                     sl->andFor1 && str[2] == '1')
            {
               AddMsg (sl, msgList, msgPos, SPEAK_AND);
               AddMsg (sl, msgList, msgPos, SPEAK_ONE);
            }
            else if (len > 1 && str[len-2] != '0' && sl->andFor1 &&
                     str[len-1] == '1')
            {
               AddMsg (sl, msgList, msgPos, SPEAK_AND);
               AddMsg (sl, msgList, msgPos, SPEAK_ONE);

            } else if (str[len-1] != '0' || len == 1) {
               if (groupCurr > 0 && str[len-1] == '1' && sl->onesBeforeGroups)
                  AddMsg (sl, msgList, msgPos, SPEAK_ONE);
               else if (groupCurr == 0 && str[len-1] == '1')
                  AddMsg (sl, msgList, msgPos, SPEAK_ONE);
               else if (str[len-1] == '0' || str[len-1] > '1')
                  AddMsg (sl, msgList, msgPos, str[len-1] - '0' + SPEAK_ZERO);
            }
            break;

         default:    // INTERNAL ERROR: Bad string passed to bslThreeNumbers().
            return false;
      }
      --i;
   }

   return true;
}
/*bslThreeNumbers*/


/* -------------------------------------------------------------------------
   Function:    CheckDate

   Purpose:     Determine if string matches a date format.
   ------------------------------------------------------------------------- */
static bool CheckDate(const char* str, int strLen)
{
   bool  ret;

   ret = false;

   if (strLen == 5 &&
       isdigit (str[0]) && isdigit (str[1]) && str[2] == '/' &&
       isdigit (str[3]) && isdigit (str[4]))
      ret = true;

   else if (strLen == 8 &&
            isdigit (str[0]) && isdigit (str[1]) && str[2] == '/' &&
            isdigit (str[3]) && isdigit (str[4]) && str[5] == '/' &&
            isdigit (str[6]) && isdigit (str[7]))
      ret = true;

   else if (strLen == 10 &&
            isdigit (str[0]) && isdigit (str[1]) && str[2] == '/' &&
            isdigit (str[3]) && isdigit (str[4]) && str[5] == '/' &&
            isdigit (str[6]) && isdigit (str[7]) &&
            isdigit (str[8]) && isdigit (str[9]))
      ret = true;

   return (ret);
}
/*CheckDate*/


/* -----------------------------------------------------------------------
   Function:    ConvertDate

   Purpose:     Parses string and builds a message list to speak the date.
                "CheckDate" should be called before calling this function.

   Notes:       - To speak year, need a function to speak "19" as
                  "nineteen".
                - **BUG** speakData "thirtieth" is wrong.
                - Missing code to determine current year.
                - No checking for leap year.
   ----------------------------------------------------------------------- */
static bool ConvertDate (
   SPEAKLANG         *sl,        // IN:     Language information structure.
   SPEAK_FORMAT_TYPE  fmt,       // IN:
   const char* str,             // IN:     String to speak.
   int               *msgList,   // OUT:    Array of messages to be played.
   int               *msgPos)    // IN/OUT: Offset into msgList.
{
   int  strLen;
   int  month;
   int  day;
   char yearStart[3];
   char yearEnd[3];
   char year[5];
                //    J   F   M   A   M   J   J   A   S   O   N   D
   int  maxDay[12] = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

   strLen = strlen (str);

   yearStart[0] = '1';  // Need to calculate default value.
   yearStart[1] = '9';  // Need to calculate default value.
   yearStart[2] = '\0';

   yearEnd[0] = '9';    // Need to calculate default value.
   yearEnd[1] = '7';    // Need to calculate default value.
   yearEnd[2] = '\0';

   month = str[0] - '0';
   month *= 10;
   month += str[1] - '0';

   day = str[3] - '0';
   day *= 10;
   day += str[4] - '0';

   if (strLen == 8) {          // Year Format 'YY' default to 19YY.
      yearEnd[0] = str[6];
      yearEnd[1] = str[7];

   } else if (strLen == 10) {  // Year Format 'YYYY'.
      yearStart[0] = str[6];
      yearStart[1] = str[7];
      yearEnd[0] = str[8];
      yearEnd[1] = str[9];
   }

   if (month > 13 || day > maxDay[month - 1])
      return (false);

   // SPKFMT_DATE:      Month Day, Year (e.g. January twenty-first two thousand)
   // SPKFMT_MONTHDAY:  Month Day (e.g. January twenty-first)
   // SPKFMT_MONTHYEAR: Month Year (e.g. January two thousand)

   AddMsg (sl, msgList, msgPos, month + SPEAK_JANUARY - 1);

   if (fmt == SPKFMT_DATE || fmt == SPKFMT_MONTHDAY) {
      if (day < 21)
         AddMsg (sl, msgList, msgPos, day + SPEAK_FIRST - 1);

      else if (day > 20 && day < 30) {
         AddMsg (sl, msgList, msgPos, SPEAK_TWENTY);
         day -= 20;
         AddMsg (sl, msgList, msgPos, day + SPEAK_FIRST - 1);

      } else if (day == 30)
         AddMsg (sl, msgList, msgPos, SPEAK_THIRTIETH);

      else {
         AddMsg (sl, msgList, msgPos, SPEAK_THIRTY);
         AddMsg (sl, msgList, msgPos, SPEAK_FIRST);
      }
   }

   if (fmt == SPKFMT_MONTHDAY)
      return true;

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - --
      if      0000 - 0999 say three digit number
      else if X000 - X009 or say X thousand + last three digits where X > 0
      else
         if speakFullYear == true
            say yearStart[0] + "thousand" + last three digits
            (this is the same as X000 - X009)
         else
            say yearStart + yearEnd (e.g. 1999 = nineteen ninety nine)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
   if (yearStart[0] == '0') {
      year[0] = yearStart[1];
      year[1] = yearEnd[0];
      year[2] = yearEnd[1];
      year[3] = '\0';
      bslThreeNumbers (sl, year, msgList, msgPos);
   }
   else if (yearStart[1] == '0' || sl->speakFullYear) {
      AddMsg (sl, msgList, msgPos, SPEAK_ZERO + yearStart[0] - '0');
      AddMsg (sl, msgList, msgPos, SPEAK_THOUSAND);
      year[0] = yearStart[1];
      year[1] = yearEnd[0];
      year[2] = yearEnd[1];
      year[3] = '\0';
      bslThreeNumbers (sl, year, msgList, msgPos);
   }
   else {
      // Speak two separate numbers (e.g. 1999 = "nineteen" "ninety nine")
      bslThreeNumbers (sl, yearStart, msgList, msgPos);
      bslThreeNumbers (sl, yearEnd, msgList, msgPos);
   }

   return true;
}
/*ConvertDate*/

/* -------------------------------------------------------------------------
   Name:        VarValidTime

   Purpose:     Check that the hour, minute, and second values in the string
                constitute a valid time.

   Notes:       - Could use unsigned integers here.  If 'DetermineType'
                  thinks this is a valid time, than it can't be a negative
                  number.
   ------------------------------------------------------------------------- */
static bool VarValidTime(const char* timeStr)
{
   int  hour;
   int  minute;
   int  second;

   sscanf (timeStr    , "%d", &hour);
   sscanf (timeStr + 3, "%d", &minute);
   if (timeStr[5])
      sscanf (timeStr + 6, "%d", &second);
   else
      second = 0;

   if (hour < 0 || hour > 23)                           // Check the hour...
      return false;

   if (minute < 0 || minute > 59)                       // Check the minutes...
      return false;

   if (second < 0 || second > 59)                       // Check the seconds...
      return false;

   // If we make it this far, time is valid.
   return true;
}
/*VarValidTime*/

/* -------------------------------------------------------------------------
   Function:    CheckTime

   Purpose:     Determine if string matches a time format.
   ------------------------------------------------------------------------- */
static bool CheckTime(const char* str, int strLen)
{
   bool  ret;

   ret = false;

   if (strLen == 5 &&
       isdigit (str[0]) && isdigit (str[1]) && str[2] == ':' &&
       isdigit (str[3]) && isdigit (str[4]))
      ret = true;

   else if (strLen == 8 &&
            isdigit (str[0]) && isdigit (str[1]) && str[2] == ':' &&
            isdigit (str[3]) && isdigit (str[4]) && str[5] == ':' &&
            isdigit (str[6]) && isdigit (str[7]))
      ret = true;

   if (ret)
      ret = VarValidTime (str);

   return (ret);
}
/*CheckTime*/


/* -------------------------------------------------------------------------
   Function:   ConvertTime12

   Purpose:    Parses a string and builds a message list to speak
               the time in 12 hour format.
               "CheckTime" should be called before calling this function.

   Modified:   05-Nov-97 (TF): Created.
   ------------------------------------------------------------------------- */
static bool ConvertTime12 (
   SPEAKLANG* sl,            // IN:     Language information structure.
   const char* str,          // IN:     String to speak.
   int* msgList,             // OUT:    Array of messages to be played.
   int* msgPos)              // IN/OUT: Offset into msgList.
{
   int  strLen;
   char hour[3];
   char minute[3];
   char second[3];
   int  numhour;
   int  nummin;
   int  numsec;
   int  am_pm;

   strLen = strlen (str);

   hour[0] = str[0];
   hour[1] = str[1];
   hour[2] = '\0';
   numhour = atoi(hour);

   am_pm = AM;
   if (numhour >= 12)
      am_pm = PM;

   if (numhour == 0)
      numhour = 12;
   else if (numhour > 12)
      numhour = numhour - 12;

   sprintf (hour, "%i", numhour);

   minute[0] = str[3];
   minute[1] = str[4];
   minute[2] = '\0';
   nummin = atoi(minute);

   if (strLen == 8) {
      second[0] = str[6];
      second[1] = str[7];
      second[2] = '\0';
      numsec = atoi(second);
   } else {
      strcpy(second, "");
      numsec = -1;            // specifies that we won't be speaking seconds.
   }

   bslThreeNumbers (sl, hour, msgList, msgPos);

   if (nummin == 0)
      AddMsg (sl, msgList, msgPos, SPEAK_OCLOCK);
   else {
      if (nummin <= 9)
         AddMsg (sl, msgList, msgPos, SPEAK_OH);
      bslThreeNumbers (sl, minute, msgList, msgPos);
   }

   if (numsec > 0) {
      AddMsg (sl, msgList, msgPos, SPEAK_AND);
      bslThreeNumbers (sl, second, msgList, msgPos);
      if (numsec == 1)
         AddMsg (sl, msgList, msgPos, SPEAK_SECOND);
      else
         AddMsg (sl, msgList, msgPos, SPEAK_SECONDS);
   }

   if (am_pm == AM)
      AddMsg (sl, msgList, msgPos, SPEAK_AM);
   else
      AddMsg (sl, msgList, msgPos, SPEAK_PM);

   return (true);
}
/*ConvertTime12*/


/* -------------------------------------------------------------------------
   Function:   ConvertTime24

   Purpose:    Parses a string and builds a message list to speak the time
               in 24 hour format.
               "CheckTime" should be called before calling this function.

   Notes:      00:00 = zero hundred hours
               01:00 = oh one hundred hours
               02:15 = oh two hundred hours and 15 minutes
               10:00 = ten hundred hours

   Modified:   05-Nov-97 (TF): Created.
   ------------------------------------------------------------------------- */
static bool ConvertTime24 (
   SPEAKLANG* sl,            // IN:     Language information structure.
   const char* str,          // IN:     String to speak.
   int* msgList,             // OUT:    Array of messages to be played.
   int* msgPos)              // IN/OUT: Offset into msgList.
{
   int  strLen;
   char hour[3];
   char minute[3];
   char second[3];
   int  nummin;
   int  numsec;

   strLen = strlen (str);

   hour[0] = str[0];
   hour[1] = str[1];
   hour[2] = '\0';

   minute[0] = str[3];
   minute[1] = str[4];
   minute[2] = '\0';
   nummin = atoi(minute);

   if (strLen == 8) {
      second[0] = str[6];
      second[1] = str[7];
      second[2] = '\0';
      numsec = atoi(second);
   } else {
      strcpy(second, "");
      numsec = -1;            // specifies that we won't be speaking seconds.
   }

   if (hour[0] == '0' && hour[1] != '0')
      AddMsg (sl, msgList, msgPos, SPEAK_OH);

   bslThreeNumbers (sl, hour, msgList, msgPos);
   AddMsg (sl, msgList, msgPos, SPEAK_HUNDRED);
   AddMsg (sl, msgList, msgPos, SPEAK_HOURS);

   if (nummin > 0) {
      AddMsg (sl, msgList, msgPos, SPEAK_AND);
      bslThreeNumbers (sl, minute, msgList, msgPos);
      if (nummin == 1)
         AddMsg (sl, msgList, msgPos, SPEAK_MINUTE);
      else
         AddMsg (sl, msgList, msgPos, SPEAK_MINUTES);
   }
   if (numsec > 0) {
      AddMsg (sl, msgList, msgPos, SPEAK_AND);
      bslThreeNumbers (sl, second, msgList, msgPos);
      AddMsg (sl, msgList, msgPos, SPEAK_SECONDS);
   }

   return (true);
}
/*ConvertTime24*/


/* -------------------------------------------------------------------------
   Function:    BuildSpkList

   Purpose:     Generate a list of files to play in order to speak a given
                value.

   Notes:       - All strings must be null terminated.

               &&& YET TO DO: SPEAK MILITARY TIME VALUES!! &&&

   Modified:   09-Aug-96 (TF): Created.
               05-Nov-97 (TF): Added speak formats
   ------------------------------------------------------------------------- */
bool BuildSpkList(
   int lang,                     // IN:  Language to use.
   const char* str,              // IN:  String to be spoken.
   int* msgList,                 // OUT: List of messages to play.
   SPEAK_FORMAT_TYPE  spkFormat) // IN:  ?
{
   bool  status = true;    // Status value to be returned from this module

   int   msgPos = 0;       // Next position to add to in msgList
   int   decimalPos = -1;  // Position of decimal point within the string
   int   wholeNumLen;      // Length of the number
   int   currGroup;        // "group" (thousand, millions, etc.)
   int   groupCnt;         //
   int   position;         // "position" within "group" (ones, tens, hundreds)
   int   i;                //
   int   j;                //
   int   inLoop;           // Loop control var. to make life easier
   int   leadingZero;      //
   int   foundDecimal;     //
   char  cvtStr[255];      // Local string to do conversions with.
   char  grpStr[25];       // Local string to store groups of three digits

   int   isMoney = false;  // Denotes if value to speak is monetary

   SPEAKLANG  sl;

   SetLang (&sl, lang);

   // Initialize variable values
   //
   msgList[0] = -1;        // end of list marker

   // If the string is empty, we're done.
   if (str[0] == '\0')
      return true;

   // Scan the string once to determine data type, remove commas
   // and non-numeric data from numeric types, and
   // Also watch for decimal point and find its position.
   //
   if (str[0] == '@') {          // Alpha/numeric
      strcpy (cvtStr, str + 1);
      inLoop = false;

   } else if (str[0] == '$') {   // Money value
      strcpy (cvtStr, "");
      i = 1;
      isMoney = true;
      inLoop = true;

   } else if (CheckDate (str, strlen (str))) {
      // MM/DD, MM/DD/YY, or MM/DD/YYYY
      status = ConvertDate (&sl, spkFormat, str, msgList, &msgPos);
      return (status);

   } else if (CheckTime (str, strlen (str))) { // HH:MM or HH:MM:YY
      if (spkFormat == SPKFMT_12HOUR)   // 12 hour AM/PM format
         status = ConvertTime12 (&sl, str, msgList, &msgPos);
      else                              // 24 hour (military) format
         status = ConvertTime24 (&sl, str, msgList, &msgPos);
      return (status);

   } else {       // Assume numeric
      // (Note: may be date or time, but will need to scan to determine)
      strcpy(cvtStr, "");
      i = 0;
      inLoop = true;
   }

   j = 0;
   leadingZero = true;
   foundDecimal = false;

   while (inLoop) {
      if (isdigit ((int)str[i])) { // copy the char. to the new string
         if (! leadingZero || str[i] != '0') {
            cvtStr[j] = str[i];
            ++j;
            leadingZero = false;
         }

      } else if (str[i] == '.') {
         if (foundDecimal)
            return false;                        // Only allow for one decimal.
         else
            foundDecimal = true;

         // keep the decimal point and mark it's position
         if (leadingZero) {
            cvtStr[j] = '0';
            ++j;
            leadingZero = false;
         }
         cvtStr[j] = str[i];
         decimalPos = j;
         ++j;

      } else if (str[i] == ',') {
         // ignore commas, we'll take care of them ourselves

      } else {
         // end string when non-numeric char. encountered, incl. the NULL char.
         if (i > 0 && leadingZero) {
            cvtStr[j] = '0';
            ++j;
         }
         cvtStr[j] = '\0';
         inLoop = false;
      }
      ++i;
      if (j == sizeof(cvtStr)) {
         // end string regardless when at maximum size of cvtStr
         cvtStr[j] = '\0';
         inLoop = false;
      }
   } // endwhile (inLoop)

   // If no decimal point found, assume end of string (ie: integer number)
   // Also, how far along decimal point is determines length of number.
   if (decimalPos == -1) {
      decimalPos = j;
      wholeNumLen = j;
   } else
      wholeNumLen = decimalPos;

   // Calculate the "group", ie: thousands, millions, billions, etc.
   // And the position within the group (ones, tens, hundreds)
   groupCnt = currGroup = (wholeNumLen-1)/3;
   position = (wholeNumLen-1)%3;

   if (groupCnt > 4)
      // ERROR: Not capable of speaking numbers larger than 999 trillion.
      return (false);

   // Finally, determine how to speak it...
   //
   if (wholeNumLen > 0) {
      i = currGroup;
      while (i >= 0) {
         if (i == groupCnt) {
            strncpy(grpStr, cvtStr, position + 1);
            grpStr[position+1] = '\0';
         } else {
            strncpy(grpStr, cvtStr + position + 1, 3);
            position = position + 3;
            grpStr[3] = '\0';
         }
         status = bslThreeNumbers (&sl, grpStr, msgList, &msgPos,
                                   groupCnt, currGroup);
         if (status == false) return (false);
         if (i != 0 && atoi(grpStr) != 0)
            AddMsg (&sl, msgList, &msgPos, i + SPEAK_THOUSAND - 1);
         --i;
      }
      if (isMoney) {
         strncpy (grpStr, cvtStr, wholeNumLen);
         grpStr[wholeNumLen] = '\0';
         if (atoi(grpStr) == 1)
            AddMsg (&sl, msgList, &msgPos, SPEAK_DOLLAR);
         else
            AddMsg (&sl, msgList, &msgPos, SPEAK_DOLLARS);
      }
   }

   if ((signed)strlen(cvtStr) > decimalPos) {
      if (isMoney) {
         AddMsg (&sl, msgList, &msgPos, SPEAK_AND);
         strncpy (grpStr, cvtStr + decimalPos + 1, 2);
         if (grpStr[1] == '\0')
            grpStr[1] = '0';
         grpStr[2] = '\0';
         status = bslThreeNumbers (&sl, grpStr, msgList, &msgPos,
                                   groupCnt, currGroup);
         if (status == false) return (false);
         if (atoi(grpStr) == 1)
            AddMsg (&sl, msgList, &msgPos, SPEAK_CENT);
         else
            AddMsg (&sl, msgList, &msgPos, SPEAK_CENTS);
      } else {
         strncpy (grpStr, cvtStr + decimalPos, strlen (cvtStr) - decimalPos);
         grpStr[strlen (cvtStr) - decimalPos] = '\0';
         status = bslAlphaNum (&sl, grpStr, msgList, &msgPos);
         if (status == false) return (false);
      }
   }

   return (status);
}
/*BuildSpkList*/


/* -------------------------------------------------------------------------
   Function:    getSpeakMsg

   Purpose:     Retrieves a message description or file specification
                for a given message number.

   Notes:       - 'msgString' will be overwritten with a new value.
                - the new 'msgString' will be null terminated.
                - 'msgString' must be at least 21 characters long,
                   which includes room for the null character.

   Modified:    11-Aug-96 (TF): Created.
   ------------------------------------------------------------------------- */
bool getSpeakMsg (
   int        lang,      // IN:  Language.
   int        msgNum,    // IN:  Message number to retrieve.
   int        msgType,   // IN:  What to return: 1 = msg file name, 2 = msg desc
   char      *msgString, // OUT: String will be placed in this array
   const char *fileExt)	 // IN:  extension of prompt file  i.e. ".wav"
{
   bool       rc = true; // Return code
   SPEAKLANG  sl;

   SetLang (&sl, lang);
   if (msgNum >= 0 && msgNum <= sl.maxElements) {
      switch (msgType) {
         case 1:
            strcpy (msgString, sl.data[msgNum].file);
            strcat (msgString, fileExt);
            break;

         case 2:
            strcpy (msgString, sl.data[msgNum].element);
            break;

         default:
            rc = false;
      }
   } else     // Message number out of bounds.
      rc = false;

   return (rc);
}
/*getSpeakMsg*/


/* -------------------------------------------------------------------------
   Function:    DisplaySpkList
   ------------------------------------------------------------------------- */
void DisplaySpkList (Log *ivLog, int lang, int *msgList)
{
   SPEAKLANG  sl;
   int        i;

   SetLang (&sl, lang);
   for (i = 0; msgList[i] != -1; ++i)
      ivLog->Print ("         %s ", sl.data[msgList[i]].element);
   ivLog->Print ("\n");
}
/*DisplaySpkList*/

