#ifndef STEP_H
#define STEP_H

#include <string>
using std::string;

#include "..\escutil\link.h"
#include "..\TFServer\ivr_defs.h"
#include "..\TFServer\ivrinfo.h"
#include "..\escutil\VarController.h"
#include "txtfile.h"

/* -------------------------------------------------------------------------
   Class:       Step

   Purpose:

   Notes:       - This step could keep track of whether it succeeded or
                  failed and return the appropriate next step number.
   ------------------------------------------------------------------------- */
class Step : public Link
{
   public:

      // Operation codes
      enum OP_CODE
      {  UNKNOWN           = -999,
//       LOOPSTART         = -2,
//       LOOPEND           = -1,
         WAITFORCALL       = 10,
         PICKUP            = 11,
         PLACECALL         = 15,
         TRANSFERCALL      = 16,
         FLASH             = 17,
         TONE              = 18,
         SETREORDERTONE    = 19,
         HANG_UP           = 20,
         BLOCKCALL         = 21,
         UNBLOCKCALL       = 22,
         SWITCH            = 25,
         SWITCHDISCON      = 26,
         PROCSTART         = 30,
         PROCEND           = 40,
         MENU              = 50,
         TTGETSTRING       = 60,
         TTGETLETTER       = 63,
         TTCLEARBUF        = 65,
         TTPENDING         = 75,
         LABEL             = 80,
         FATALSTEP			= 81,
         SPACEPOINT        = 82,
         COMMENTS          = 85,
         WAIT              = 90,
         MESSAGESTEP       = 95,
         SAVELOG           = 96,
         VOXPLAY           = 100,
         VOXVARSET         = 105, // Same as VARSET. Designer step is different.
//       VOXLOWOPEN        = 106,
         VOXBEEP           = 110,
         VOXSPEAK          = 115,
         VOXREC            = 120,
         DICTATE           = 121,
//       VOXLOWCLOSE       = 125,
         VOXDEL            = 130,
//       VOXLOWPLAY        = 135,
         VOXVOLUME         = 140,
//       VOXLOWSTOP        = 145,
         VOXCOPY           = 150,
//       VOXLOWSPEED       = 155,
         REORDERTONEDETECTIONON = 160,
         REORDERTONEDETECTIONOFF = 161,
//       VOXLOWVOLUME      = 165,
//       VOXLOWPOSITION    = 175,
         TEXTSPEECH        = 180,
         CALLTRIGGER       = 190,
         STARTRECORD       = 195,
         STOPRECORD        = 196,
         PERMITSTREAMING   = 197,
         CONFOPEN          = 200,
         CONFCLOSE         = 201,
         CONFJOIN          = 202,
         CONFLEAVE         = 203,
         RELOADON          = 220,   // Safe to shutdown / reload.
         RELOADOFF         = 230,   // Not safe to shutdown / reload.
         SRBARGEINON       = 231,
         SRBARGEINOFF      = 232,
//       VARCONTROL        = 290,
         VARSET            = 300,
//       VAREDIT           = 301,
         VARRANDOM         = 302,
         VARSETPRIV        = 305,
         REGISTRYREAD      = 306,
         VARCMP            = 310,
         CASE              = 311,
         VARMATH           = 320,
         VARDATEINFO       = 324,
         FINCALC           = 325,
         VARGET            = 330,
         CREATEDIR         = 350,
         SCANDIR           = 355,
         FILEOPEN          = 360,
         FILEWRITE         = 365,
//         FILECLOSE         = 370,
         UDPIPBIND         = 370,
         UDPIPCLOSE        = 371,
         UDPIPSEND         = 372,
         UDPIPRECEIVE      = 373,
         PARSE             = 380,
         SENDEMAIL         = 385,
         TCPIPCONNECT      = 386,
         TCPIPDISCONNECT   = 387,
         TCPIPSEND         = 388,
         TCPIPRECEIVE      = 389,
         HTTP              = 390,
         SIMPULLREQUEST    = 391,   // "Simpull Request" (client specific step)
         XMLEND            = 392,   // "XML End"
         XMLFETCH          = 393,   // "XML Fetch"
         XMLLOAD           = 394,   // "XML Load"
         XMLWRITE          = 395,   // "XML Write"
         SIMPULLPARMS      = 396,   // "Simpull Parameters" (client specific step)
         SETBOARDPARM      = 399,   // set board manufacturer's API parameter
         VOCRECINIT        = 400,
         VOCRECRELEASE     = 410,
         VOCRECGETDIGITS   = 420,
         VOCRECGETYN       = 430,
         VOCRECGETSTRING   = 440,
         VOCPASSREG        = 450,   // Voice password registration step
         VOCPASSAUTH       = 451,   // voice authentication (login) step
         SRCLEARAUDIO      = 452,
         SRFLAGRESULT      = 455,
         RUNFLOWCHART      = 500,
         RUNTAP            = 505,
         GOTOLABEL         = 510,
         RUNBASICSCRIPT    = 517,
         RUNPROGRAM        = 520,
         DLLFUNCTION       = 523,
         COM               = 525,
         HASH              = 540,
         ENCRYPT           = 541,
         DECRYPT           = 542,
//       LINEACQUIRE       = 550,
//       LINERELEASE       = 555,
//       CALLREQUEST       = 560,
//       CALLANSWER        = 565,
//       CALLACCEPT        = 567,
//       CALLREJECT        = 575,
//       CALLRELEASE       = 580,
         FAXSEND           = 600,
         FAXRECEIVE        = 610,
         FAXQUEUEADD       = 620,
         FAXQUEUECLR       = 630,
         SETGLOBAL         = 700,
         SETLANGUAGE       = 705,
         SIPREGISTER       = 750,
         SETVOIPMEDIA      = 751,
         SQLCONNECT        = 800,
         SQLDISCON         = 805,
         SQLBEGINST        = 810,
         SQLLOCK           = 815,
         SQLBEGINTR        = 820,
         SQLENDTR          = 825,
         SQLENDST          = 830,
         SQLFETCH          = 835,
//       SQLCALL           = 845,

      // "action" steps:
         IVRTTOTHER        = 890,
         IVRTT0            = 900,
         IVRTT1            = 901,
         IVRTT2            = 902,
         IVRTT3            = 903,
         IVRTT4            = 904,
         IVRTT5            = 905,
         IVRTT6            = 906,
         IVRTT7            = 907,
         IVRTT8            = 908,
         IVRTT9            = 909,
         IVRTTSTAR         = 910,
         IVRTTPOUND        = 911,
         IVRTTA            = 915,
         IVRTTB            = 916,
         IVRTTC            = 917,
         IVRTTD            = 918,
         APPTIMEOUT        = 920,
         APPNOVOICE        = 921,
         APPSILENCE        = 922,
         APPTHRESHOLD      = 925,
         APPFAILURE        = 930,
         APPBUSY           = 935,
         APPCHECK          = 940,
         APPEX             = 950,
         APPTRUE           = 955,
         APPFALSE          = 956,
         NORESOURCE        = 957
         //, LABEL         = 960
         //, COMMENT       = 970
         //, CONNECT_POINT = 980
      };

      Step(STEPREC& stepRecord);
      virtual ~Step(void);

      static Step* Create(STEPREC& stepRecord);

      virtual IV_STATUS Execute(PIVRINFO pInfo) = 0;
      virtual const char* GetStepName(void) const = 0;

      int GetCurrentStep(void) const;
      int GetNextStep(void) const;
      int GetFailureStep(void) const;
      int GetTrueStep(void) const;
      int GetFalseStep(void) const;
      int GetThresholdStep(void) const;
      int GetTimeoutStep(void) const;
      int GetNoVoiceStep(void) const;
      int GetSilenceStep(void) const;
      int GetBusyStep(void) const;
      int GetNoResourceStep(void) const;
      const char* GetLabel(void) const;
      const char* GetNextLabel(void) const;
      const char* GetNextTAMName(void) const;

      void SetLabel(const char* s);
      void SetNextStep(int stepID);
      void SetCurrentStep(int stepID);
      void SetFailureStep(int stepID);
      void SetTrueStep(int stepID);
      void SetFalseStep(int stepID);
      void SetThresholdStep(int stepID);
      void SetTimeoutStep(int stepID);
      void SetNoVoiceStep(int stepID);
      void SetSilenceStep(int stepID);
      void SetBusyStep(int stepID);
      void SetNoResourceStep(int stepID);
      IV_STATUS SetString(const char* src, char** dest);
      void SetNextTAMName(const char* value);
      void SetNextLabel(const char* value);

   protected:

      int   cStep;     // Step number.
      int   nStep;     // Step to go to if successful.
      int   failure;   // Step to go to on a failure.
      int	tStep;		// Step to go to if true
      int	fStep;		// Step to go to if false
      int	thStep;		// Step to go to on a threshold failure
      int   timeout;   // Step to go to on a timeout.
      int   busy;      // Step to go to on a busy.
      int   noVoice;    // Step to go to for no voice
      int   silence;    // Step to go to for silence timeout
      int   noResource; // Step to go to for no voice
      string label;     // Step label, if applicable.
      string nextLabel; // This is only set by StepGotoLabel.
      string nextTAMName;

      IV_STATUS Status(PIVRINFO pInfo) const;
      IV_STATUS PortRequired(PIVRINFO pInfo) const;
      IV_STATUS GetScriptCodeFileNameAndFormat(PIVRINFO pInfo, const char* audioCode, char*& fileName, SOUND_FORMAT& format) const;

   private:

      // private copy constructor and operator= (define these and make them
      // public to enable copy and assignment of the class)
      Step(const Step&);
      Step& operator=(const Step&);
};

/* -------------------------------------------------------------------------
   Class:       Steps
   ------------------------------------------------------------------------- */
class Steps : public Head
{
   public:

      Steps(void);
      virtual ~Steps(void);
      Step* FindStep(int stepNo);

   private:

      // private copy constructor and operator= (define these and make them
      // public to enable copy and assignment of the class)
      Steps(const Steps&);
      Steps& operator=(const Steps&);
};

#endif
