#include <stdlib.h>            // atoi
#include <string.h>

#include "step_audioplay.h"
#include "..\escutil\iv_str.h"	// strtrim
#include "..\BoardDLLs\brd_generic.h"
#include "..\Voice\Voice.h"

StepAudioPlay::StepAudioPlay(STEPREC& stepRecord) : Step(stepRecord)
{
   bInt = (bool) stepRecord.u1;
   bFancy = (bool) stepRecord.u4;
   format = (SOUND_FORMAT) stepRecord.u2;
   msgCd = stepRecord.s1.arr;
   fileNm = stepRecord.s6.arr;
   playFromTime = stepRecord.s3.arr;
   playForTime = stepRecord.s4.arr;
}

StepAudioPlay::~StepAudioPlay(void)
{
}

const char* StepAudioPlay::GetStepName(void) const
{
	return("Play");
}

// -------------------------------------------------------------------------
//   Name:        StepAudioPlay::Execute
//
//   Questions:   - If the user specifies a play code, why try and read in the
//                  file name?
// -------------------------------------------------------------------------
IV_STATUS StepAudioPlay::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   char* value;
   string parsedScriptCode;
   string parsedFileName;
   string pathedFileName;
   int iPlayFromTime;
   int iPlayForTime;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if(!playFromTime.empty())
   {
      if((value = pInfo->variables->ParseParameter("Play From time", playFromTime.c_str(),
         "0", "Empty Play From time defaults to play from beginning")) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         iPlayFromTime = atoi(value);
         delete[] value;

         if(iPlayFromTime < 0)
         {
		      pInfo->ivLog->Print("      ERROR: Play From time '%s' must be >= 0\n", playFromTime.c_str());
            bParmsValid = false;
         }
      }
   }
   else
   {
      iPlayFromTime = 0;
   }

   if(!playForTime.empty())
   {
      // Note that FOREVER == -1
      if((value = pInfo->variables->ParseParameter("Play For time", playForTime.c_str(),
         "-1", "Empty Play For time defaults to play to end")) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         iPlayForTime = strlen(value) == 0 ? FOREVER : atoi(value);
         delete[] value;

         if((iPlayForTime < 0) && (iPlayForTime != FOREVER))
         {
		      pInfo->ivLog->Print("      ERROR: Play For time '%s' must be >= 0\n", playForTime.c_str());
            bParmsValid = false;
         }
      }
   }
   else
   {
      iPlayForTime = FOREVER;
   }

   if(!msgCd.empty())
   {
      if((value = pInfo->variables->ParseParameter("Script code", msgCd.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedScriptCode = value;
         delete[] value;
      }

      if(GetScriptCodeFileNameAndFormat(pInfo, parsedScriptCode.c_str(), value, format) != IVR_SUCCESS)
      {
         bParmsValid = false;
      }
      else
      {
         parsedFileName = value;
         delete[] value;
      }
   }
   else if(!fileNm.empty())
   {
      if((value = pInfo->variables->ParseParameter("Voice file name", fileNm.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedFileName = value;
         delete[] value;
      }
   }
   else
   {
      pInfo->ivLog->Print("      ERROR: No audio defined\n");
      bParmsValid = false;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   pathedFileName = value = pInfo->variables->FilePathSearch(parsedFileName.c_str(), pInfo->pszVoiceSearchPath);
   delete[] value;

   pInfo->ivLog->Print("      Playing '%s'\n", pathedFileName.c_str());
   if(iPlayFromTime > 0)
   {
      pInfo->ivLog->Print("         From %d ms\n", iPlayFromTime);
   }
   if(iPlayForTime > 0)
   {
      pInfo->ivLog->Print("         For %d ms\n", iPlayForTime);
   }

   status = pInfo->ivBoard->AudioPlay(pathedFileName.c_str(), bInt, format, atoi(pInfo->variables->Get("@VOLUME")),
      bFancy, iPlayFromTime, iPlayForTime);

   // Capture play information for SpeechTrainer
   if(pInfo->speechRecognition != NULL)
   {
      pInfo->speechRecognition->SRCapturePlayStats(pInfo->pszAppFile, GetCurrentStep(),
         GetStepName(), parsedScriptCode.c_str(), pathedFileName.c_str());
   }

   return(status);
}
