#include <stdlib.h>               // atoi
#include "step_confopen.h"
#include "..\BoardDLLs\brd_generic.h"

StepConfOpen::StepConfOpen(STEPREC& stepRecord) : Step(stepRecord)
{
   handleVar = stepRecord.s1.arr;
   maxSeats = stepRecord.s2.arr;
   bEchoCancel = (bool) stepRecord.u1;
   bDTMFClamping = (bool) stepRecord.u2;
   bToneClamping = (bool) stepRecord.u3;
   bCoaching = (bool) stepRecord.u4;
}

StepConfOpen::~StepConfOpen(void)
{
}

const char* StepConfOpen::GetStepName(void) const
{
	return("Open Conference");
}

IV_STATUS StepConfOpen::Execute(PIVRINFO pInfo)
{
   IV_STATUS   status;
   // Evaluate all parameters before exiting
   bool        bParmsValid = true;
   int         iHandle = 0;
   int         iSeats;
   char*       value;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(handleVar.empty())
   {
   	pInfo->ivLog->Print("      ERROR: Must provide variable to receive conference handle\n");
      bParmsValid = false;
   }
   else if(!pInfo->variables->IsVariableFormat(handleVar.c_str()))
   {
   	pInfo->ivLog->Print("      ERROR: Invalid conference handle variable name '%s'\n", handleVar.c_str());
      bParmsValid = false;
   }

   if((value = pInfo->variables->ParseParameter("Maximum number of seats", maxSeats.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iSeats = atoi(value);
      delete[] value;
      // Allow any number of seats, even 0
      // Some hardware types support this (NMS does)
      // Specific board implementation will return an error if an invalid
      // value is passed to the hardware API
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   status = pInfo->ivBoard->ConfCreate(iSeats,
                                       bEchoCancel,
                                       bDTMFClamping,
                                       bToneClamping,
                                       bCoaching,
                                       &iHandle);
   if (status == IVR_SUCCESS)
   {
      value = new char[20];
      sprintf(value, "%d", iHandle);
      if(!pInfo->variables->Set(handleVar.c_str(), value))
      {
         pInfo->ivLog->Print ("      ERROR: Failed to set conference handle variable '%s'\n", handleVar.c_str());
         status = IVR_FATAL;
      }
      delete [] value;
   }

   return(status);
}
// StepConfOpen::Execute
