#ifndef STEP_DICTATE_H
#define STEP_DICTATE_H

#include "step.h"

class StepDictate : public Step
{
   public:

      // Used in Dictate to specify the action taken when a touch tone is pressed
      enum DICTATE_ACTION
      {
         ACTION_EXIT = 0,
         ACTION_PLAY_FROM_BEGINNING,
         ACTION_REWIND_AND_PLAY,
         ACTION_PLAY_START_PAUSE,
         ACTION_RECORD_START_PAUSE,
         ACTION_FAST_FORWARD_AND_PLAY,
         ACTION_FAST_FORWARD_TO_END,
         ACTION_NO_ACTION,
         ACTION_END_RECORDING,
         ACTION_PLAY_MENU,
         ACTION_INVALID
      };

      // Used in Dictate to specify the initial behaviour
      enum START_BEHAVIOUR
      {
         BEHAVIOUR_RECORD_FROM_END = 0,
         BEHAVIOUR_RECORD_FROM_BEGINNING,
         BEHAVIOUR_RECORD_FROM_POSITION,
         BEHAVIOUR_PLAY_FROM_BEGINNING,
         BEHAVIOUR_PLAY_FROM_POSITION
      };

      // Used in Dictate to specify the behaviour when the end of the audio is reached
      enum END_BEHAVIOUR
      {
         BEHAVIOUR_EXIT = 0,
         BEHAVIOUR_PLAY_MENU
      };

      // Structure to hold the dictate action associated with a touch tone plus any additional information
      struct TTAction
      {
         StepDictate::DICTATE_ACTION action;
         char* info;
      };

      StepDictate(STEPREC& stepRecord);
      virtual ~StepDictate(void);

		const char* GetStepName(void) const;

      IV_STATUS Execute(PIVRINFO pInfo);

      // Returns the name of action
      static const char* ActionName(StepDictate::DICTATE_ACTION action);

      // Returns the name of behaviour
      static const char* StartBehaviourName(StepDictate::START_BEHAVIOUR behaviour);

      // Returns the name of behaviour
      static const char* StepDictate::EndBehaviourName(StepDictate::END_BEHAVIOUR behaviour);

   protected:

      StepDictate::TTAction TTActions[12];            // Table of actions for each TT
      string TTActionsString;                             // s1: List of actions for each TT
      string silenceTimeout;                        // s2: Silence timeout in ms
      string exitPositionVariable;                  // s3: Variable to populate with audio position in ms when Dictate exitted
      string maximumRecordTime;                     // s4: Record timeout in ms
      string startPosition;                         // s5: Audio position in ms where Dictate starts
      string dictationFileName;                     // s6: Name of file to dictate into
      string menuMessageCode;                       // s7: Message code to play for menu
      string menuTimeout;                           // s8: Timeout before repeating menu
      string menuFileName;                          // s9: Name of file to play for menu
      string menuRetries;                           // s10: Number of times to repeat menu
      StepDictate::START_BEHAVIOUR startBehaviour;    // u1: Behaviour that Dictate starts with
      bool automaticGainControl;                      // u2: Should AGC be used?
      SOUND_FORMAT dictationFormat;                   // u3: Audio format
      bool disableBeep;                               // u4: Disable record beep?
      StepDictate::END_BEHAVIOUR endBehaviour;        // u6: Behaviour when Dictate reaches end of audio
      SOUND_FORMAT menuFormat;                        // u3: Audio format

      // Interprets a string from the designer into a StepDictate::DICTATE_ACTION
      StepDictate::DICTATE_ACTION TranslateAction(const char* action);
};

#endif
