#include "step_faxreceive.h"
#include "..\BoardDLLs\brd_generic.h"

StepFaxReceive::StepFaxReceive(STEPREC& stepRecord) : Step(stepRecord)
{
   fileNm = stepRecord.s6.arr;
}

StepFaxReceive::~StepFaxReceive(void)
{
}

const char* StepFaxReceive::GetStepName(void) const
{
	return("Fax Receive");
}

// -------------------------------------------------------------------------
//   Name:        StepFaxReceive::Execute
//
//   Notes:       - Does this function need special handling for timeout to
//                  allow for waiting forever?
// -------------------------------------------------------------------------
IV_STATUS StepFaxReceive::Execute(PIVRINFO pInfo)
{
   IV_STATUS  status = IVR_SUCCESS;
   char* value;
   char* pathName;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((value = pInfo->variables->ParseParameter("Image file", fileNm.c_str())) == NULL)
   {
      return(IVR_FATAL);
   }

   pathName = pInfo->variables->FilePath(value);
   delete[] value;

   pInfo->ivLog->Print("      Receiving file '%s' \n", pathName);

   // Faxing will only function on licensed TeleFlow
   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED\n");
      status = IVR_NOTSUPPORTED;
   }
   else
   {
      status = pInfo->ivBoard->FaxReceive(pathName, NULL);
   }

   delete[] pathName;

   if(status == IVR_NOTSUPPORTED)
   {
	   pInfo->ivLog->Print("      Faxing option not installed.\n");
   }

   return(status);
}
// StepFaxReceive::Execute

