#include <stdlib.h>               // atoi
#include "step_faxsend.h"
#include "..\BoardDLLs\brd_generic.h"

StepFaxSend::StepFaxSend(STEPREC& stepRecord) : Step(stepRecord)
{
   fileNm = stepRecord.s6.arr;
   phone = stepRecord.s2.arr;
   ring = stepRecord.s3.arr;
}

StepFaxSend::~StepFaxSend(void)
{
}

const char* StepFaxSend::GetStepName(void) const
{
	return("Fax Send");
}

// -------------------------------------------------------------------------
//   Name:        StepFaxSend::Execute
//
//   Notes:       - Does this function need special handling for timeout to
//                  allow for waiting forever?
//                - Why is ring count passed FaxSend as a character string?
// -------------------------------------------------------------------------
IV_STATUS StepFaxSend::Execute(PIVRINFO pInfo)
{
   IV_STATUS status = IVR_SUCCESS;
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string phoneNo;
   int ringCount;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((value = pInfo->variables->ParseParameter("Rings before TIMEOUT", ring.c_str(), "4")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      ringCount = atoi(value);
      delete[] value;

      if(ringCount <= 0)
      {
         pInfo->ivLog->Print("      ERROR: Rings before TIMEOUT '%s' must be at least 1\n", ring.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Telephone number", phone.c_str())) == NULL)
   {
      // bParmsValid = false;
      // Allow fax send to use existing connection - TF 2010/01/13
   }
   else
   {
      phoneNo = value;
      delete[] value;
   }

   if(!fileNm.empty())
   {
      if((value = pInfo->variables->ParseParameter("File to send", fileNm.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         char* pathName = pInfo->variables->FilePath(value);
         delete[] value;

         pInfo->ivLog->Print("      Adding file '%s' \n", pathName);

         status = pInfo->ivBoard->FaxQueueAdd(pathName);
         delete[] pathName;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   // Faxing will only function on licensed TeleFlow
   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED\n");
      status = IVR_NOTSUPPORTED;
   }

   if(status == IVR_SUCCESS)
   {
      status = pInfo->ivBoard->FaxSend(phoneNo.c_str(), ringCount, "");
   }

   if(status == IVR_NOTSUPPORTED)
   {
	   pInfo->ivLog->Print("      Faxing option not installed.\n");
   }

   return(status);
}

