#include "step_fincalc.h"

StepFinCalc::StepFinCalc(STEPREC& stepRecord) : Step(stepRecord)
{
   result = stepRecord.s4.arr;
   annualIntRate = stepRecord.s1.arr;
   mortgageAmt = stepRecord.s2.arr;
   grossIncome = stepRecord.s3.arr;
   iFn = (FINCALC_FN) stepRecord.u1;
}

StepFinCalc::~StepFinCalc(void)
{
}

const char* StepFinCalc::GetStepName(void) const
{
	return("Financial Math");
}

IV_STATUS StepFinCalc::Execute(PIVRINFO pInfo)
{
   string rate;
   string amount;
   string gross;
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Annual interest rate", annualIntRate.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      rate = value;
      delete[] value;
   }

   if(iFn == FIN_MTHLY_PYMT)
   {
      if((value = pInfo->variables->ParseParameter("Mortgage amount", mortgageAmt.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         amount = value;
         delete[] value;
      }
   }
   else
   {
      if((value = pInfo->variables->ParseParameter("Gross income", grossIncome.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         gross = value;
         delete[] value;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   return(VarFinCalc(pInfo, result.c_str(), rate.c_str(), amount.c_str(), gross.c_str(), iFn));
}
