#include <stdlib.h>             // Needed for "atoi".
#include "..\escutil\iv_str.h"
#include "step_getletter.h"
#include "..\BoardDLLs\brd_generic.h"

StepGetLetter::StepGetLetter(STEPREC& stepRecord) : StepTones(stepRecord)
{
   varName = stepRecord.s1.arr;
   timeout = stepRecord.s2.arr;
   switch(stepRecord.u1)
   {
      case 1:  ltrTypeToCollect = alphaOnly;    break;
      case 2:  ltrTypeToCollect = numbersOnly;  break;
      case 3:
      default: ltrTypeToCollect = alphaNumeric; break;
   }
   qzLocation = stepRecord.u2 == 1 ? qzUseZero : qzUse7or9;
}

StepGetLetter::~StepGetLetter(void)
{
}

const char* StepGetLetter::GetStepName(void) const
{
	return("Get Letter");
}

IV_STATUS StepGetLetter::Execute(PIVRINFO pInfo)
{
   IV_STATUS   status;
   IV_STATUS   retStatus;
   bool        bLoop;
   int         volLvl;
   int         iTimeout;
   int         iLtrOffset;
   int         iFirstLtr;
   int         iLastLtr;
   int         iKeyOffset;
   int         iTTPressedSize;
   char* pszTTPressed = NULL;
   char* value;
   const char* pszLang;
   char        cNext = 0;
   char        cPrev;
   char        pszPlayLtr[2];
   char        pszKeypad[10][6] =
      { "0QZ", "1",     "2ABC", "3DEF", "4GHI",
        "5JKL", "6MNO", "7PRS", "8TUV", "9WXY" };

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((value = pInfo->variables->ParseParameter("TIMEOUT", timeout.c_str(), "@TIMEOUT")) == NULL)
   {
      return(IVR_FATAL);
   }
   else
   {
      iTimeout = atoi(value);
      delete[] value;

      if(iTimeout < 0)
      {
		   pInfo->ivLog->Print("      ERROR: TIMEOUT '%s' must be >= 0\n", timeout.c_str());
         return(IVR_FATAL);
      }
   }

   // Blank out system variable to receive touch-tones.
   if(!pInfo->variables->SetSystem("@SPELLTONES", ""))
   {
      pInfo->ivLog->Print("      ERROR: Failed to clear System @SPELLTONES variable\n");
      return(IVR_FATAL);
   }

   if(!pInfo->variables->Set(varName.c_str(), ""))
   {
      pInfo->ivLog->Print("      ERROR: Failed to clear Touch Tone Letters variable '%s'\n", varName.c_str());
      return(IVR_FATAL);
   }

   switch(ltrTypeToCollect)
   {
      case numbersOnly:
      {
         iFirstLtr = 0;
         iLastLtr  = 0;
         break;
      }

      case alphaNumeric:
      {
         iFirstLtr = 0;
         iLastLtr  = 4;
         break;
      }

      case alphaOnly:
      default:
      {
         iFirstLtr = 1;
         iLastLtr  = 4;
      }
   }

   if(qzLocation == qzUse7or9)
   {
      strcpy(pszKeypad[0], "0");
      strcpy(pszKeypad[7], "7PQRS");
      strcpy(pszKeypad[9], "9WXYZ");
   }

   volLvl = atoi(pInfo->variables->Get("@VOLUME"));
   pszLang = pInfo->variables->Get("@PROMPT");

   pszPlayLtr[0] = '\0';
   pszPlayLtr[1] = '\0';
   bLoop = true;

   iTTPressedSize = 0;
   retStatus = IVR_SUCCESS;

   while(bLoop)
   {
      cPrev = cNext;
      status = pInfo->ivBoard->GetDigit(cNext, iTimeout);

      if (status == IVR_SUCCESS) {
         charadd (pszTTPressed, cNext, iTTPressedSize);
         if (cNext != cPrev)
            iLtrOffset = iFirstLtr;

         if (cNext == '*') {
            SetNextStep (GetTT (14));
            bLoop = false;
         } else if (cNext == '#') {
            SetNextStep (GetTT (15));
            bLoop = false;
         } else {
            iKeyOffset = cNext - '0';
            if (pszKeypad[iKeyOffset][iLtrOffset]) {
               pszPlayLtr[0] = pszKeypad[iKeyOffset][iLtrOffset];
               pInfo->ivBoard->Say(pszPlayLtr, true, pszLang, volLvl, LanguageModule::SAY_ALPHA);
               ++iLtrOffset;
               if (iLtrOffset > iLastLtr || !pszKeypad[iKeyOffset][iLtrOffset])
                  iLtrOffset = iFirstLtr;
            }
         }

      } else if (status == IVR_TIMEOUT) {
         retStatus = status;
         bLoop = false;

      } else {
         delete [] pszTTPressed;
         return status;                         // No need to continue further.
      }
   }

   if(pszTTPressed)
   {
      if(!pInfo->variables->SetSystem("@SPELLTONES", pszTTPressed))
      {
         delete[] pszTTPressed;
         pInfo->ivLog->Print("      ERROR: Failed to set System variable '@SPELLTONES'\n");
         return(IVR_FATAL);
      }
   }

   if(!pInfo->variables->Set(varName.c_str(), pszPlayLtr))
   {
      pInfo->ivLog->Print("      ERROR: Failed to set Touch Tone Letters variable '%s'\n", varName.c_str());
      retStatus = IVR_FATAL;
   }

   if(pszPlayLtr[0] != '\0')
   {
      pInfo->ivLog->Print("      Got: '%c'\n", pszPlayLtr[0]);
   }

   delete[] pszTTPressed;
   return(retStatus);
}
// StepGetLetter::Execute
