#include "step_volume.h"

StepVolume::StepVolume(STEPREC& stepRecord) : Step(stepRecord)
{
   iVol = stepRecord.u1;
}

StepVolume::~StepVolume(void)
{
}

const char* StepVolume::GetStepName(void) const
{
	return("Set Volume");
}

// -------------------------------------------------------------------------
//   Name:        StepVolume::Execute
//
//   Purpose:     Set the volume setting variable.  Restrict the values for
//                the volume to -3 <= level <= 3.
// -------------------------------------------------------------------------
IV_STATUS StepVolume::Execute(PIVRINFO pInfo)
{
   char szVol[5];

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((iVol < -3) || (iVol > 3))
   {
      pInfo->ivLog->Print("      ERROR: Volume must be between -3 and 3.\n");
      return(IVR_FATAL);
   }

   sprintf(szVol, "%d", iVol);
   
   return(pInfo->variables->SetGlobal("@VOLUME", szVol) ? IVR_SUCCESS : IVR_FATAL);
}
