#include "step_xmlwrite.h"
#include "..\xmlutil\xmllist.h"

StepXMLWrite::StepXMLWrite(STEPREC& stepRecord) : Step(stepRecord)
{
   handle = stepRecord.s1.arr;
   fileNm = stepRecord.s2.arr;
   bAppend = (bool) stepRecord.u1;
}

StepXMLWrite::~StepXMLWrite(void)
{
}

const char* StepXMLWrite::GetStepName(void) const
{
	return("XML Write");
}

// -------------------------------------------------------------------------
//   Name:        StepXMLWrite::Execute
//
///   Purpose:     Write the contents of an XML object to the log
//                or to a file.
//
//   Modified:    May 14/02 - TF: Created
// -------------------------------------------------------------------------
IV_STATUS StepXMLWrite::Execute (PIVRINFO pInfo)
{
   string parsedFile;          // local copy of file name, after variables parsed
   char* value;                // used to get the value of the XML handle
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int iHandle;                // local integer copy of xml handle
   XML* pxmlReq;               // pointer to TeleFlow XML object
   char* pszText = NULL;       // used to retrieve XML so it can be output
   FILE* fh;                   // file handle

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // --- Retrieve handle number
   if((value = pInfo->variables->ParseParameter("Handle", handle.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iHandle = atoi(value);
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("File Name", fileNm.c_str(), "", "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedFile = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }
   
   // Find the handle requested
   if((pxmlReq = pInfo->xmlList->FindXML(iHandle)) == NULL)
   {
      pInfo->ivLog->Print("      ERROR: Handle '%i' not open.\n", iHandle);
      return(IVR_FATAL);
   }

   if(pxmlReq->CopyToBuffer(pszText) != IVR_SUCCESS)
   {
      pInfo->ivLog->Print("      ERROR: Could not retrieve XML.\n");
      return(IVR_FATAL);
   }

   if(!parsedFile.empty())
   {
      // Write to file
      // -------------
      if((fh = fopen(parsedFile.c_str(), bAppend ? "a" : "w")) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Could not open file '%s'.\n", parsedFile.c_str());
         delete[] pszText;
         return(IVR_FATAL);
      }

      if(!pInfo->variables->IsEmpty(pszText))
      {
         fprintf (fh, "%s\n", pszText);
      }
      fclose (fh);
   }
   else
   {
      // Write to log
      // ------------
      if(!pInfo->variables->IsEmpty(pszText))
      {
         pInfo->ivLog->Print ("%s\n", pszText);
      }
   }

   delete[] pszText;

   return(IVR_SUCCESS);
}
// StepXMLWrite::Execute
