/* -------------------------------------------------------------------------
   Notes:       - The width of the TImageList must be the width of the
                  bitmaps being loaded.
                - EAccessViolations are occurring.  How do I trap these to
                  find out where they are occurring?
   ------------------------------------------------------------------------- */
#include <vcl.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>							// _argc, _argv
#include <algorithm>
using std::max;
#pragma hdrstop

#include "..\escutil\help_fns.h"
#include "mon_main.h"
#include "dlg_host.h"
#include "dlg_status.h"
#include "mon_about.h"
#include "..\TFServer\ivrinfo.h"
#include "..\TFServer\ivr_defs.h"
#include "form_msgs.h"
#include "dlg_reconnect.h"
#include "dlg_options.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMon *FormMon;
MonEvent *monEvent;

typedef struct {
   char   Title[30];
   int    textWidth;
} COLHEAD;

//SLOTINFO MonSlots[MAXSLOTS];
COLHEAD  colHead[9];
int      yLead;                   // Offset text by this number of pixels.

/* -------------------------------------------------------------------------
   Name:        TFormMon::TFormMon                             (Constructor)
   ------------------------------------------------------------------------- */
__fastcall TFormMon::TFormMon(TComponent* Owner)
    : TForm(Owner)
{
   threadColour = clBtnFace;
   monEvent = 0;
   appNumber = 0;
   refreshInterval = 0;
   bPleaseClose = false;
   bAutoConnect = false;
   connectionPort = DEFAULT_IVR_PORT;
   ReadRegistry();
   SetupGrid();
	mnuFileReloadTAL->Enabled = false;
// SetupLights();
   EnableSocketMenu (false);
   mnuConnectionDisconnect->Enabled = false;
   ConfigureMenus();

	// Initialise the packet logs
	for(int counter1 = 0; counter1 < MAX_CLIENTS; counter1++)
   {
		for(int counter2 = 0; counter2 < INFOLOG_MAX; counter2++)
      {
			appLogs[counter1].log[counter2].currentTAMName[0] = '\0';
			appLogs[counter1].log[counter2].stepNo = 0;
			appLogs[counter1].log[counter2].stepName[0] = '\0';
			appLogs[counter1].log[counter2].lineNo = 0;
      }
      appLogs[counter1].index = 0;
      appLogs[counter1].logWrapped = false;
   }

   // Did we pass in an argument from the command line for an IP
   if(_argc > 1)
   {
   	ConnectAuto(_argv[1]);
   }
	// Should we autoconnect?
   else if(bAutoConnect && (autoConnectHost != NULL))
   {
   	ConnectAuto(autoConnectHost);
   }
}
/*TFormMon::TFormMon*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::~TFormMon                             (Destructor)
   ------------------------------------------------------------------------- */
__fastcall TFormMon::~TFormMon()
{
   // Save settings, in particular window settings, to registry
   WriteRegistry();

   if(autoConnectHost != NULL)
   {  delete[] autoConnectHost;
   }
}
/*TFormMon::~TFormMon*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::WndProc

   Purpose:     There is a bug in Windows that sometimes causes owner-drawn
                TStatusBar panels to display the text of a menu item of the
                same form.

                To fix the problem, change the 'CtlType' field to something
                other than ODT_MENU before calling the default handler for
                any object other than TMainMenu.

   Solution:    I found the folowing information on the Web:

                The owner-drawn TStatusBar triggers a WM_DRAWITEM message on
                the parent window.  From Delphi 4 on, this message is also
                caught by the TCustomForm.WndProc procedure to allow for
                owner-drawn menus.  Here is part of the relevant VCL code:
                   ...
                   WM_DRAWITEM:
                      with PDrawItemStruct(Message.LParam)^ do
                         if (CtlType = ODT_MENU) and Assigned(Menu) then
                   ...

                The problem is that the CtlType field of the DRAWITEMSTRUCT
                contains random information when triggered for the
                TStatusBar, which sometimes - this depends on the
                application's memory usage - happens to be the ODT_MENU
                value.

                CtlType doesn't appear to have a specific value for a status
                window. This looks like a Windows bug, but it is actually
                documented in the Win32.HLP (search key: "Owner-Drawn Status
                Windows").
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::WndProc(Messages::TMessage &Message)
{
   if (Message.Msg == WM_DRAWITEM) {
      DRAWITEMSTRUCT *p = (DRAWITEMSTRUCT*)Message.LParam;
      if (p->CtlType == ODT_MENU && Menu && p->hwndItem != Menu->Handle)
      {
         p->CtlType = ODT_STATIC;
      }
   }
   TForm::WndProc(Message);
}
/*TFormMon::~TFormMon*/

/* *************************************************************************
   ************************* SOCKET EVENT HANDLERS *************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        TFormMon::ThreadDone
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::ThreadDone()
{
   sbarInfo->Invalidate();

   // Check to see if application should close.
   if(bPleaseClose)
   {  Close();
   }
   else
   {
		if(bLogPackets)
   	{
			DumpPacketLogs();
	   }

   	if(!bPleaseReconnect)
	   {
   	   FormReconnect->Reset();
	   }
   	else if(bPleaseReconnect && bAutoReconnect)
	   {
   	   FormReconnect->StartTimer();
      	FormReconnect->Left = Left + (Width / 2) - (FormReconnect->Width / 2);
	      FormReconnect->Top = Top + (Height / 2) - (FormReconnect->Height / 2);
   	   FormReconnect->ShowModal();
	   }
   }
}
/*TFormMon::ThreadDone*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::SendMsgForApp

   Purpose:     Sends a message to the host requesting that the specified
                action be taken.  This is used to stop or restart
                applications.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::SendMsgForApp (int appID, int netEvent)
{
   SendData  *myData;
   NET_MSG    msg;

   if (monEvent) {
      msg.id       = netEvent;
      msg.value    = (unsigned long) appID;
      msg.data     = 0;
      msg.dataSize = 0;
      myData = new SendData (&msg, sizeof (NET_MSG));
      monEvent->myQueue.AddLast ((Link*)myData);
   }
}
/*TFormMon::SendMsgForApp*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::SendMsgForAllApps

   Purpose:     Sends a message to the host requesting that the specified
                action be taken on all running Engines.  This is used to
                stop or restart all applications.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::SendMsgForAllApps (int netEvent)
{
   SendMsgForApp (0, netEvent);
}
/*TFormMon::SendMsgForAllApps*/

/* *************************************************************************
   **************************** MENU OPTIONS *******************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        TFormMon::SetPanelMessage
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::SetPanelMessage (char *msg)
{
   sbarInfo->Panels->Items[1]->Text = msg;
   sbarInfo->Invalidate();
}
/*TFormMon::SetPanelMessage*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::ShowStatusDialog
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::ShowStatusDialog (bool bShow, char *msg)
{
   int  x = Left + (ClientWidth / 2);
   int  y = Top + (ClientHeight / 2);

   if (bShow) {
      DlgStatus->Top = y - (DlgStatus->ClientHeight / 2);
      DlgStatus->Left = x - (DlgStatus->ClientWidth / 2);
      DlgStatus->lblStatusMsg->Caption = msg;
      DlgStatus->Show();
   } else
      DlgStatus->Hide();
}
/*TFormMon::ShowStatusDialog*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::ConnectAuto
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::ConnectAuto(char* host)
{
	ClearGrid();

   bWaitForReply = false;
   SetPanelMessage ("Connecting to server...");

   if(monEvent)
   {
      // Tell the thread it should terminate.
      threadColour = clBtnFace;
      sbarInfo->Invalidate();

		// The thread will delete monEvent
      SetEvent(monEvent->hCloseEvent);
   }

	if(host != NULL)
   {
   	monEvent = new MonEvent (host, connectionPort);
   }
	else
   {
   	monEvent = new MonEvent (FormConnect->cboxHost->Items->Strings[0].c_str(), connectionPort);
   }

   ResetEvent(monEvent->hCloseEvent);

   // There is a bug in Borland's NT libraries that prevents the primary
   // thread from deleting the thread class instance.  This work-around lets
   // Borland's libraries delete the thread.  The thread will signal when it
   // finally terminates.
   monEvent->FreeOnTerminate = true;
	if(threadColour == clRed)
   {
	   threadColour = clYellow;
   }

   sbarInfo->Invalidate();
   monEvent->Resume();
	gridEng->Invalidate();

   bPleaseReconnect = true;

	// This functionality has been disabled
//	SendMsgForAllApps(bFatMonitor ? NET_EVT_FATMONITOR_ON : NET_EVT_FATMONITOR_OFF);

   // Remote (not localhost) monitors do not get initial information without this...
   // They used to...
   SendMsgForApp ((int)true, NET_EVT_SEND_UPDATE);
}
/*TFormMon::ConnectAuto*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuConnectionConnectClick

   Purpose:     Opens a dialog window to prompt the user for a host to
                monitor.  After the user selects a host, the network
                connection is initiated and a thread is created to wait for
                network events from the host.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuConnectionConnectClick(TObject *Sender)
{
   FormConnect->ShowModal();
   if (FormConnect->ModalResult == mrOk)
   {
      ConnectAuto();
   }
}
/*TFormMon::mnuConnectionConnectClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuConnectionDisconnectClick

   Purpose:     Closes the connection with the server and terminates the
                thread monitoring socket events.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuConnectionDisconnectClick(TObject *Sender)
{
   if(monEvent)
   {
      // Tell the thread it should terminate.
      threadColour = clBtnFace;
      sbarInfo->Invalidate();
      SetEvent(monEvent->hCloseEvent);
      monEvent = NULL;
   }
//   mnuConnectionDisconnect->Enabled = false;
   bPleaseReconnect = false;
}
/*TFormMon::mnuConnectionDisconnectClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuViewRefreshClick
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuViewRefreshClick(TObject *Sender)
{
	ClearGrid();
   SendMsgForApp ((int)true, NET_EVT_SEND_UPDATE);
   gridEng->Invalidate();
}
/*TFormMon::mnuViewRefreshClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuFileExitClick

   Purpose:     Close the form.  If connected to a host, the form will first
                close the connection to the host.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuFileExitClick(TObject *Sender)
{
   Close();
}
/*TFormMon::mnuFileExitClick*/


void __fastcall TFormMon::mnuFileReloadTALClick(TObject *Sender)
{
	ClearGrid();
	SendMsgForAllApps(NET_EVT_RELOAD_TAL);
}
//---------------------------------------------------------------------------

/* -------------------------------------------------------------------------
   Name:        TFormMon::FormCloseQuery

   Purpose:     Before closing the form, ensure that the connection to the
                host has been terminated and the socket monitor thread has
                been terminated.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   msgHead.Clear();
   if (monEvent) {
      mnuConnectionDisconnect->Click();
      bPleaseClose = true;
      CanClose = false;
   }
}
/*TFormMon::FormCloseQuery*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationSDNormalClick

   Purpose:     Request the host to shutdown the currently selected
                application.  The application should continue processing
                until it is safe to shutdown.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationSDNormalClick(TObject *Sender)
{
   if (clientMsgs[gridEng->Row - 1].appID)
      SendMsgForApp (clientMsgs[gridEng->Row - 1].appID, NET_EVT_SD_NORMAL);
}
/*TFormMon::mnuApplicationSDNormalClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationSDAbortClick

   Purpose:     Request the host to shutdown the currently selected
                application.  The application should terminate immediately.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationSDAbortClick(TObject *Sender)
{
   if (clientMsgs[gridEng->Row - 1].appID)
      SendMsgForApp (clientMsgs[gridEng->Row - 1].appID, NET_EVT_SD_ABORT);
}
/*TFormMon::mnuApplicationSDAbortClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationLoadNormalClick

   Purpose:     Request the host to reload the currently selected
                application.  The application should continue processing
                until it is safe to shutdown.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationLoadNormalClick(TObject *Sender)
{
   if (clientMsgs[gridEng->Row - 1].appID)
      SendMsgForApp (clientMsgs[gridEng->Row - 1].appID, NET_EVT_RELOAD_NORMAL);
}
/*TFormMon::mnuApplicationLoadNormalClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationLoadAbortClick

   Purpose:     Request the host to reload the currently selected
                application.  The application should terminate immediately.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationLoadAbortClick(TObject *Sender)
{
   if (clientMsgs[gridEng->Row - 1].appID)
      SendMsgForApp (clientMsgs[gridEng->Row - 1].appID, NET_EVT_RELOAD_ABORT);
}
/*TFormMon::mnuApplicationLoadAbortClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationSDAllNormalClick

   Purpose:     Request the host to terminate all applications.  The
                applications should continue processing until it is safe to
                shutdown.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationSDAllNormalClick(TObject *Sender)
{
   SendMsgForAllApps (NET_EVT_ALL_SD_NORMAL);
}
/*TFormMon::mnuApplicationSDAllNormalClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationSDAllAbortClick

   Purpose:     Request the host to terminate all applications.  The
                applications should terminate immediately.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationSDAllAbortClick(TObject *Sender)
{
   SendMsgForAllApps (NET_EVT_ALL_SD_ABORT);
}
/*TFormMon::mnuApplicationSDAllAbortClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::EnableSocketMenu
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::EnableSocketMenu (bool bEnable)
{
   mnuApplicationShowMsgs->Enabled     = bEnable;
   mnuApplicationLoadNormal->Enabled   = bEnable;
   mnuApplicationLoadAbort->Enabled    = bEnable;
   mnuApplicationSDNormal->Enabled     = bEnable;
   mnuApplicationSDAbort->Enabled      = bEnable;
   mnuApplicationReloadAllNormal->Enabled  = bEnable;
   mnuApplicationReloadAllAbort->Enabled  = bEnable;
   mnuApplicationSDAllNormal->Enabled  = bEnable;
   mnuApplicationSDAllAbort->Enabled   = bEnable;

   ConfigureMenus();
}
/*TFormMon::EnableSocketMenu*/

/* *************************************************************************
   **************************** GRID FUNCTIONS *****************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        TFormMon::SetupGrid
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::SetupGrid(void)
{
   // Helps prevent flickering by drawing the whole TFrawGrid to a bit map first,
   // and then to the window 
   gridEng->DoubleBuffered = true;

	ClearGrid();

   memset (clientMsgs, 0, sizeof(clientMsgs));
   yLead = (gridEng->DefaultRowHeight - gridEng->Canvas->TextHeight("X")) / 2;
   strcpy(colHead[APP_STATUS].Title, "");
   colHead[APP_STATUS].textWidth = gridEng->Canvas->TextWidth(colHead[APP_STATUS].Title);
   strcpy(colHead[APP_NAME].Title, "App. Name");
   colHead[APP_NAME].textWidth = gridEng->Canvas->TextWidth(colHead[APP_NAME].Title);
   strcpy(colHead[APP_RESTART_COUNT].Title, "Restarts");
   colHead[APP_RESTART_COUNT].textWidth = gridEng->Canvas->TextWidth(colHead[APP_RESTART_COUNT].Title);
   strcpy(colHead[APP_TAP].Title, "App. File (.TAP)");
   colHead[APP_TAP].textWidth = gridEng->Canvas->TextWidth(colHead[APP_TAP].Title);
   strcpy(colHead[APP_TAM].Title, "Flowchart (.TAM)");
   colHead[APP_TAM].textWidth = gridEng->Canvas->TextWidth(colHead[APP_TAM].Title);
   strcpy(colHead[APP_STEP_NUMBER].Title, "Step #");
   colHead[APP_STEP_NUMBER].textWidth = gridEng->Canvas->TextWidth(colHead[APP_STEP_NUMBER].Title);
   strcpy(colHead[APP_STEP_NAME].Title, "Step Name");
   colHead[APP_STEP_NAME].textWidth = gridEng->Canvas->TextWidth(colHead[APP_STEP_NAME].Title);
   strcpy(colHead[APP_DATA].Title, "Data");
   colHead[APP_DATA].textWidth = gridEng->Canvas->TextWidth(colHead[APP_DATA].Title);
   strcpy(colHead[APP_PORT].Title, "Port");
   colHead[APP_PORT].textWidth = gridEng->Canvas->TextWidth(colHead[APP_PORT].Title);
}
/*TFormMon::SetupGrid*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::SetupLights

   Purpose:     Loads in the bitmaps for the lights.
   ------------------------------------------------------------------------- */
/*
void __fastcall TFormMon::SetupLights()
{
   // clLtGray is the background colour used in the bitmap.
   imgListLights->ResourceLoad (rtBitmap, "BMP_GRN", clLtGray);
   imgListLights->ResourceLoad (rtBitmap, "BMP_YLW", clLtGray);
   imgListLights->ResourceLoad (rtBitmap, "BMP_RED", clLtGray);
   imgListLights->ResourceLoad (rtBitmap, "BMP_OFF", clLtGray);
}
*/
/*TFormMon::SetupLights*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::gridEngDrawCell
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::gridEngDrawCell(TObject *Sender, int Col,
      int Row, TRect &Rect, TGridDrawState State)
{
   int    index;
   int    x;
   int    y;

   // Default the background color to the cell.
   if (State.Contains (gdFixed))
      gridEng->Canvas->Brush->Color = clBtnFace;
   else if (gridEng->Row == Row)
      gridEng->Canvas->Brush->Color = clHighlight;
   else
      gridEng->Canvas->Brush->Color = clWindow;
   if (Row > 0 && Col + 1 < gridEng->ColCount)
      ++Rect.Right;  // Remove thin white line between cells.
   gridEng->Canvas->FillRect (Rect);

   x = Rect.Left + 2;
   y = Rect.Top + yLead;
   // Fill the cells in with specific information.
   if(Row == 0)
   {
      if(gridEng->ColWidths[Col] > colHead[Col].textWidth)
      {
         x = Rect.Left + ((gridEng->ColWidths[Col] - colHead[Col].textWidth) / 2);
      }

      gridEng->Canvas->TextRect(Rect, x, y, colHead[Col].Title);
   }
   else if (Row <= MAX_CLIENTS)
   {
      index = Row - 1;

      // Leave slot blank if not running.
      if (clientMsgs[index].status == lsEmpty)
      {
         return;
      }

      switch(Col)
      {
         case APP_STATUS:
         {
            if(Row == index + 1)
            {
               UpdateStatusMessage();
            }

            if(gridEng->ColWidths[APP_STATUS] > imgListLights->Width)
            {
               x = Rect.Left + ((gridEng->ColWidths[APP_STATUS] - imgListLights->Width) / 2);
            }

            imgListLights->Draw (gridEng->Canvas, x, y,
                                 clientMsgs[index].status - 1, true);

				if((clientMsgs[index].status == lsNotRunning) || (clientMsgs[index].status == lsCrashed))
            {
            	int counter;

	            // Are any apps running?
   	         for(counter = 0; (counter < (gridEng->RowCount - 1))
      	       	&& ((clientMsgs[counter].status == lsNotRunning)
         	      || (clientMsgs[counter].status == lsCrashed)); counter++);

	            // counter will pass end only if no apps are running
// Temporarily disabled since it causes system crashes               
//   	       	mnuFileReloadTAL->Enabled = (counter >= (gridEng->RowCount - 1));

               ConfigureMenus();
            }
            else
            {
	            mnuFileReloadTAL->Enabled = false;
            }

            break;
         }

         case APP_NAME:
         {
            gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].appNm);
            break;
         }

         case APP_RESTART_COUNT:
         {
            char restarts[24];

            if(clientMsgs[index].restartMax == -1)
            {
               sprintf(restarts, "%d", clientMsgs[index].restartCnt);
            }
            else
            {
               sprintf(restarts, "%d / %d", clientMsgs[index].restartCnt, clientMsgs[index].restartMax);
            }

            gridEng->Canvas->TextRect(Rect, x, y, restarts);
            break;
         }

         case APP_TAP:
         {
            gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].appFile);
            break;
         }

         case APP_TAM:
         {
            gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].currentTAMName);
            break;
         }

         case APP_STEP_NUMBER:
         {
            // Valid step number
            if(clientMsgs[index].stepNo > 0)
            {
            	gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].stepNo);
            }
            break;
         }

         case APP_STEP_NAME:
         {
            gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].stepName);
            break;
         }

         case APP_DATA:
         {
            gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].stepData);
            break;
         }

         case APP_PORT:
         {
            // Valid line, 1-based
            if(clientMsgs[index].lineNo > 0)
            {
               gridEng->Canvas->TextRect(Rect, x, y, clientMsgs[index].lineNo);
            }
         }
      }
   }
}
/*TFormMon::gridEngDrawCell*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::InvalidateRow

   Purpose:     Invalidates a specific row in the grid to cause the row to
                be re-drawn.  This is faster than re-drawing the entire
                grid.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::InvalidateRow (int iRow)
{
   int   i;
   RECT  rowRect;

   if (iRow > 0 && iRow < gridEng->RowCount) {
      rowRect.left   = gridEng->Left;
      rowRect.top    = 0;
      for (i = 0; i < iRow; ++i)
         rowRect.top += gridEng->RowHeights[i] + gridEng->GridLineWidth;
      rowRect.right  = rowRect.left + gridEng->Width;
      rowRect.bottom = rowRect.top + gridEng->RowHeights[iRow];
      if(!refreshInterval)
      {  InvalidateRect (gridEng->Handle, &rowRect, true);
      }
   }
}
/*TFormMon::InvalidateRow*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::gridEngSelectCell

   Purpose:     Updates the status bar with information about the currently
                selected row.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::gridEngSelectCell(TObject *Sender, int Col,
      int Row, bool &CanSelect)
{
   UpdateStatusMessage();
}
/*TFormMon::gridEngSelectCell*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::sbarInfoDrawPanel
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::sbarInfoDrawPanel(TStatusBar *StatusBar,
      TStatusPanel *Panel, const TRect &Rect)
{
	StatusBar->Canvas->Brush->Color = threadColour;
   StatusBar->Canvas->FillRect(Rect);
}
/*TFormMon::sbarInfoDrawPanel*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::UpdateStatusMessage

   Purpose:     Updates the status bar with information about the currently
                selected row.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::UpdateStatusMessage()
{
   char  *pszText;
   int    index;

   index = gridEng->Row - 1;
   if (gridEng->Row > MAX_CLIENTS || clientMsgs[index].status == lsEmpty)
   {  lsCurrentLine = lsEmpty;
   }
   else
   {  lsCurrentLine = (LINE_STATUS) clientMsgs[index].status;
   }

   if(bConnectedToHost)
   {  switch (lsCurrentLine)
      {  case lsNormal:
            pszText = "Application running normally.";
            break;
         case lsWarning:
            pszText = "Application running normally after restarting.";
            break;
         case lsCrashed:
            pszText = "Application terminated abnormally.";
            break;
         case lsNotRunning:
            pszText = "Application completed normally.";
            break;
         case lsEmpty:
         default:
            pszText = "";
      }
   }
   else
   {  pszText = "Not Connected";
   }

   sbarInfo->Panels->Items[1]->Text = pszText;
}
/*TFormMon::UpdateStatusMessage*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::gridEngMouseDown

   Purpose:     Display a popup menu when the user clicks the right mouse
                button allowing the user to control a specific Engine.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::gridEngMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   // Only display a popup menu if the user clicks the right mouse button.
   if (Button == mbRight) {
      int     iCol;
      int     iRow;
      TPoint  ScreenOrigin;
      TPoint  ClientPoint;
      bool    bEnable;
      int     index;

      // Highlight the row under the mouse by moving to this row.
      gridEng->MouseToCell (X, Y, iCol, iRow);
      if (iRow == 0)
         return;      // Do nothing if they right-click on the column headings.

      gridEng->Row = iRow;

      // Convert the Grid X, Y values to screen X, Y values.
      ClientPoint.x = X;
      ClientPoint.y = Y;
      ScreenOrigin = gridEng->ClientToScreen (ClientPoint);

      index = gridEng->Row - 1;

      // Check if cursor is over an occupied row.
      // Enable menu options only if an Engine is at that location.
      if (clientMsgs[index].status != lsEmpty)
         bEnable = bConnectedToHost;
      else
         bEnable = false;
      mnuGridShowMsgs->Enabled   = bEnable;
      mnuGridLoadNormal->Enabled = bEnable;
      mnuGridLoadAbort->Enabled  = bEnable;
      mnuGridSDNormal->Enabled   = bEnable;
      mnuGridSDAbort->Enabled    = bEnable;

      ConfigureMenus();

      // Display menu (usually to the right of the mouse cursor).
      mnuGrid->Popup (ScreenOrigin.x, ScreenOrigin.y);
   }
}
/*TFormMon::gridEngMouseDown*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationClick
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationClick(TObject *Sender)
{
   bool     bEnable;
   int      index;

   index = gridEng->Row - 1;

   // Check if cursor is over an occupied row.
   // Enable menu options only if an Engine is at that location.
   if (clientMsgs[index].status != lsEmpty)
      bEnable = bConnectedToHost;
   else
      bEnable = false;

   EnableSocketMenu(bEnable);
}
/*TFormMon::mnuApplicationClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuHelpAboutClick
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuHelpAboutClick(TObject *Sender)
{
   AboutBox->ShowModal();
}
/*TFormMon::mnuHelpAboutClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuApplicationShowMsgsClick
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuApplicationShowMsgsClick(TObject *Sender)
{
   MsgLink  *msgLink;
   int       index;
   bool      bFound = false;

   index = gridEng->Row - 1;

   // Check to ensure cursor is not over a blank row.  This will never be true
   // since the menu option is only enabled if the row isn't blank.
   if (clientMsgs[index].status == lsEmpty)
      return;

   // Check to see if an existing window is already open.
   // If so, set focus to that window.
   msgLink = (MsgLink*)msgHead.First();
   while (msgLink && !bFound) {
      if (msgLink->iAppID == clientMsgs[index].appID)
         bFound = true;
      else
         msgLink = (MsgLink*)msgLink->Suc();
   }
   if (msgLink) {
		msgLink->myFormMsgs->mmoMsgs->Visible = false;
      msgLink->myFormMsgs->Show();
   }
   else
   {
   	// If we make it this far, create a message window for this application.
	   msgLink = new MsgLink (Application, clientMsgs[index].appID,
                          clientMsgs[index].appNm);
   	msgHead.AddLast ((Link*)msgLink);
   }
   SendMsgForApp (clientMsgs[index].appID, NET_EVT_SEND_OUTPUT);
}
/*TFormMon::mnuApplicationShowMsgsClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::MsgWindowClosed
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::MsgWindowClosed (int appID)
{
   MsgLink  *msgLink;
   bool      bFound = false;

   msgLink = (MsgLink*)msgHead.First();
   while (msgLink && !bFound) {
      if (msgLink->iAppID == appID)
         bFound = true;
      else
         msgLink = (MsgLink*)msgLink->Suc();
   }
   if (msgLink)
      delete msgLink;
}
/*TFormMon::MsgWindowClosed*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::mnuGridShowMsgsClick
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::mnuGridShowMsgsClick(TObject *Sender)
{
   mnuApplicationShowMsgsClick(Sender);
}
/*TFormMon::mnuGridShowMsgsClick*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::MsgDisplayLines
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::MsgDisplayLines (int appID, char *lines)
{
   char     *line;
   char     *temp;
   MsgLink  *msgLink;
   bool      bFound = false;

   msgLink = (MsgLink*)msgHead.First();
   while (msgLink && !bFound) {
      if (msgLink->iAppID == appID)
         bFound = true;
      else
         msgLink = (MsgLink*)msgLink->Suc();
   }
   if (msgLink) {
      msgLink->myFormMsgs->mmoMsgs->Clear();
      line = lines;
      while (line) {
         temp = strchr (line, '\n');
         if (temp)
            *temp = 0;
         if (temp || (!temp && *line))
            msgLink->myFormMsgs->mmoMsgs->Lines->Add (line);
         if (temp) {
            *temp = '\n';
            line = temp + 1;
         } else
            line = 0;
      }
   }

	msgLink->myFormMsgs->mmoMsgs->Visible = true;
}
/*TFormMon::MsgDisplayLines*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::DisplayError
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::DisplayError (char *errorMsg)
{
   Application->MessageBox (errorMsg, "Error", MB_OK);
}
/*TFormMon::DisplayError*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::ReadRegistry

   Purpose:     Reads in the registry values into application memory.

   Notes:       - When a new field is added to the form, it won't exist in
                  the registry.  Don't stop loading registry settings if one
                  value is missing.
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::ReadRegistry(void)
{  LONG  ret;
   HKEY  hkey;
   DWORD dwLen;
   DWORD dwValue;
   DWORD dwType;
   char  szKey[30];  // Capable of holding "Host %d" for up to 64-bit int counter
   char *pszValue;

   try
   {
   	ret = RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFMONITOR_REG_KEY, 0, KEY_READ, &hkey);
      if(ret != ERROR_SUCCESS)
      {
      	return;
      }

      // Get length of string
      ret = RegQueryValueEx (hkey, "AutoConnect", NULL, &dwType, NULL, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	// Create copy of string
         pszValue = new char[dwLen + 1];
         ret = RegQueryValueEx (hkey, "AutoConnect", NULL, &dwType, pszValue, &dwLen);

         // Set AutoConnect boolean
         if (ret == ERROR_SUCCESS)
         {
         	bAutoConnect = !strcmp(pszValue, "True");
         }

         delete[] pszValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {  bAutoConnect = false;
      }

      // Get length of string
      ret = RegQueryValueEx (hkey, "Host 0", NULL, &dwType, NULL, &dwLen);
      if(ret == ERROR_SUCCESS)
      {  // Create copy of string
         autoConnectHost = new char[dwLen + 1];
         RegQueryValueEx (hkey, "Host 0", NULL, &dwType, autoConnectHost, &dwLen);
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {  autoConnectHost = new char[1];
         *autoConnectHost = '\0';
      }

      // Get length of string
      ret = RegQueryValueEx(hkey, "AutoReconnect", NULL, &dwType, NULL, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	// Create copy of string
         pszValue = new char[dwLen + 1];
         ret = RegQueryValueEx(hkey, "AutoReconnect", NULL, &dwType, pszValue, &dwLen);

         if(ret == ERROR_SUCCESS)
         {
         	bAutoReconnect = !strcmp(pszValue, "True");
         }

         delete[] pszValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {
      	bAutoReconnect = true;
      }

      ret = RegQueryValueEx(hkey, "Port", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	connectionPort = dwValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {  connectionPort = DEFAULT_IVR_PORT;
      }

      ret = RegQueryValueEx(hkey, "Interval", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	refreshInterval = dwValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {
        refreshInterval = 0;
      }
      // Note that for a value of 0 the timer will not execute an OnTimer event
      TimerRefresh->Interval = refreshInterval;

		/* This functionality has been disabled
      // Get length of string
      ret = RegQueryValueEx(hkey, "FatMonitor", NULL, &dwType, NULL, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	// Create copy of string
         pszValue = new char[dwLen + 1];
         ret = RegQueryValueEx(hkey, "FatMonitor", NULL, &dwType, pszValue, &dwLen);

         if(ret == ERROR_SUCCESS)
         {
         	bFatMonitor = !strcmp(pszValue, "True");
         }

         delete pszValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {
      	bFatMonitor = false;
      }
		*/

      // Get length of string
      ret = RegQueryValueEx(hkey, "LogPackets", NULL, &dwType, NULL, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	// Create copy of string
         pszValue = new char[dwLen + 1];
         ret = RegQueryValueEx(hkey, "LogPackets", NULL, &dwType, pszValue, &dwLen);

         if(ret == ERROR_SUCCESS)
         {
// Functionality disabled
//         	bLogPackets = !strcmp(pszValue, "True");
            bLogPackets = false;
         }

         delete[] pszValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {
      	bLogPackets = false;
      }

      // Get length of string
      ret = RegQueryValueEx (hkey, "ReadOnly", NULL, &dwType, NULL, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	// Create copy of string
         pszValue = new char[dwLen + 1];
         ret = RegQueryValueEx (hkey, "ReadOnly", NULL, &dwType, pszValue, &dwLen);

         // Set AutoConnect boolean
         if (ret == ERROR_SUCCESS)
         {
         	bReadOnly = !strcmp(pszValue, "True");
         }

         delete[] pszValue;
      }
      // Default value if registry key doesn't exists or can't be read
      else
      {
         bReadOnly = false;
      }

      ret = RegQueryValueEx(hkey, "Top", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Top = dwValue;
      }

      ret = RegQueryValueEx(hkey, "Left", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Left = dwValue;
      }

      ret = RegQueryValueEx(hkey, "Height", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Height = dwValue;
      }

      ret = RegQueryValueEx(hkey, "Width", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
      if(ret == ERROR_SUCCESS)
      {
      	Width = dwValue;
      }

      for(int col = 1; col < gridEng->ColCount; col++)
      {
      	sprintf(szKey, "ColWidth %d", col);

         ret = RegQueryValueEx(hkey, szKey, NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	gridEng->ColWidths[col] = dwValue;
         }
      }

     RegCloseKey(hkey);
   }

   catch(Exception &e)
   {  Application->MessageBox(
         e.Message.c_str(),
         "ReadRegistry",
         MB_OK | MB_ICONERROR);
   }
}
/*TFormMon::ReadRegistry*/

/* -------------------------------------------------------------------------
   Name:        TFormMon::WriteRegistry
   ------------------------------------------------------------------------- */
void __fastcall TFormMon::WriteRegistry(void)
{  LONG  ret;
   HKEY  hkey;
   DWORD dwDisposition;
   DWORD dwValue;
   DWORD dwLen;
   char  szKey[30];  // Capable of holding "Host %d" for up to 64-bit int counter
   char szValue[255];

   try
   {  ret = RegCreateKeyEx(
         HKEY_LOCAL_MACHINE,      // Handle of an open key.
         TFMONITOR_REG_KEY,              // Address of subkey name.
         0,                       // Reserved.
         0,                       // Address of class string.
         REG_OPTION_NON_VOLATILE, // Special options flag.
         KEY_WRITE,               // Desired security access.
         0,                       // Address of key security structure.
         &hkey,                   // Address of buffer for opened handle.
         &dwDisposition);         // Address of disposition value buffer.

      if(ret != ERROR_SUCCESS)
      {  Application->MessageBox(
            "Failed to create registry key.",
            "Registry Save Failed",
            MB_ICONERROR | MB_OK);
      }
      else
      {
      	ret = RegSetValueEx(hkey, "AutoConnect", 0, REG_SZ, bAutoConnect ? "True" : "False", bAutoConnect ? 5 : 6);

         ret &= RegSetValueEx(hkey, "AutoReconnect", 0, REG_SZ, bAutoReconnect ? "True" : "False", bAutoReconnect ? 5 : 6);

         ret &= RegSetValueEx(hkey, "Port", 0, REG_DWORD, (LPBYTE) &connectionPort, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Interval", 0, REG_DWORD, (LPBYTE) &refreshInterval, sizeof(DWORD));

			// This functionality has been disabled
//       ret &= RegSetValueEx(hkey, "FatMonitor", 0, REG_SZ, bFatMonitor ? "True" : "False", bFatMonitor ? 5 : 6);

         ret &= RegSetValueEx(hkey, "LogPackets", 0, REG_SZ, bLogPackets ? "True" : "False", bLogPackets ? 5 : 6);

         ret &= RegSetValueEx(hkey, "ReadOnly", 0, REG_SZ, bReadOnly ? "True" : "False", bReadOnly ? 5 : 6);

         ret &= RegSetValueEx(hkey, "Top", 0, REG_DWORD, (LPBYTE) &Top, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Left", 0, REG_DWORD, (LPBYTE) &Left, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Height", 0, REG_DWORD, (LPBYTE) &Height, sizeof(DWORD));

         ret &= RegSetValueEx(hkey, "Width", 0, REG_DWORD, (LPBYTE) &Width, sizeof(DWORD));

         for(int col = 1; col < gridEng->ColCount; col++)
         {
         	sprintf(szKey, "ColWidth %d", col);
         	dwValue = gridEng->ColWidths[col];
            ret &= RegSetValueEx(hkey, szKey, 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));
         }

         if(ret != ERROR_SUCCESS)
         {  Application->MessageBox(
               "Failed to write host entry.",
               "Registry Save Failed",
               MB_ICONERROR | MB_OK);
         }
      }

      RegCloseKey(hkey);
   }

   catch(Exception &e)
   {  Application->MessageBox(
         e.Message.c_str(),
         "ReadRegistry",
         MB_OK | MB_ICONERROR);
   }
}
/*TFormMon::WriteRegistry*/

void __fastcall TFormMon::mnuFileOptionsClick(TObject *Sender)
{
	FormOptions->ShowModal();

	/* This functionality has been disabled
   if(FormOptions->ModalResult == mrOk)
   {
		SendMsgForAllApps(bFatMonitor ? NET_EVT_FATMONITOR_ON : NET_EVT_FATMONITOR_OFF);
   }
   */
}
//---------------------------------------------------------------------------
void __fastcall TFormMon::mnuApplicationReloadAllNormalClick(
      TObject *Sender)
{
	SendMsgForAllApps(NET_EVT_ALL_RELOAD_NORMAL);
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::mnuApplicationReloadAllAbortClick(
      TObject *Sender)
{
	SendMsgForAllApps(NET_EVT_ALL_RELOAD_ABORT);
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::ResizeGrid(unsigned int rows)
{
	// appNumber grows to match apps but does not shrink
   appNumber = max(rows, appNumber);

   // Always at least 2 rows
   gridEng->RowCount = max(appNumber + 1, (unsigned int) 2);
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::TimerRefreshTimer(TObject *Sender)
{  // Redraw the whole grid
   gridEng->Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::ClearGrid(void)
{
  // Clear all the rows except the column title row
   for(int counter = 0; counter < gridEng->RowCount - 1; counter++)
   {
   	clientMsgs[counter].status = lsEmpty;
   }

   // Reduce the grid to a column title row and a one blank row
   gridEng->RowCount = 2;

   appNumber = 0;

   gridEng->Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::FormResize(TObject *Sender)
{
	gridEng->Invalidate();
}
//---------------------------------------------------------------------------

bool __fastcall TFormMon::DumpPacketLogs(void)
{
	FILE* file;
	char* fileName;
   AnsiString text;

   for(unsigned int lineIndex = 0; lineIndex < appNumber; lineIndex++)
   {
		if(clientMsgs[lineIndex].appNm && *(clientMsgs[lineIndex].appNm))
      {
			fileName = new char[strlen(clientMsgs[lineIndex].appNm) + 21];
   	   strcpy(fileName, clientMsgs[lineIndex].appNm);
	      strcat(fileName, ("_" + FormatDateTime("yyyymmdd", Date()) + "_"
         	+ FormatDateTime("hhnnss", Time()) + ".mon").c_str());

		   if((file = fopen(fileName, "w")) == NULL)
   		{
				delete fileName;
   	   	return(false);
		   }

			delete fileName;

	      // Write title line of file
			fprintf(file, clientMsgs[lineIndex].appNm);
      	fprintf(file, " running ");
	      fprintf(file, clientMsgs[lineIndex].appFile);
      	fprintf(file, (" stopped on " + FormatDateTime("yyyy mm dd", Date())
         	+ " at " + FormatDateTime("hh:nn:ss", Time()) + "\n\n").c_str());

		   int logStart = appLogs[lineIndex].logWrapped ? appLogs[lineIndex].index : 0;
   	   int logEnd = appLogs[lineIndex].index;

			// Write the first line
	      fprintf(file, "Procedure: ");
   	   fprintf(file, appLogs[lineIndex].log[logStart].currentTAMName);
      	fprintf(file, "   Step#: ");
	      text = appLogs[lineIndex].log[logStart].stepNo;
   	   fprintf(file, text.c_str());
      	fprintf(file, "   Step Name: ");
	      fprintf(file, appLogs[lineIndex].log[logStart].stepName);
   	   fprintf(file, "   Line#: ");
      	// Print "none" for lines less than 1
	      if(appLogs[lineIndex].log[logStart].lineNo < 1)
   	   {
      		text = "none";
	      }
   	   else
      	{
	      	text = (appLogs[lineIndex].log[logStart].lineNo);
   	   }
			fprintf(file, text.c_str());
	      fprintf(file, "\n");

   	   for(int logIndex = (logStart + 1) % INFOLOG_MAX; logIndex != logEnd; (++logIndex) %= INFOLOG_MAX)
      	{
		      fprintf(file, "Procedure: ");
   		   fprintf(file, appLogs[lineIndex].log[logIndex].currentTAMName);
      		fprintf(file, "   Step#: ");
         	text = appLogs[lineIndex].log[logIndex].stepNo;
		      fprintf(file, text.c_str());
   		   fprintf(file, "   Step Name: ");
      		fprintf(file, appLogs[lineIndex].log[logIndex].stepName);
	      	fprintf(file, "   Line#: ");
	         // Print "none" for lines with -1
		      if(appLogs[lineIndex].log[logIndex].lineNo < 1)
   		   {
      			text = "none";
		      }
   		   else
      		{
	      		text = (appLogs[lineIndex].log[logIndex].lineNo);
	   	   }
		      fprintf(file, text.c_str());
      		fprintf(file, "\n");
	      }

		   fclose(file);

         // Reset the log
			for(int counter = 0; counter < INFOLOG_MAX; counter++)
   	   {
				appLogs[lineIndex].log[counter].currentTAMName[0] = '\0';
				appLogs[lineIndex].log[counter].stepNo = 0;
				appLogs[lineIndex].log[counter].stepName[0] = '\0';
				appLogs[lineIndex].log[counter].lineNo = 0;
      	}
	      appLogs[lineIndex].index = 0;
   	   appLogs[lineIndex].logWrapped = false;
	   }
   }

   return(true);
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::SetConnectedToHost(bool connected)
{
	if(!connected)
   {
		monEvent = NULL;
   }

	mnuConnectionDisconnect->Enabled = bConnectedToHost = connected;
   FormMon->SetPanelMessage(connected ? "Connected" : "Not Connected");
}
//---------------------------------------------------------------------------

void __fastcall TFormMon::ConfigureMenus(void)
{
   if(bReadOnly)
   {
      mnuApplicationLoadNormal->Enabled = false;
      mnuApplicationLoadAbort->Enabled = false;
      mnuApplicationSDNormal->Enabled = false;
      mnuApplicationSDAbort->Enabled = false;
      mnuApplicationReloadAllNormal->Enabled = false;
      mnuApplicationReloadAllAbort->Enabled = false;
      mnuApplicationSDAllNormal->Enabled = false;
      mnuApplicationSDAllAbort->Enabled = false;

   	mnuFileReloadTAL->Enabled = false;

      mnuGridLoadNormal->Enabled = false;
      mnuGridLoadAbort->Enabled  = false;
      mnuGridSDNormal->Enabled   = false;
      mnuGridSDAbort->Enabled    = false;
   }
}
void __fastcall TFormMon::t1Click(TObject *Sender)
{
   // function in ESCUTIL: help_fns.cpp, opens help file and links to topic requested
   HelpLink("ServerConfig_Monitor.htm");
}
//---------------------------------------------------------------------------

