#include "waittimer.h"

/* -------------------------------------------------------------------------
   Name:        WaitTimer::WaitTimer
   ------------------------------------------------------------------------- */
WaitTimer::WaitTimer (bool bManualReset)
{
   m_bManualReset = bManualReset;
   m_hWaitTimer = CreateWaitableTimer (0, m_bManualReset, 0);
   m_waitTimeout.QuadPart = (LONGLONG) 0;
}
/*WaitTimer::WaitTimer*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::~WaitTimer
   ------------------------------------------------------------------------- */
WaitTimer::~WaitTimer()
{
   CloseHandle (m_hWaitTimer); 
}
/*WaitTimer::~WaitTimer*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::SetTimeoutInSeconds

   Notes:       - Waitable timers use time measured in 100 nanosecond
                  intervals (10^-7 = .0000001).
                - To convert to seconds times by 10,000,000
                  (1 = 10^0 = 10^-7 x 10^7).
   ------------------------------------------------------------------------- */
void WaitTimer::SetTimeoutInSeconds (int iSeconds)
{
   m_waitTimeout.QuadPart = -(LONGLONG) iSeconds * 10000000;
}
/*WaitTimer::SetTimeoutInSeconds*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::SetTimeoutInMilliseconds

   Notes:       - Waitable timers use time measured in 100 nanosecond
                  intervals (10^-7 = .0000001).
                - To convert to milliseconds times by 10,000
                  (.001 = 10^-3 = 10^-7 x 10^4).
   ------------------------------------------------------------------------- */
void WaitTimer::SetTimeoutInMilliseconds (int iMilliseconds)
{
   m_waitTimeout.QuadPart = -(LONGLONG) iMilliseconds * 10000;
}
/*WaitTimer::SetTimeoutInMilliseconds*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::Start
   ------------------------------------------------------------------------- */
void WaitTimer::Start()
{
   SetWaitableTimer (m_hWaitTimer, &m_waitTimeout, 0, 0, 0, false);
}
/*WaitTimer::Start*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::StartSec
   ------------------------------------------------------------------------- */
void WaitTimer::StartSec (int iSeconds)
{
   SetTimeoutInSeconds (iSeconds);
   Start();
}
/*WaitTimer::StartSec*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::StartMS
   ------------------------------------------------------------------------- */
void WaitTimer::StartMS (int iMilliseconds)
{
   SetTimeoutInMilliseconds (iMilliseconds);
   Start();
}
/*WaitTimer::StartMS*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::Cancel
   ------------------------------------------------------------------------- */
void WaitTimer::Cancel()
{
   if (m_bManualReset) {
      // The only way to reset a waitable timer used for notification is to
      // use "SetWaitableTimer" and then cancel it before it expires.
      SetTimeoutInSeconds (5);
      Start();
      CancelWaitableTimer (m_hWaitTimer);
   }
   else {
      CancelWaitableTimer (m_hWaitTimer);
      WaitForSingleObject (m_hWaitTimer, 0);
   }
}
/*WaitTimer::Cancel*/

/* -------------------------------------------------------------------------
   Name:        WaitTimer::GetHandle
   ------------------------------------------------------------------------- */
HANDLE WaitTimer::GetHandle()
{
   return m_hWaitTimer; 
}
/*WaitTimer::GetHandle*/
